/*
** Module   : Cache.java
** Abstract : Simple cache interface with has a map-like API
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 ECF 20140922 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.cache;

/**
 * A simple interface for a cache, with a map-like API.
 * 
 * @param   K
 *          Type of cache element key.
 * @param   V
 *          Type of cache element value.
 */
public interface Cache<K, V>
{
   /**
    * Retrieve a value, given its key.
    * 
    * @param   key
    *          A hashable key.
    * 
    * @return  Value associated with the given key, or <code>null</code> if there is no mapping
    *          for the given key.
    */
   public V get(K key);
   
   /**
    * Add an entry to the cache, identified by the given key.
    * 
    * @param   key
    *          A hashable key to associate with the value.
    * @param   value
    *          Value to store.
    * 
    * @return  The old value stored with the given key, or <code>null</code> if there was no
    *          mapping for the given key.
    */
   public Object put(K key, V value);
   
   /**
    * Remove the entry associated with the given key from the cache.
    * 
    * @param   key
    *          A hashable key identifying the entry to remove.
    * 
    * @return  The value removed, or <code>null</code> if there was no mapping for the given key.
    */
   public Object remove(K key);
}