/*
** Module   : CompiledConstants.java
** Abstract : Constants shared across expression compiler classes
**
** Copyright (c) 2001-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- -----------------------------Description-----------------------------
** 001 ECF 20041130   @19649 Production version from JTA, reworked into a new package and with 
**                           minor cleanups in javadoc.
** 002 ECF 20050201   @19650 Added several opcodes to support expression compiler overhaul.
** 003 ECF 20050207   @19651 Inserted TYPE_VAR_EXPANDED result type to support code which prevents
**                           the superfluous addition of branching instructions to symbol 
**                           resolution code units.
** 004 ECF 20050526   @21309 Modifications to support the expression engine rewrite. All integral
**                           TYPE_* constants were obsoleted. Numerous instruction opcode 
**                           constants were added. Some additional constants for class and method 
**                           names and descriptors were added as well.
** 005 CA  20140630          Always use ldc_w instead of ldc, otherwise the constant pool index 
**                           is limited to a max of 255.
**/

package com.goldencode.expr;

/**
 * Constants used by the expression compilation support classes. This
 * includes class and method names and descriptors and instruction opcode
 * aliases used by the bytecode assembler.
 */
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
interface CompilerConstants
{
   /** Fully qualified name of CompiledExpression class. */
   static final String CLASS_COMPILEDEXPRESSION =
      Compiler.PKG_PREFIX + "CompiledExpression";
   
   /** Name of initialization (constructor) method */
   static final String METHOD_INIT = "<init>";
   
   /** Name of execute() method */
   static final String METHNAME_EXECUTE = "execute";
   
   /** Method descriptor for execute() method */
   static final String METHDESC_EXECUTE = "()Ljava/lang/Object;";
   
   // Bytecode aliases.
   static final int _aconst_null       = 0x01;
   static final int _iconst_0          = 0x03;
   static final int _iconst_1          = 0x04;
   static final int _iconst_2          = 0x05;
   static final int _iconst_3          = 0x06;
   static final int _iconst_4          = 0x07;
   static final int _iconst_5          = 0x08;
   static final int _lconst_0          = 0x09;
   static final int _lconst_1          = 0x0A;
   static final int _dconst_0          = 0x0E;
   static final int _dconst_1          = 0x0F;
   static final int _ldc_w             = 0x13;
   static final int _ldc2_w            = 0x14;
   static final int _aload_0           = 0x2A;
   static final int _aload_1           = 0x2B;
   static final int _aload_2           = 0x2C;
   static final int _aload_3           = 0x2D;
   static final int _astore_1          = 0x4C;
   static final int _astore_2          = 0x4D;
   static final int _astore_3          = 0x4E;
   static final int _aastore           = 0x53;
   static final int _pop               = 0x57;
   static final int _pop2              = 0x58;
   static final int _dup               = 0x59;
   static final int _swap              = 0x5F;
   static final int _iadd              = 0x60;
   static final int _ladd              = 0x61;
   static final int _dadd              = 0x63;
   static final int _isub              = 0x64;
   static final int _lsub              = 0x65;
   static final int _dsub              = 0x67;
   static final int _imul              = 0x68;
   static final int _lmul              = 0x69;
   static final int _dmul              = 0x6B;
   static final int _idiv              = 0x6C;
   static final int _ldiv              = 0x6D;
   static final int _ddiv              = 0x6F;
   static final int _irem              = 0x70;
   static final int _lrem              = 0x71;
   static final int _drem              = 0x73;
   static final int _ineg              = 0x74;
   static final int _lneg              = 0x75;
   static final int _dneg              = 0x77;
   static final int _ishl              = 0x78;
   static final int _lshl              = 0x79;
   static final int _ishr              = 0x7A;
   static final int _lshr              = 0x7B;
   static final int _iushr             = 0x7C;
   static final int _lushr             = 0x7D;
   static final int _iand              = 0x7E;
   static final int _land              = 0x7F;
   static final int _ior               = 0x80;
   static final int _lor               = 0x81;
   static final int _ixor              = 0x82;
   static final int _lxor              = 0x83;
   static final int _i2l               = 0x85;
   static final int _i2d               = 0x87;
   static final int _l2i               = 0x88;
   static final int _l2d               = 0x8A;
   static final int _d2i               = 0x8E;
   static final int _d2l               = 0x8F;
   static final int _lcmp              = 0x94;
   static final int _dcmpg             = 0x98;
   static final int _ifeq              = 0x99;
   static final int _ifne              = 0x9A;
   static final int _iflt              = 0x9B;
   static final int _ifge              = 0x9C;
   static final int _ifgt              = 0x9D;
   static final int _ifle              = 0x9E;
   static final int _if_icmpeq         = 0x9F;
   static final int _if_icmpne         = 0xA0;
   static final int _if_icmplt         = 0xA1;
   static final int _if_icmpge         = 0xA2;
   static final int _if_icmpgt         = 0xA3;
   static final int _if_icmple         = 0xA4;
   static final int _goto              = 0xA7;
   static final int _ireturn           = 0xAC;
   static final int _dreturn           = 0xAF;
   static final int _areturn           = 0xB0;
   static final int _return            = 0xB1;
   static final int _getfield          = 0xB4;
   static final int _invokevirtual     = 0xB6;
   static final int _invokespecial     = 0xB7;
   static final int _invokestatic      = 0xB8;
   static final int _invokeinterface   = 0xB9;
   static final int _new               = 0xBB;
   static final int _anewarray         = 0xBD;
   static final int _checkcast         = 0xC0;
   static final int _ifnull            = 0xC6;
   static final int _ifnonnull         = 0xC7;
}