/*
** Module   : NameAndTypeConstant.java
** Abstract : 
**
** Copyright (c) 2001-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 ECF 20041130 Production version from JTA, reworked into a new package and with minor
**                  cleanups in javadoc.
** 002 ECF 20150715 Replace StringBuffer with StringBuilder.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.expr;

import java.io.*;

public final class NameAndTypeConstant
extends Constant
{
   private Utf8Constant nameConstant;
   
   private Utf8Constant descriptorConstant;
   
   static NameAndTypeConstant create(ConstantPool pool, 
                                     String name,
                                     String descriptor)
   {
      return (NameAndTypeConstant) pool.get(
         new NameAndTypeConstant(pool, name, descriptor));
   }
   
   private NameAndTypeConstant(ConstantPool pool, 
                               String name, 
                               String descriptor)
   {
      super(NAMEANDTYPE);
      
      nameConstant = Utf8Constant.create(pool, name);
      descriptorConstant = Utf8Constant.create(pool, descriptor);
   }
   
   public int hashCode()
   {
      return (int) (hashBase +
                    nameConstant.hashCode() -
                    descriptorConstant.hashCode());
   }
   
   public boolean equals(Object o)
   {
      if (!(o instanceof NameAndTypeConstant))
      {
         return false;
      }
      
      NameAndTypeConstant n = (NameAndTypeConstant) o;
      return (nameConstant.equals(n.nameConstant) &&
              descriptorConstant.equals(n.descriptorConstant));
   }
   
   public int compareTo(Object o)
   {
      int comp = super.compareTo(o);
      
      if (comp == 0)
      {
         NameAndTypeConstant n = (NameAndTypeConstant) o;
         comp = nameConstant.compareTo(n.nameConstant);
      }
      
      if (comp == 0)
      {
         NameAndTypeConstant n = (NameAndTypeConstant) o;
         comp = descriptorConstant.compareTo(n.descriptorConstant);
      }
         
      return comp;
   }
   
   public String toString()
   {
      StringBuilder buf = new StringBuilder();
      buf.append(nameConstant)
         .append(descriptorConstant);
         
      return buf.toString();
   }
   
   protected void writeContent(DataOutput out)
   throws IOException
   {
      out.writeShort(nameConstant.getCpIndex());
      out.writeShort(descriptorConstant.getCpIndex());
   }
}