/*
** Module   : AdminDef.java
** Abstract : container for the admin related definitions
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20090602   @42517 Created initial version.
** 002 CA  20090615   @42690 Added a map with the description for each 
**                           resource name.
** 003 NVS 20091005   @44118 Added the admin profile data.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin;

import java.io.Serializable;
import java.util.*;

import com.goldencode.p2j.security.Description;

/**
 * Defines class names the admin client may download and use as 
 * pluggable custom extensions.
 * Additional menu choices are available through this mechanism as well
 * as account editors and resource access control lists editors. 
 */
public class AdminDef
implements Serializable
{
   /**
    * Name of the custom admin client extension class.
    * This class implements AdminCustomMenu interface that provides extra
    * custom menu items.
    * This same eclass implements ClientAccountExtension interface that
    * provides account extensions.
    */
   public String customClientExt;

   /**
    * Maps all known abstract resource types to their corresponding
    * ACL editor class names.
    * This map includes all resource types known to the server, not just
    * those currently configured.
    */
   public Map<String, String> customAclEditors;
   
   /**
    * Maps all known abstract resource names to their corresponding
    * description.
    */
   public Map<String, Description[]> resourceDescription;
   
   /**
    * Admin profile. A set of permissions for the major admin resources.
    */
   public Profile[] profile;
   
   /**
    * Describes the admin access permission with regards to the named
    * admin resource instance
    */
   public static class Profile
   implements Serializable
   {
      /** admin resource instance name */
      public final String name;
      
      /** access permission */
      public final boolean permission;
      
      /**
       * Constructor
       *
       * @param name
       *        admin resource instance this profile is for
       *
       * @param permission
       *        permission for this resource instance. <code>true</code> means
       *        this resource instance is accessible 
       */
      public Profile(String name, boolean permission)
      {
         this.name = name;
         this.permission = permission;
      }
   }
}