/*
** Module   : UserDef.java
** Abstract : Container for the user account definition
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20090506   @42138 Created the initial version.
** 002 NVS 20090603   @42585 Added enabled flag.
** 003 NVS 20090604   @42586 Added protect flag.
** 004 SVL 20100916          Added authPlugin parameter.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin;

import com.goldencode.p2j.directory.DateValue;
import com.goldencode.p2j.directory.TimeValue;

import java.io.Serializable;

/**
 * Provides storage for all the pieces of information that make a user
 * account definition. All pieces are accessible directly.
 */
public class UserDef
implements Serializable
{
   /** user account name */
   public String subjectId;
   
   /** state of the account */
   public boolean enabled;
   
   /** account owner */
   public String person;
   
   /** Truststore alias of the associated certificate entry; may be null */
   public String alias;
   
   /** Group accounts where this user belongs; may be empty */
   public String[] groups;
   
   /** 
    * Password protected account flag.
    * <p>
    * If set to <code>true</code>, password protection is in effect for this
    * account. The password field is expected to be present. If no password
    * is given to the constructor, all binary zeros are used as the password
    * hash, making this account effectively disabled.
    * <p>
    * If set to <code>false</code>, this account is not protected. The contents
    * of the password hash field does not matter and no check will be done.
    */
   public boolean protect;
   
   /** Hashed password for this account; may be null if not protected. */
   public byte[] password;

   /** Date password was last changed; may be null */
   public DateValue date;

   /** Time password was last changed; may be null */
   public TimeValue time;

   /** Authorization mode override for this account. */
   public int mode;

   /** Custom authentication plugin class. */
   public String authPlugin;
}
