/*
** Module   : AdminCertificates.java
** Abstract : Certificates management client
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20090512  @42163  Created initial version.
** 002 CA  20090519  @42328  Change alias - save the new certificate alias.
**                           Add certificate - after saving, clear PEM data.
** 003 NVS 20090723  @43350  Tables don't allow moving columns.
** 004 NVS 20090729  @43426  Implemented certificate table printing.
** 005 NVS 20090803  @43479  Using a better approach to printing with extended
**                           detail choice.
** 006 NVS 20091116  @44345  Eliminating confusing empty Admin Client messages.
** 007 SIY 20091116  @44339  Refactoring of things related to TaggedName. 
**                           Minor cleanups.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.PrinterException;
import java.text.MessageFormat;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import com.goldencode.p2j.admin.*;

/**
 * This is a Certificate management class for the admin session. 
 * 
 * @author NVS
 */
public class AdminCertificates
extends AdminScreen
implements PrintRequest
{
   /** Screen switcher - user accounts screen ID */
   public static final String SCREEN_ACC_CERTS = "adm_acc_certs";
   
   /** Screen title */
   public static final String TITLE_ACC_CERTS = "Certificates Management";

   // west panel widgets, all named wp*

   /** west panel - works with the list of certificates */
   private JPanel west = null;
   
   /** scroller for list of certificates */
   private JScrollPane wpScroller = null;
   
   /** table to display certificates */
   private JTable wpCerts = null;

   /** related table model */
   private TaggedDataTableModel wpCtm = null;

   /** related table selection model */
   private ListSelectionModel wpCsm = null;

   /** sub-panel that holds all controls for list of certificates */
   private JPanel wpCtlPanel = null;

   /** sub-panel that holds radio buttons for certificate type switches */
   private JPanel wpTypePanel = null;
   
   /** Populate button */
   private JButton wpbR = null;

   /** Add Certificate button */
   private JButton wpbA = null;
   
   /** Change Alias button */
   private JButton wpbE = null;

   /** Delete Selected button */
   private JButton wpbD = null;

   /** View Details button */
   private JButton wpbV = null;

   /** Only Show Unassigned check-box */
   private JCheckBox wpcU = null;

   /** Show CA Certificates radio */
   private JRadioButton wprCA = null;
      
   /** Show Peer Certificates radio */
   private JRadioButton wprPeer = null;
   
   // east panel widgets, all named ep*
   
   /** east panel - works with the certificate detail panel */
   private JPanel east = null;

   /** title border for list of details */
   private TitledBorder epBorder = null;
   
   /** scroller for list of details */
   private JScrollPane epScroller = null;
   
   /** table for certificate details*/
   private JTable epDetail = null;

   /** related table model */
   private TaggedDataTableModel epCtm = null;

   /** related table selection model */
   private ListSelectionModel epCsm = null;

   // dialog for certificate specific widgets, all named wpd*
   
   /** modal dialog panel for Certificate Definition function */
   private JPanel wpdpCertPanel = null;

   /** certificate name label */
   private JLabel wpdlN = null;
   
   /** certificate name field */
   private JTextField wpdfN = null;

   /** certificate PEM data label */
   private JLabel wpdlP = null;
   
   /** certificate PEM data text area*/
   private JTextArea wpdfP = null;
   
   /** certificate PEM data scroller */
   private JScrollPane wpdfPScroller = null;
   
   /** Cancel button */
   private JButton wpdbC = null;
   
   /** Save button */
   private JButton wpdbS = null;

   // dialog specific widgets to change Alias
   
   /** modal dialog panel for Change Certificate Alias function */
   private JPanel wpdpCertAliasPanel = null;
   
   /** certificate name label */
   private JLabel wpdlAN = null;
   
   /** certificate name field */
   private JTextField wpdfAN = null;
   
   /** Cancel button */
   private JButton wpdbAC = null;
   
   /** Save button */
   private JButton wpdbAS = null;
   
   // dialog specific widgets to view certificate details
   
   /** modal dialog panel for Certificate Details function */
   private JPanel wpDDetailPanel = null;

   /** scroller for list of certificate details */
   private JScrollPane wpDScroller = null;
   
   /** label indicating certificate validity */
   private JLabel wpDLValid = null;
   
   /** table to display certificate details */
   private JTable wpDCerts = null;

   /** related table model */
   private TaggedDataTableModel wpDCtm = null;

   /** related table selection model */
   private ListSelectionModel wpDCsm = null;

   /** Cancel button */
   private JButton wpDC = null;

   /** state machine's array of components */
   /* west,                    */
   /*    wpScroller,           */
   /*       wpCerts,           */
   /*    wpCtlPanel,           */
   /*       wpbR,              */
   /*       wpbA,              */
   /*       wpbD,              */
   /*       wpbE,              */
   /*       wpbV,              */
   /*       wpcU,              */
   /*  east,                   */
   /*     epScroller,          */
   /*        epDetails         */
   
   /** state definition - initial empty */
   private final boolean[] STATE_INITIAL_EMPTY =
   {
   /* west,                    */       true,
   /*    wpScroller,           */       true,
   /*       wpCerts,           */       true,
   /*    wpCtlPanel,           */       true,
   /*       wpbR,              */       true,
   /*       wpbA,              */       true,
   /*       wpbD,              */ false,
   /*       wpbE,              */ false,
   /*       wpbV,              */ false,
   /*       wpcU,              */       true,
   /*  east,                   */ false,
   /*     epScroller,          */ false,
   /*        epDetails         */ false
   };

   /** state definition - single selection in certificate list */
   private final boolean[] STATE_CERTS_SINGLE =
   {
   /* west,                    */       true,
   /*    wpScroller,           */       true,
   /*       wpCerts,           */       true,
   /*    wpCtlPanel,           */       true,
   /*       wpbR,              */       true,
   /*       wpbA,              */       true,
   /*       wpbD,              */       true,
   /*       wpbE,              */       true,
   /*       wpbV,              */       true,
   /*       wpcU,              */       true,
   /*  east,                   */       true,
   /*     epScroller,          */       true,
   /*        epDetails         */       true
   };

   /** state definition - multiple selection in certificate list */
   private final boolean[] STATE_CERTS_MULTI =
   {
   /* west,                    */       true,
   /*    wpScroller,           */       true,
   /*       wpCerts,           */       true,
   /*    wpCtlPanel,           */       true,
   /*       wpbR,              */       true,
   /*       wpbA,              */       true,
   /*       wpbD,              */       true,
   /*       wpbE,              */ false,
   /*       wpbV,              */ false,
   /*       wpcU,              */       true,
   /*  east,                   */ false,
   /*     epScroller,          */ false,
   /*        epDetails         */ false
   };

   /** indicates single certificate selection state */
   private boolean singleCertificate = false;

   /** the selected certificate alias, for single certificate selection */
   private String certificateAlias = null;
   
   /** flag indicating if currently Peer certificates are displayed */
   private boolean isPeer = false;

   /**
    * Constructor.
    * 
    * @param    client
    *           instance of AdminClient
    */
   AdminCertificates(AdminClient client)
   {
      // store useful references
      this.client = client;
      adm = client.getExports();
      
      // create the main screen and add it to the swapper
      mainPanel = new JPanel();
      mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.X_AXIS));
      client.add(mainPanel, SCREEN_ACC_CERTS, this);

      GroupLayout mainLayout = new GroupLayout(mainPanel);
      mainPanel.setLayout(mainLayout);
      mainLayout.setAutoCreateGaps(true);
      mainLayout.setAutoCreateContainerGaps(true);
      
      // top panel with group layout manages
      // west and east sub-panels
      //  +-------screen-----+
      //  |+------+  +------+|
      //  ||      |  |      ||
      //  ||      |  |      ||
      //  || west |  | east ||
      //  ||      |  |      ||
      //  |+------+  +------+|
      //  +------------------+
      
      west = new JPanel();
      east = new JPanel();

      // Create a sequential group for the horizontal axis:
      {
         GroupLayout.SequentialGroup hGroup = 
            mainLayout.createSequentialGroup();
         
         // three panels 
         hGroup.
            addComponent(west).
            addComponent(east)
         ;
         
         mainLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a parallel group for the vertical axis
      {
         GroupLayout.ParallelGroup vGroup = 
            mainLayout.createParallelGroup(GroupLayout.Alignment.BASELINE);
            
         vGroup.
            addComponent(west).
            addComponent(east)
         ;
            
         mainLayout.setVerticalGroup(vGroup);
      }

      // stuffing west panel with components:
      // west panel is controlled with BorderLayout
      west.setLayout(new BorderLayout());
      
      //  +---------west-----------+
      //  | +--------------------+ |
      //  | |  wpTypePanel NORTH | |
      //  | +--------------------+ |
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    wpScroller      | |
      //  | |      CENTER        | |
      //  | |                    | |
      //  | |                    | |
      //  | +--------------------+ |
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    wpCtlPanel      | |
      //  | |      SOUTH         | |
      //  | +--------------------+ |
      //  +------------------------+
      
      wpCtm = new TaggedDataTableModel("Alias", "Description");
      wpCerts = new JTable(wpCtm);
      wpCerts.getTableHeader().setReorderingAllowed(false);
      wpCsm = wpCerts.getSelectionModel();
      
      wpScroller = new JScrollPane();
      wpScroller.getViewport().add(wpCerts);
      wpScroller.setBorder(new TitledBorder
       (new EtchedBorder(EtchedBorder.RAISED), " Defined Certificates "));
      
      wpTypePanel = new JPanel();
      west.add(wpTypePanel, BorderLayout.NORTH);

      wpScroller.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      west.add(wpScroller, BorderLayout.CENTER);

      wpCtlPanel = new JPanel();
      west.add(wpCtlPanel, BorderLayout.SOUTH);
      
      // wpCtlPanel panel is controlled with GroupLayout
      //  +------wpCtlPanel--------+
      //  |                        |
      //  |                        |
      //  |     buttons here       |   
      //  |                        |
      //  |                        |
      //  +------------------------+
      
      // creating control widgets
      wpbR = new JButton("Refresh");
      wpbA = new JButton("Add Certificate");
      wpbD = new JButton("Delete Selected");
      wpbV = new JButton("View Certificate");
      wpbE = new JButton("Change Alias");
      wpcU = new JCheckBox("Only Show Unassigned");

      // create layout of the wpCtlPanel panel with all controls
      GroupLayout wpCtlLayout = new GroupLayout(wpCtlPanel);
      wpCtlPanel.setLayout(wpCtlLayout);
    
      wpCtlLayout.setAutoCreateGaps(true);
      wpCtlLayout.setAutoCreateContainerGaps(true);
    
      // Create a sequential group for the horizontal axis:
      {
         GroupLayout.SequentialGroup hGroup = 
            wpCtlLayout.createSequentialGroup();
         
         // parallel group of Refresh and Change buttons
         hGroup.addGroup(
            wpCtlLayout.createParallelGroup().
            // Refresh button
            addComponent(wpbR).
            // Clone group button
            addComponent(wpbE)
         );
         
         // parallel group of Add and View buttons
         hGroup.addGroup(
            wpCtlLayout.createParallelGroup().
            // Add button
            addComponent(wpbA).
            // Edit group button
            addComponent(wpbV)
         );
         
         // parallel group of Delete and Unassigned buttons
         hGroup.addGroup(
            wpCtlLayout.createParallelGroup().
            // Delete Selected button
            addComponent(wpbD).
            // Switch to ACLs button
            addComponent(wpcU)
         );
         
         wpCtlLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a sequential group for the vertical axis
      {
         GroupLayout.SequentialGroup vGroup = wpCtlLayout.
                                              createSequentialGroup();
         vGroup.
            // parallel group of three buttons
            addGroup(wpCtlLayout.createParallelGroup(
                                              GroupLayout.Alignment.BASELINE).
               // Refresh button
               addComponent(wpbR).
               // Add button
               addComponent(wpbA).
               // Delete Selected button
               addComponent(wpbD)
               ).
            // parallel group of three more buttons
            addGroup(wpCtlLayout.createParallelGroup(
                                              GroupLayout.Alignment.BASELINE).
               // View button
               addComponent(wpbV).
               // Details button
               addComponent(wpbE).
               // Unassigned check-box
               addComponent(wpcU)
               )
            ;
            
         wpCtlLayout.setVerticalGroup(vGroup);
      }
      
      wpCtlLayout.linkSize(wpbR, wpbE);
      wpCtlLayout.linkSize(wpbA, wpbV);
      wpCtlLayout.linkSize(wpbD, wpcU);
      
      // add components to the panel
      //  +------wpTypePanel-------+
      //  | ( ) CA        ( ) Peer |
      //  +------------------------+
      
      wprCA = new JRadioButton("CA Certificates");
      wprPeer = new JRadioButton("Peer Certificates");
      
      // group radio buttons
      ButtonGroup grp = new ButtonGroup();
      grp.add(wprCA);
      grp.add(wprPeer);
      
      wpTypePanel.add(wprCA);
      wpTypePanel.add(wprPeer);
      
      // stuffing east panel with components:
      // east panel is controlled with BorderLayout
      east.setLayout(new BorderLayout());

      //  +---------east-----------+
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    epScroller      | |
      //  | |      CENTER        | |
      //  | |                    | |
      //  | |                    | |
      //  | +--------------------+ |
      //  +------------------------+
      
      epCtm = new TaggedDataTableModel("Account", "Description");
      epDetail = new JTable(epCtm);
      epDetail.getTableHeader().setReorderingAllowed(false);
      epDetail.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      epCsm = epDetail.getSelectionModel();
      
      epBorder = new TitledBorder(new EtchedBorder(EtchedBorder.RAISED), 
                                  " Associated Accounts ");
      epScroller = new JScrollPane();
      epScroller.getViewport().add(epDetail);
      epScroller.setBorder(epBorder);
      
      epScroller.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      east.add(epScroller, BorderLayout.CENTER);

      // create "Certificate Def" modal dialog panel with Group layout
      wpdpCertPanel = new JPanel();

      wpdlN = new JLabel(" Alias ");
      wpdfN = new JTextField(12);
      wpdlP = new JLabel(" PEM data ");
      wpdfP = new JTextArea(20, 64);
      wpdfPScroller = new JScrollPane();
      wpdfPScroller.getViewport().add(wpdfP);
      
      wpdbC = new JButton("Cancel");
      wpdbS = new JButton("Save");

      wpdpCertPanel.setBorder(new TitledBorder
         (new EtchedBorder(EtchedBorder.RAISED), " Certificate Definition "));
      GroupLayout wpdGroupLayout = new GroupLayout(wpdpCertPanel);
      wpdpCertPanel.setLayout(wpdGroupLayout);

      // Turn on automatically adding gaps
      wpdGroupLayout.setAutoCreateGaps(true);
      wpdGroupLayout.setAutoCreateContainerGaps(true);
    
      // Create a sequential group for the horizontal axis.
      {
         GroupLayout.SequentialGroup hGroup = wpdGroupLayout.
                                              createSequentialGroup();
       
         hGroup.addGroup(wpdGroupLayout.createParallelGroup().
            addComponent(wpdlN).
            addComponent(wpdlP).
            addComponent(wpdbC)
            );
         hGroup.addGroup(wpdGroupLayout.createParallelGroup().
            addComponent(wpdfN).
            addComponent(wpdfPScroller).
            addComponent(wpdbS)
            );
         
         wpdGroupLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a sequential group for the vertical axis.
      {
         GroupLayout.SequentialGroup vGroup = wpdGroupLayout.
                                              createSequentialGroup();
       
         vGroup.addGroup(wpdGroupLayout.createParallelGroup(GroupLayout.
                                                          Alignment.BASELINE).
            addComponent(wpdlN).
            addComponent(wpdfN)
            );
         vGroup.addGroup(wpdGroupLayout.createParallelGroup(GroupLayout.
                                                          Alignment.BASELINE).
            addComponent(wpdlP).
            addComponent(wpdfPScroller)
            );
         vGroup.addGroup(wpdGroupLayout.createParallelGroup(GroupLayout.
               Alignment.BASELINE).
               addComponent(wpdbC).
               addComponent(wpdbS)
         );
         
         wpdGroupLayout.setVerticalGroup(vGroup);
      }
      
      // data for alias change
      wpdpCertAliasPanel = new JPanel();

      wpdlAN = new JLabel(" New Alias ");
      wpdfAN = new JTextField(12);
      
      wpdbAC = new JButton("Cancel");
      wpdbAS = new JButton("Save");

      wpdpCertAliasPanel.setBorder(new TitledBorder
         (new EtchedBorder(EtchedBorder.RAISED), " Change Certificate Alias "));
      GroupLayout wpdAGroupLayout = new GroupLayout(wpdpCertAliasPanel);
      wpdpCertAliasPanel.setLayout(wpdAGroupLayout);

      // Turn on automatically adding gaps
      wpdAGroupLayout.setAutoCreateGaps(true);
      wpdAGroupLayout.setAutoCreateContainerGaps(true);
    
      // Create a sequential group for the horizontal axis.
      {
         GroupLayout.SequentialGroup hGroup = wpdAGroupLayout.
                                              createSequentialGroup();
       
         hGroup.addGroup(wpdAGroupLayout.createParallelGroup().
            addComponent(wpdlAN).
            addComponent(wpdbAC)
            );
         hGroup.addGroup(wpdAGroupLayout.createParallelGroup().
            addComponent(wpdfAN).
            addComponent(wpdbAS)
            );
         
         wpdAGroupLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a sequential group for the vertical axis.
      {
         GroupLayout.SequentialGroup vGroup = wpdAGroupLayout.
                                              createSequentialGroup();
       
         vGroup.addGroup(wpdAGroupLayout.createParallelGroup(GroupLayout.
                                                          Alignment.BASELINE).
            addComponent(wpdlAN).
            addComponent(wpdfAN)
            );
         vGroup.addGroup(wpdAGroupLayout.createParallelGroup(GroupLayout.
               Alignment.BASELINE).
               addComponent(wpdbAC).
               addComponent(wpdbAS)
         );
         
         wpdAGroupLayout.setVerticalGroup(vGroup);
      }
      
      // data for certificate details
      wpDDetailPanel = new JPanel();

      wpDCtm = new TaggedDataTableModel("Property", "Description");
      wpDCtm.sort = false;
      wpDCerts = new JTable(wpDCtm);
      wpDCerts.getTableHeader().setReorderingAllowed(false);
      wpDCerts.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      wpDCsm = wpDCerts.getSelectionModel();

      JPanel cpanel = new JPanel();
      // add validity label
      wpDLValid = new JLabel();
      cpanel.add(wpDLValid);
      // add close button
      wpDC = new JButton("Close");
      cpanel.add(wpDC);

      wpDScroller = new JScrollPane();
      wpDScroller.getViewport().add(wpDCerts);
      wpDScroller.setBorder(new TitledBorder
         (new EtchedBorder(EtchedBorder.RAISED), " Certificate Details "));

      wpDDetailPanel.setLayout(new BorderLayout());      
      wpDDetailPanel.add(wpDScroller, BorderLayout.NORTH);
      wpDDetailPanel.add(cpanel, BorderLayout.SOUTH);
      
      // Certificates table selection events
      wpCsm.addListSelectionListener(new ListSelectionListener() 
      {
         @Override
         public void valueChanged(ListSelectionEvent evt) 
         {
            handleCertificateSelection(evt);
         }
      });

      // Populate button
      wpbR.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handlePopulateCertificates();
         }
      });

      // Add button
      wpbA.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleAddCertificate();
         }
      });

      // Delete Selected button
      wpbD.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleDeleteSelectedCertificates();
         }
      });

      // Change Alias button
      wpbE.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleChangeAlias();
         }
      });

      // View Certificate Ls button
      wpbV.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleViewCertificate();
         }
      });

      // Show unassigned check-box
      wpcU.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handlePopulateCertificates();
         }
      });

      // View CA or Peer radio's

      // switch to CA view
      wprCA.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handlePopulateCertificates();
         }
      });

      // switch to Peer view
      wprPeer.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handlePopulateCertificates();
         }
      });

      // SAVE/CANCEL dialog buttons
      wpdbS.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleSave(true);
         }
      });

      // Cancel button for Add dialog
      wpdbC.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleCancel();
         }
      });

      wpdbAS.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleSave(false);
         }
      });

      // Cancel button for Add dialog
      wpdbAC.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleCancel();
         }
      });

      // Close button for View Details dialog
      wpDC.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")          
         public void actionPerformed(ActionEvent evt) 
         {
            handleCancel();
         }
      });
      
      // create the array of components for the state machine
      state = new JComponent[]
      {
         west,
            wpScroller,
               wpCerts,
            wpCtlPanel,
               wpbR, wpbA, wpbD,
               wpbE, wpbV, wpcU,
          east,
             epScroller,
                epDetail
      };

      // select CA Certificates radio
      wprCA.setSelected(true);
      
      populateCertificatesList();
      setCertificateNoSelection();
   }

   /**
    * Performs printing of the current screen data.
    * This screen prints the current view of the certificates table.
    */
   public void print()
   {
      MessageFormat header = new MessageFormat("Certificates");
      MessageFormat footer = new MessageFormat("page {0, number}");
      
      try
      {
         wpCerts.print(JTable.PrintMode.FIT_WIDTH, header, footer);
      }
      catch (PrinterException pe)
      {
         client.beep("printer exception" + pe);
      }
      catch (SecurityException se)
      {
         // nothing to do
      }
   }
   
   /**
    * Queries the extended printing capability. If the extended capability is
    * not provided, the printExtended() method should not be called.
    * 
    * @return   <code>true</code> to tell the caller the extended printing is
    *           implemented
    */
   public boolean canPrintExtended()
   {
      return true;
   }

   /**
    * Creates extended print preview panel off the current screen data.
    * This screen prints the current view of the certificates table.
    */
   public void printExtended()
   {
      // TO-DO
   }
   
   /**
    * Init the screen to automatically select the given certificate.
    * 
    * @param    alias
    *           The certificate alias
    */
   void init(String alias)
   {
      CertDef cdef = adm.getCert(alias);
      if (cdef == null)
      {
         client.alarm();
         return;
      }
      
      wprCA.setSelected(cdef.authority);
      wprPeer.setSelected(!cdef.authority);

      wpCsm.clearSelection();
      
      // list Peer or CA certificates 
      handlePopulateCertificates();
      
      int idx = wpCtm.getRowIndex(cdef.alias);
      wpCsm.setSelectionInterval(idx, idx);
   }

   /** 
    * Presents the refreshed user accounts management screen.
    */
   void manageCertificates() 
   {
      // switch to this screen without changing state of components
      client.show(SCREEN_ACC_CERTS, TITLE_ACC_CERTS);
   }

   /**
    * Populate the list of certificates with fresh data from the server, 
    * depending on the selection.
    */
   private void populateCertificatesList() 
   {
      boolean unassigned = wpcU.isSelected();
      
      // clear selection if any
      wpCsm.clearSelection();
      
      // clear the old list
      wpCtm.clear();
      
      isPeer = wprPeer.isSelected();
      
      // get the list of certificates from the server
      AdminUtils.busy(client);

      TaggedName[] list = isPeer ? adm.listPeerCerts(unassigned)
                                 : adm.listAuthCerts(unassigned);
      // set certificate description to Common Name
      setCertificateDescription(list, CERT_DESCRIPTION_PROP);
      
      epBorder.setTitle(isPeer ? " Associated Accounts " 
                               : " Certificate Hierarchy ");
      epScroller.repaint();
      
      AdminUtils.idle(client);
      
      if (list != null)
      {
         if (list.length > 0)
         {
            wpCtm.loadAll(list);
         }
      }
      else
      {
         client.beep("no certificates");
      }
   }
   
   /**
    * Populate the list of certificates with fresh data from the server, when
    * the Refresh button is pressed.
    */
   private void handlePopulateCertificates()
   {
      // populate the certificate list
      populateCertificatesList();

      // set the state accordingly
      setCertificateNoSelection();
   }

   /**
    * Changes state of the components in response to certificate list
    * selections. There are three distinctive states:
    * <ul>
    *    <li>no selection or empty list
    *    <li>single entry selected
    *    <li>multiple entries selected
    * </ul>
    * This method determines the selection state of the list and changes the
    * state of the components accordingly.
    * <p>
    * No selection means that the state should be STATE_INITIAL_EMPTY. Only
    * the "Populate" and "Add Certificate" buttons are enabled.
    * <p>
    * Single selection means that all buttons of the west panel are enabled.
    * Also, the east panel will be filled with data about the selected cert.
    * <p>
    * Multiple selection means that buttons, specific to the single selection,
    * are disabled. The enabled buttons on the west panel are Populate, Add
    * Process and Delete selected. The east panel will be empty.
    * 
    * @param    evt
    *           event caused this method call
    */
   private void handleCertificateSelection(ListSelectionEvent evt)
   {
      // do not bother until the dust is fully settled
      if (evt.getValueIsAdjusting())
      {
         return;
      }

      // figure out the selection state
      if (wpCsm.isSelectionEmpty())
      {
         // no selection
         setCertificateNoSelection();
      }
      else if (wpCerts.getSelectedRows().length == 1)
      {
         // single selection
         setCertificateSingleSelection();
      }
      else
      {
         // multiple selection
         setCertificateMultiSelection();
      }
   }

   /**
    * Sets state of components to no selection in the Certificates list
    * <p>
    * No selection means that the state should be STATE_INITIAL_EMPTY.
    * Only the "Populate" and "Add process" buttons are enabled.
    */
   private void setCertificateNoSelection()
   {
      // reset flags
      singleCertificate = false;
      certificateAlias = null;

      // set enabled state of the list accordingly
      stateMachine(STATE_INITIAL_EMPTY);

      // load certificate details
      loadCertificateDetails();
   }

   /**
    * Sets state of components to single selection in the Certificate list 
    */
   private void setCertificateSingleSelection()
   {
      // busy
      AdminUtils.busy(client);
      
      // set flags
      singleCertificate = true;
      certificateAlias = (String) wpCerts.getValueAt(wpCerts.getSelectedRow(), 
                                                     0);

      if (certificateAlias == null)
      {
         setCertificateNoSelection();

         client.beep("");
      }
      else
      {
         // set enabled state of the components
         stateMachine(STATE_CERTS_SINGLE);

         client.ok();
      }

      // load certificate details
      loadCertificateDetails();
      
      // idle
      AdminUtils.idle(client);
   }

   /**
    * Sets state of components to multiple selection in the Certificate list 
    */
   private void setCertificateMultiSelection()
   {
      // reset flags
      singleCertificate = false;
      certificateAlias = null;
      
      // set enabled state of the components
      stateMachine(STATE_CERTS_MULTI);

      // load certificate details
      loadCertificateDetails();
   }

   /**
    * When only one certificate is selected, loads its details in the east
    * panel.
    */
   private void loadCertificateDetails()
   {
      // empty previous details
      epCsm.clearSelection();
      epCtm.clear();
      
      if (singleCertificate)
      {
         TaggedName[] data = null;

         if (isPeer)
         {
            // load accounts for this certificate
            data = adm.getCertAccounts(certificateAlias);
         }
         else
         {
            // load certificate hierarchy
            data = CertificateUtils.getCertificateHierarchy(certificateAlias,
                                                            adm);
            
            setCertificateDescription(data, CERT_DESCRIPTION_PROP);
         }
         
         epCtm.sort = isPeer; // only peer details are sorted

         epCtm.loadAll(data);            
      }
   }
   
   /**
    * Handle Add button press in the west panel.
    * <p>
    * This will reset all fields in the Add Cert dialog and will display it.
    */
   private void handleAddCertificate()
   {
      // reset dialog fields         
      wpdfN.setText("");
      wpdfP.setText("");

      // focus on certificate alias
      wpdfN.requestFocusInWindow();

      // display modal dialog
      dialog = AdminUtils.modalDialog(client, 
                                      "Certificate Definition", 
                                      wpdpCertPanel);
      dialog.setVisible(true);

      if (dialog != null)
      {
         // dialog dismissed by closing window
         handleCancel();
      }
   }

   /**
    * Handle Save button press in the Certificate Def panel. Save button tries
    * and adds or update the process account definition. When adding, if the
    * server responds OK, it will add the new row to the west-side table. When
    * editing, it will update the currently selected row.
    * <p>
    * The following fields are mandatory: certificate alias and PEM data.
    * 
    * @param    add
    *           <code>true</code> when adding a new certificate
    */
   private void handleSave(boolean add)
   {
      JTextField fname = add ? wpdfN : wpdfAN;
      
      // get and check the certificate alias
      String alias = validateStringField(fname, "type a certificate alias");
      if (alias == null)
      {
         return;
      }
      
      // get and check the certificate PEM data, only if in ADD mode
      String pem = add ? validateStringField(wpdfP, 
                                             "enter certificate PEM data") 
                       : "";
      if (pem == null)
      {
         return;
      }

      // dispatch the action
      if (add)
      {
         String[] cert = pem.trim().split("\n");

         // add the definition to the server and iterate
         // list remains with no selection
         if (saveCertificateDef(alias, cert, isPeer))
         {
            CertDef cd = adm.getCert(alias);

            // get certificate description
            String desc = CertificateUtils.
                 getCertificateDNProperty(cd, CERT_DESCRIPTION_PROP, true);

            // successfully added to the server; add to the list
            client.ok();
            wpCtm.addRow(new TaggedName(alias, desc));

            // refocus name field
            wpdfN.requestFocusInWindow();
            
            // clear PEM data
            wpdfP.setText("");

            // clear selection now when the table is modified
            wpCsm.clearSelection();
            return;
         }

         // adding definition failed; stay in the editing
         wpdfN.requestFocusInWindow();
      }
      else
      {
         // change the existing definition
         // list retains the selection
         if (changeCertAlias(certificateAlias, alias))
         {
            CertDef cd = adm.getCert(alias);
            
            // get certificate description
            String desc = 
               CertificateUtils.getCertificateDNProperty(cd,
                                                         CERT_DESCRIPTION_PROP, 
                                                         true);

            // successfully saved changes
            client.ok();
            wpCtm.refreshRow(new TaggedName(alias, desc),
                             wpCerts.getSelectedRow());
            
            // hide the dialog
            dialog.setVisible(false);
            dialog = null;

            loadCertificateDetails();
            
            return;
         }
         
         // saving definition failed; stay in the editing 
         wpdfN.requestFocusInWindow();
         client.alarm();
      }
   }
   
   /**
    * Adds new certificate definition to the server.
    * 
    * @param    alias
    *           new certificate alias
    * @param    cert
    *           PEM representation of this certificate
    * @param    peer
    *           <code>true</code> if this is a Peer certificate
    * 
    * @return   <code>true</code> if added successfully
    */
   private boolean saveCertificateDef(String   alias, 
                                      String[] cert, 
                                      boolean  peer)
   {
      CertDef cd = new CertDef();
      cd.alias = alias;
      cd.certificate = cert;
      cd.authority = peer == false;

      boolean rc = true;
      
      AdminUtils.busy(client);      
      int state = CertificateUtils.validateCertificate(cd, adm);
      AdminUtils.idle(client);

      if (state != CertificateUtils.CERT_VALID)
      {
         String msg = CertificateUtils.getValiditityMessage(state);
         client.beep("certificate is " + msg);
         rc = false;
      }
      else
      {
         // verify if user wants and can add a root CA cert
         AdminUtils.busy(client);
         boolean root = cd.authority && 
                        CertificateUtils.isSelfSigned(cd);
         AdminUtils.idle(client);

         if (root)
         {            
            // check if there is only one root certificate
            AdminUtils.busy(client);
            TaggedName[] authCerts = adm.listAuthCerts(false);
            AdminUtils.idle(client);

            if (authCerts.length == 0)
            {
               rc = AdminUtils.confirmDialog(client, "Add root certificate", 
                      "Are you sure you want to add a root CA certificate ?");
            }
            else
            {
               rc = false;

               client.beep("CA certificate list not empty - adding root CA " +
               		"certificate is not allowed");
            }
         }

         if (rc)
         {
            AdminUtils.busy(client);
            rc = adm.addCert(cd);
            AdminUtils.idle(client);
            
            if (!rc)
            {
               // display server-side error message
               client.alarm();
            }
         }
      }
      

      return rc;
   }
   
   /**
    * Changes the alias for the specified certificate.
    * 
    * @param    oldAlias
    *           old certificate alias
    * @param    newAlias
    *           new certificate alias
    */
   private boolean changeCertAlias(String oldAlias, String newAlias)
   {
      if (oldAlias.equals(newAlias))
      {
         client.beep("new alias is equal to old one");
         return false;
      }
            
      AdminUtils.busy(client);
      boolean rc = adm.moveCert(oldAlias, newAlias);
      AdminUtils.idle(client);
      
      certificateAlias = newAlias;

      return rc;
   }
      
   /**
    * Handle Delete Selected button press in the west panel.
    */
   private void handleDeleteSelectedCertificates()
   {
      // get the selection
      int[] sel = wpCerts.getSelectedRows();
      
      // safety check
      if (sel.length == 0)
      {
         return;
      }
      
      // ask confirmation
      String quest = sel.length == 1 ?
                     "1 certificate being deleted. Continue?" :
                     Integer.toString(sel.length) + 
                     " certificates being deleted. Continue?";
      
      if (!AdminUtils.confirmDialog(client, 
                                    "Deleting Certificates", 
                                    quest))
      {
         // declined
         return;
      }
      
      // busy
      AdminUtils.busy(client);

      // sort the array of selected indices to walk backwards
      Arrays.sort(sel);

      // walk the array backwards and delete selected certificates
      for (int i = sel.length - 1; i >= 0; i--)
      {
         // pick up the alias of certificate being deleted
         String alias = (String) wpCerts.getValueAt((sel[i]), 0);
         
         // check if this certificate is assigned to any accounts
         TaggedName[] accounts = adm.getCertAccounts(alias);
         if (accounts != null && accounts.length > 0)
         {
            // accounts exist, get their name into a string
            String sacc = "";
            for (int j = 0; j < accounts.length; j++)
            {
               sacc = sacc + accounts[j].getName();
               if (j < accounts.length - 1)
                      sacc = sacc + ", ";
            }

            // display a Yes/No/Cancel dialog
            String question = 
               "Certificate <" + alias + "> is assigned to the following " +
               		"accounts: " + sacc + ".\n" + 
               "Select Yes to delete it, No to skip it or Cancel to stop " +  
               "deletion.";
            Boolean res = AdminUtils.confirmCancelDialog(client, 
                              "Delete Certificate " + alias, question);
            if (res == null)
            {
               // deletion is canceled
               break;
            }
            if (!res.booleanValue())
            {
               // deletion is skipped for this certificate
               continue;
            }
         }
         
         // attempt deletion
         boolean rc = adm.deleteCert(alias, true);
         if (!rc)
         {
            // attempt has failed
            client.alarm();
         }
         else
         {
            // attempt has succeeded; delete this row
            client.ok();
            wpCtm.deleteRow(sel[i]);
         }
      }

      // idle
      AdminUtils.idle(client);
   }

   /**
    * Handle Change Alias button press in the west-panel.
    */
   private void handleChangeAlias()
   {
      // load fields
      wpdfAN.setText(certificateAlias);

      // focus on certificate alias
      wpdfAN.requestFocusInWindow();

      // display modal dialog
      dialog = AdminUtils.modalDialog(client, 
                                      "Change Certificate Alias", 
                                      wpdpCertAliasPanel);
      dialog.setVisible(true);

      if (dialog != null)
      {
         // dialog dismissed by closing window
         handleCancel();
      }
   }

   /**
    * Display information about the selected certificate: owner, issuer, 
    * validity period.
    */
   @SuppressWarnings("unchecked")
   private void handleViewCertificate()
   {
      // clear selection if any
      wpDCsm.clearSelection();
      
      // clear the old list
      wpDCtm.clear();
      
      CertDef cd = adm.getCert(certificateAlias);
      if (cd == null)
      {
         client.alarm();
         return;
      }

      // get certificate information
      Map info = CertificateUtils.getCertificateProperties(cd);
      
      if (info == null)
      {
         client.beep("Certificate definition is invalid.");
         return;
      }
      
      ArrayList<TaggedName> data = new ArrayList<TaggedName>();

      Iterator itr = info.keySet().iterator();
      while (itr.hasNext())
      {
         String key = (String) itr.next();
         Object val = info.get(key);
         String lbl = CertificateUtils.getPropertyLabel(key);
         
         if (val instanceof Map)
         {
            data.addAll(buildCertificateInfo((Map) val, lbl));
         }
         else
         {
            data.add(new TaggedName(lbl, val.toString()));
         }
      }
      
      if (data.size() > 0)
      {
         TaggedName[] list = data.toArray(new TaggedName[0]);
         wpDCtm.loadAll(list);
      }
      
      // check certificate validity
      int state = CertificateUtils.validateCertificate(cd, adm);
      String msg = CertificateUtils.getValiditityMessage(state);
      
      if (state != CertificateUtils.CERT_VALID)
      {
         wpDLValid.setForeground(Color.RED);
      }
      else
      {
         wpDLValid.setForeground(Color.BLACK);
      }
      
      wpDLValid.setText("certificate is " + msg);
      
      
      // display modal dialog
      dialog = AdminUtils.modalDialog(client, 
                                      "Certificate Details", 
                                      wpDDetailPanel);
      dialog.setSize(500, 500);
      dialog.setLocationRelativeTo(client);
      dialog.setVisible(true);

      if (dialog != null)
      {
         // dialog dismissed by closing window
         handleCancel();
      }
   }

   /**
    * Builds {@link TaggedName} objects from the specified source. Also, it
    * adds a title row with the specified text.
    * 
    * @param    source
    *           a map for which the its <code>prop</code> entry holds a
    * @param    title
    *           the title to be added
    * 
    * @return   a list of rows.
    */
   @SuppressWarnings("unchecked")
   private ArrayList<TaggedName> buildCertificateInfo(Map source, 
                                                      String title)
   {
      ArrayList<TaggedName> res = new ArrayList<TaggedName>();
      
      res.add(new TaggedName(title, ""));
      
      Iterator itr = source.keySet().iterator();
      while (itr.hasNext())
      {
         String key = (String) itr.next();
         String val = (String) source.get(key);
         
         String lbl = CertificateUtils.getPropertyLabel(key);
         
         res.add(new TaggedName(lbl, val));
      }
      
      // add an empty row
      res.add(new TaggedName("", ""));
      
      return res;
   }

   /**
    * Handle Cancel button press in the Certificate Def dialog.
    */
   private void handleCancel()
   {
      // clear error messages
      client.ok();
      
      // hide the dialog
      dialog.setVisible(false);
      dialog = null;
   }
}