/*
** Module   : AdminCustomMenu.java
** Abstract : Plugin definition for handling customer-specific client menus.
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20090806  @43537  Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client;

import java.util.List;
import javax.swing.*;

/**
 * Plugin definition for handling customer-specific admin client menus.
 * <p>
 * The methods of this interface receive the current menu as a list of
 * <code>JMenuItem</code>s, which can be modified to suit the application.
 * <p>
 * The initial contents of the list are as follows:
 * <ul>
 *   <li>index 0 - <code>JMenu</code> for Console
 *   <li>index 1 - <code>JMenu</code> for Accounts
 *   <li>index 2 - <code>JMenu</code> for Access Control
 *   <li>index 3 - <code>JMenu</code> for Runtime Setting
 *   <li>index 4 - <code>JMenu</code> for Configuration
 *   <li>index 5 - <code>JMenu</code> for Target
 *   <li>index 6 - <code>JMenuItem</code> for Print
 *   <li>index 7 - <code>JMenuItem</code> for Log Off
 *   <li>index 8 - <code>JMenuItem</code> for a filler
 *   <li>index 9 - <code>JMenu</code> for Help
 * </ul>
 * The contents of the <code>JMenu</code>s may change. Please refer to the
 * AdminMenu class for the up to date information.
 *
 * @author NVS
 */
public interface AdminCustomMenu
{
   /**
    * Attaches the plugin to the AdminClient instance.
    * Called once upon plugin loading.
    *  
    * @param client
    *        instance of AdminClient
    */
   public void attach(AdminClient client);
   
   /**
    * Inserts custom major and minor menu items.
    * The inserted menu items have to have the event listeners attached
    * for the menu items to have effect.
    *  
    * @param menu
    *        the main menu list
    */
   public void insertCustomMenuItems(List<JMenuItem> menu);

   /**
    * Conditionally removes custom major and minor menu items.
    * This method should remove all menu items that were inserted by the 
    * last call to {@link #insertCustomMenuItems} 
    *  
    * @param menu
    *        the main menu list
    */
   public void removeCustomMenuItems(List<JMenuItem> menu);
}
