/*
** Module   : AdminGroupAccounts.java
** Abstract : Group Accounts management client
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ------------------Description--------------------
** 001 NVS 20090325   @41687 Created the first version of the file.
** 002 NVS 20090428   @42037 Completed implementation of all functions.
** 003 NVS 20090429   @42079 Capitalized labels and button texts. Implemented
**                           case-insensitive filtering in All Users table.
** 004 CA  20090507   @42107 Extracted some common functionalities into class
**                           AdminScreen.
** 005 CA  20090522   @42429 Moved createGroup to  AdminScreen. Fixed an 
**                           IndexOutOfBounds exception in 
**                           handleAllUsersSelection.
** 006 GES 20090617   @42758 Moved filter logic into AdminUtils.createFilter().
** 007 CA  20090720   @43243 Added code for the Switch to ACLs button.
** 008 NVS 20090723   @43351 Tables don't allow moving columns.
** 009 NVS 20090729   @43428 Implemented groups printing.
** 010 NVS 20090803   @43481 Implemented extended detail groups printing.
** 011 NVS 20090804   @43513 Removed some old code and added accidentally 
**                           deleted method call.
** 012 NVS 20090805   @43531 Added busy/idle feedback to printing methods.
** 013 SIY 20091116   @44336 Refactoring of things related to TaggedName. 
**                           Minor cleanups.
** 014 SVL 20090916          Added auth mode and auth plugin fields.
** 015 SVL 20100920          Changed to conform new AuthModeComboBoxes API.
** 016 EVL 20160223          Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client;

import java.util.*;
import java.util.List;
import java.awt.*;
import java.awt.event.*;
import java.text.MessageFormat;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;

import com.goldencode.p2j.admin.TaggedName;
import com.goldencode.p2j.admin.GroupDef;

/**
 * This is a Group Accounts management class for the admin session. 
 * 
 * @author NVS
 */
public class AdminGroupAccounts 
extends AdminScreen
implements PrintRequest
{
   /** Screen switcher - group accounts screen ID */
   public static final String SCREEN_ACC_GROUPS = "adm_acc_groups";
   
   /** Screen title */
   private static final String TITLE_ACC_GROUPS = "Group Accounts Management";

   // west panel widgets, all named wp*
   
   /** west panel - works with the list of groups */
   private JPanel west = null;
   
   /** scroller for list of groups */
   private JScrollPane wpScroller = null;
   
   /** table to display all groups */
   private JTable wpGroups = null;

   /** related table model */
   private TaggedDataTableModel wpGtm = null;

   /** related table selection model */
   private ListSelectionModel wpGsm = null;

   /** sub-panel that holds all controls for list of groups */
   private JPanel wpCtlPanel = null;
   
   /** Populate button */
   private JButton wpbR = null;

   /** Add Group button */
   private JButton wpbA = null;
   
   /** Edit Group button */
   private JButton wpbE = null;

   /** Delete Selected button */
   private JButton wpbD = null;
      
   /** Switch to ACLs button */
   private JButton wpbS = null;
   
   /** Name field label */
   private JLabel wplN = null;
   
   /** Name field */
   private JTextField wpfN = null;
   
   /** With ACLs label */
   private JLabel wplW = null;

   /** With ACLs check box */
   private JCheckBox wpcW = null;

   /** Clone Group button */
   private JButton wpbC = null;
   
   /** Clone button  in modal dialog*/
   private JButton wpbG = null;
   
   /** modal dialog panel for Clone As function */
   private JPanel wpClonePanel = null;

   // center panel widgets, all named cp* 

   /** center panel - works with the lselected group */
   private JPanel center = null;
   
   /** scroller for list of group users */
   private JScrollPane cpScroller = null;

   /** table to display group users */
   private JTable cpUsers = null;

   /** related table model */
   private TaggedDataTableModel cpUtm = null;

   /** related table selection model */
   private ListSelectionModel cpUsm = null;
   
   /** Name field label */
   @SuppressWarnings("unused")
   private JLabel cplN = null;
   
   /** Name field */
   private JTextField cpfN = null;
   
   /** Add User button */
   private JButton cpbA = null;
   
   /** Delete Selected button */
   private JButton cpbD = null;
   
   /** Group Name field label */
   private JLabel cplG = null;
   
   /** Group Name field */
   private JTextField cpfG = null;
   
   /** Group Description field label */
   private  JLabel cplD = null;
   
   /** Group Description field */
   private JTextField cpfD = null;

   /** Auth mode and auth plugin combo-boxes. */
   private AuthModeComboBoxes authElements = null;

   /** Cancel button */
   private JButton cpbC = null;
   
   /** Save button */
   private JButton cpbS = null;
   
   /** sub-panel that holds all controls for list of group users */
   private JPanel cpCtlPanel = null;

   /** pane that holds all controls for Add user group */
   private JPanel cpAddPanel = null;
   
   /** modal dialog panel for Group Definition function */
   private JPanel cpGroupPanel = null;

   // east panel widgets, all named ep*
   
   /** east panel - works with the list of all users */
   private JPanel east = null;

   /** scroller for list of all users */
   private JScrollPane epScroller = null;
   
   /** scroller's titled border - all */
   private TitledBorder epScrollTitleA = null; 
 
   /** scroller's titled border - filtered */
   private TitledBorder epScrollTitleF = null; 

   /** table to display group users */
   private JTable epUsers = null;

   /** related table model */
   private TaggedDataTableModel epUtm = null;

   /** related table selection model */
   private ListSelectionModel epUsm = null;
   
   /** sub-panel that holds all controls for list of all users */
   private JPanel epCtlPanel = null;

   /** sub-panel that holds Search related controls */
   private JPanel epSearchPanel = null;
   
   /** Populate button */
   private JButton epbP = null;

   /** Add Selected button */
   private JButton epbA = null;
   
   /** search argument field */
   private JTextField epfS = null;
   
   /** Filter button */
   private JButton epbF = null;
   
   /* All users table sorter */
   private TableRowSorter<TaggedDataTableModel> epSorter = null;    
   
   // miscellaneous
   
   /** keeps the selected group definition while editing */
   private String editingGroup = null;

   /** keeps the selected group auth mode while editing */
   private int editingAuthMode = 0;

   /** keeps the selected group auth plugin while editing */
   private String editingAuthPlugin = null;

   /** state saved on Add group button press */
   private boolean[] modal = null;
   
   /* west,                    */
   /*    wpScroller,           */
   /*       wpGroups,          */
   /*    wpCtlPanel,           */
   /*       wpbR,              */
   /*       wpbA,              */
   /*       wpbD,              */
   /*       wpbC,              */
   /*       wpbE,              */
   /*       wpbS,              */
   /* center,                  */
   /*    cpScroller,           */
   /*       cpUsers,           */
   /*    cpCtlPanel,           */
   /*       cpfN,              */
   /*       cpbA,              */
   /*       cpbD,              */
   /*  east,                   */
   /*     epScroller,          */
   /*        epUsers,          */
   /*     epCtlPanel,          */
   /*        epbP,             */
   /*        epbA,             */
   /*        epSearchPanel,    */
   /*           epfS,          */
   /*           epbF           */
   
   /** state definition - initial empty */
   private final boolean[] STATE_INITIAL_EMPTY =
   {
   /* west,                    */       true,
   /*    wpScroller,           */ false,
   /*       wpGroups,          */ false,
   /*    wpCtlPanel,           */       true,
   /*       wpbR,              */       true,
   /*       wpbA,              */       true,
   /*       wpbD,              */ false,
   /*       wpbC,              */ false,
   /*       wpbE,              */ false,
   /*       wpbS,              */ false,
   /* center,                  */ false,
   /*    cpScroller,           */ false,
   /*       cpUsers,           */ false,
   /*    cpCtlPanel,           */ false,
   /*       cpfN,              */ false,
   /*       cpbA,              */ false,
   /*       cpbD,              */ false,
   /*  east,                   */       true,
   /*     epScroller,          */ false,
   /*        epUsers,          */ false,
   /*     epCtlPanel,          */       true,
   /*        epbP,             */       true,
   /*        epbA,             */ false,
   /*        epSearchPanel,    */ false,
   /*           epfS,          */ false,
   /*           epbF           */ false
   };
   
   /** state definition - group definition creation */
   private final boolean[] STATE_GROUP_ADD =
   {
   /* west,                    */ false,       
   /*    wpScroller,           */ false,
   /*       wpGroups,          */ false,
   /*    wpCtlPanel,           */ false,
   /*       wpbR,              */ false,
   /*       wpbA,              */ false,
   /*       wpbD,              */ false,
   /*       wpbC,              */ false,
   /*       wpbE,              */ false,
   /*       wpbS,              */ false,
   /* center,                  */       true,
   /*    cpScroller,           */ false,
   /*       cpUsers,           */ false,
   /*    cpCtlPanel,           */       true,
   /*       cpfN,              */ false,
   /*       cpbA,              */ false,
   /*       cpbD,              */ false,
   /*  east,                   */ false,
   /*     epScroller,          */ false,
   /*        epUsers,          */ false,
   /*     epCtlPanel,          */ false,
   /*        epbP,             */ false,
   /*        epbA,             */ false,
   /*        epSearchPanel,    */ false,
   /*           epfS,          */ false,
   /*           epbF           */ false
   };

   /** state definition - group definition editing */
   private final boolean[] STATE_GROUP_EDIT =
   {
   /* west,                    */ false,       
   /*    wpScroller,           */ false,
   /*       wpGroups,          */ false,
   /*    wpCtlPanel,           */ false,
   /*       wpbR,              */ false,
   /*       wpbA,              */ false,
   /*       wpbD,              */ false,
   /*       wpbC,              */ false,
   /*       wpbE,              */ false,
   /*       wpbS,              */ false,
   /* center,                  */       true,
   /*    cpScroller,           */ false,
   /*       cpUsers,           */ false,
   /*    cpCtlPanel,           */       true,
   /*       cpfN,              */ false,
   /*       cpbA,              */ false,
   /*       cpbD,              */ false,
   /*  east,                   */ false,
   /*     epScroller,          */ false,
   /*        epUsers,          */ false,
   /*     epCtlPanel,          */ false,
   /*        epbP,             */ false,
   /*        epbA,             */ false,
   /*        epSearchPanel,    */ false,
   /*           epfS,          */ false,
   /*           epbF           */ false
   };

   /** state definition - single selection in groups list */
   private final boolean[] STATE_GROUPS_SINGLE =
   {
   /* west,                    */       true,
   /*    wpScroller,           */       true,
   /*       wpGroups,          */       true,
   /*    wpCtlPanel,           */       true,
   /*       wpbR,              */       true,
   /*       wpbA,              */       true,
   /*       wpbD,              */       true,
   /*       wpbC,              */       true,
   /*       wpbE,              */       true,
   /*       wpbS,              */       true,
   /* center,                  */       true,
   /*    cpScroller,           */       true,
   /*       cpUsers,           */       true,
   /*    cpCtlPanel,           */       true,
   /*       cpfN,              */       true,
   /*       cpbA,              */       true,
   /*       cpbD,              */ false,
   /*  east,                   */       true,
   /*     epScroller,          */ false,
   /*        epUsers,          */ false,
   /*     epCtlPanel,          */       true,
   /*        epbP,             */       true,
   /*        epbA,             */ false,
   /*        epSearchPanel,    */ false,
   /*           epfS,          */ false,
   /*           epbF           */ false
   };
   
   /** state definition - multiple selection in groups list */
   private final boolean[] STATE_GROUPS_MULTI =
   {
   /* west,                    */       true,
   /*    wpScroller,           */       true,
   /*       wpGroups,          */       true,
   /*    wpCtlPanel,           */       true,
   /*       wpbR,              */       true,
   /*       wpbA,              */       true,
   /*       wpbD,              */       true,
   /*       wpbC,              */ false,
   /*       wpbE,              */ false,
   /*       wpbS,              */ false,
   /* center,                  */ false,
   /*    cpScroller,           */ false,
   /*       cpUsers,           */ false,
   /*    cpCtlPanel,           */ false,
   /*       cpfN,              */ false,
   /*       cpbA,              */ false,
   /*       cpbD,              */ false,
   /*  east,                   */       true,
   /*     epScroller,          */ false,
   /*        epUsers,          */ false,
   /*     epCtlPanel,          */       true,
   /*        epbP,             */       true,
   /*        epbA,             */ false,
   /*        epSearchPanel,    */ false,
   /*           epfS,          */ false,
   /*           epbF           */ false
   };
   
   /** state definition - all users list populated */
   private final boolean[] STATE_ALL_USERS =
   {
   /* west,                    */ false,
   /*    wpScroller,           */ false,
   /*       wpGroups,          */ false,
   /*    wpCtlPanel,           */ false,
   /*       wpbR,              */ false,
   /*       wpbA,              */ false,
   /*       wpbD,              */ false,
   /*       wpbC,              */ false,
   /*       wpbE,              */ false,
   /*       wpbS,              */ false,
   /* center,                  */ false,
   /*    cpScroller,           */ false,
   /*       cpUsers,           */ false,
   /*    cpCtlPanel,           */ false,
   /*       cpfN,              */ false,
   /*       cpbA,              */ false,
   /*       cpbD,              */ false,
   /*  east,                   */       true,
   /*     epScroller,          */       true,
   /*        epUsers,          */       true,
   /*     epCtlPanel,          */       true,
   /*        epbP,             */       true,
   /*        epbA,             */ false,
   /*        epSearchPanel,    */       true,
   /*           epfS,          */       true,
   /*           epbF           */       true,
   };
   
   /** mask definition - west-center */
   private final boolean[] MASK_WEST_CENTER =
   {
   /* west,                    */ true,
   /*    wpScroller,           */ true,
   /*       wpGroups,          */ true,
   /*    wpCtlPanel,           */ true,
   /*       wpbR,              */ true,
   /*       wpbA,              */ true,
   /*       wpbD,              */ true,
   /*       wpbC,              */ true,
   /*       wpbE,              */ true,
   /*       wpbS,              */ true,
   /* center,                  */ true,
   /*    cpScroller,           */ true,
   /*       cpUsers,           */ true,
   /*    cpCtlPanel,           */ true,
   /*       cpfN,              */ true,
   /*       cpbA,              */ true,
   /*       cpbD,              */ true,
   /*  east,                   */ false,
   /*     epScroller,          */ false,
   /*        epUsers,          */ false,
   /*     epCtlPanel,          */ false,
   /*        epbP,             */ false,
   /*        epbA,             */ false,
   /*        epSearchPanel,    */ false,
   /*           epfS,          */ false,
   /*           epbF           */ false
   };
   
   /** mask definition - east */
   private final boolean[] MASK_EAST =
   {
   /* west,                    */ false,
   /*    wpScroller,           */ false,
   /*       wpGroups,          */ false,
   /*    wpCtlPanel,           */ false,
   /*       wpbR,              */ false,
   /*       wpbA,              */ false,
   /*       wpbD,              */ false,
   /*       wpbC,              */ false,
   /*       wpbE,              */ false,
   /*       wpbS,              */ false,
   /* center,                  */ false,
   /*    cpScroller,           */ false,
   /*       cpUsers,           */ false,
   /*    cpCtlPanel,           */ false,
   /*       cpfN,              */ false,
   /*       cpbA,              */ false,
   /*       cpbD,              */ false,
   /*  east,                   */ true,
   /*     epScroller,          */ true,
   /*        epUsers,          */ true,
   /*     epCtlPanel,          */ true,
   /*        epbP,             */ true,
   /*        epbA,             */ true,
   /*        epSearchPanel,    */ true,
   /*           epfS,          */ true,
   /*           epbF           */ true
   };
   
   /** indicates single group selection state */
   private boolean singleGroup = false;

   /** name of the selected group for single group selection */
   private String groupName = null;

   /** recursion prevention for all users list */
   private boolean selecting = false;
   
   /** current filter specification */
   @SuppressWarnings("unused")
   private String lastFilter = null;

   /** print: sortable table model */
   private TableModel backTm = null;
   
   /** print: sortable table */
   private JTable backTable = null;
   
   /** print: extended table model */
   private TableModel frontTm = null;
   
   /** print: extended table */
   private JTable frontTable = null;
   
   /** print header */
   private MessageFormat header = null;
   
   /** print footer */
   private MessageFormat footer = null;
   
   /**
    * Constructor.
    * 
    * @param    client
    *           instance of AdminClient
    */
   AdminGroupAccounts(AdminClient client)
   {
      // store useful references
      this.client = client;
      adm = client.getExports();
      
      // create the main screen and add it to the swapper
      mainPanel = new JPanel();
      client.add(mainPanel, SCREEN_ACC_GROUPS, this);
      GroupLayout mainLayout = new GroupLayout(mainPanel);
      mainPanel.setLayout(mainLayout);
      mainLayout.setAutoCreateGaps(true);
      mainLayout.setAutoCreateContainerGaps(true);
      
      // top panel with group layout manages
      // west, center and east sub-panels
      //  +-------screen---------------+
      //  |+------+  +------+  +------+|
      //  ||      |  |      |  |      ||
      //  ||      |  |      |  |      ||
      //  || west |  |center|  | east ||
      //  ||      |  |      |  |      ||
      //  |+------+  +------+  +------+|
      //  +----------------------------+
      
      west = new JPanel();
      center = new JPanel();
      east = new JPanel();

      // Create a sequential group for the horizontal axis:
      {
         GroupLayout.SequentialGroup hGroup = 
            mainLayout.createSequentialGroup();
         
         // three panels 
         hGroup.
            addComponent(west).
            addComponent(center).
            addComponent(east)
         ;
         
         mainLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a parallel group for the vertical axis
      {
         GroupLayout.ParallelGroup vGroup = 
            mainLayout.createParallelGroup(GroupLayout.Alignment.BASELINE);
            
         vGroup.
            addComponent(west).
            addComponent(center).
            addComponent(east)
         ;
            
         mainLayout.setVerticalGroup(vGroup);
      }

      // stuffing west panel with components:
      // west panel is controlled with BorderLayout
      west.setLayout(new BorderLayout());
      
      //  +---------west-----------+
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    wpScroller      | |
      //  | |      CENTER        | |
      //  | |                    | |
      //  | |                    | |
      //  | +--------------------+ |
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    wpCtlPanel      | |
      //  | |      SOUTH         | |
      //  | +--------------------+ |
      //  +------------------------+
      
      wpGtm = new TaggedDataTableModel("Group ID", "Description");
      wpGroups = new JTable(wpGtm);
      wpGroups.getTableHeader().setReorderingAllowed(false);
      wpGsm = wpGroups.getSelectionModel();
      
      wpScroller = new JScrollPane();
      wpScroller.getViewport().add(wpGroups);
      wpScroller.setBorder(new TitledBorder
         (new EtchedBorder(EtchedBorder.RAISED), " Defined Groups "));
      
      wpScroller.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      west.add(wpScroller, BorderLayout.CENTER);
      
      wpCtlPanel = new JPanel();
      west.add(wpCtlPanel, BorderLayout.SOUTH);

      // wpCtlPanel panel is controlled with GroupLayout
      //  +------wpCtlPanel--------+
      //  |                        |
      //  |                        |
      //  |     buttons here       |   
      //  |                        |
      //  |                        |
      //  +------------------------+
      
      // creating control widgets
      wpbR = new JButton("Refresh");
      wpbA = new JButton("Add Group");
      wpbD = new JButton("Delete Selected");
      wpbC = new JButton("Clone Group");
      wpbE = new JButton("Edit Group");
      wpbS = new JButton("Switch to ACLs");
      
      // create layout of the wpCtlPanel panel with all controls
      GroupLayout wpCtlLayout = new GroupLayout(wpCtlPanel);
      wpCtlPanel.setLayout(wpCtlLayout);
    
      wpCtlLayout.setAutoCreateGaps(true);
      wpCtlLayout.setAutoCreateContainerGaps(true);
    
      // Create a sequential group for the horizontal axis:
      {
         GroupLayout.SequentialGroup hGroup = 
            wpCtlLayout.createSequentialGroup();
         
         // parallel group of Refresh and Clone buttons
         hGroup.addGroup(
            wpCtlLayout.createParallelGroup().
            // Refresh button
            addComponent(wpbR).
            // Clone group button
            addComponent(wpbC)
         );
         
         // parallel group of Add and Edit buttons
         hGroup.addGroup(
            wpCtlLayout.createParallelGroup().
            // Add button
            addComponent(wpbA).
            // Edit group button
            addComponent(wpbE)
         );
         
         // parallel group of Delete and Switch buttons
         hGroup.addGroup(
            wpCtlLayout.createParallelGroup().
            // Delete Selected button
            addComponent(wpbD).
            // Switch to ACLs button
            addComponent(wpbS)
         );
         
         wpCtlLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a sequential group for the vertical axis
      {
         GroupLayout.SequentialGroup vGroup = wpCtlLayout.
                                              createSequentialGroup();
         vGroup.
            // parallel group of three buttons
            addGroup(wpCtlLayout.createParallelGroup(
                                              GroupLayout.Alignment.BASELINE).
               // Refresh button
               addComponent(wpbR).
               // Add button
               addComponent(wpbA).
               // Delete Selected button
               addComponent(wpbD)
               ).
            // parallel group of three more buttons
            addGroup(wpCtlLayout.createParallelGroup(
                                              GroupLayout.Alignment.BASELINE).
               // Clone button
               addComponent(wpbC).
               // Edit button
               addComponent(wpbE).
               // Switch to ACLs button
               addComponent(wpbS)
               )
            ;
            
         wpCtlLayout.setVerticalGroup(vGroup);
      }
      
      wpCtlLayout.linkSize(wpbR, wpbC);
      wpCtlLayout.linkSize(wpbA, wpbE);
      wpCtlLayout.linkSize(wpbD, wpbS);
                           
      // create "Clone as" modal dialog panel with Group layout
      wpClonePanel = new JPanel();
      
      wplN = new JLabel("New Name");
      wpfN = new JTextField(12);
      wplW = new JLabel("With ACLs");
      wpcW = new JCheckBox();
      wpbG = new JButton("Clone Group");

      wpClonePanel.setBorder(new TitledBorder
         (new EtchedBorder(EtchedBorder.RAISED), " Clone Selected Group As "));
      GroupLayout wpCloneLayout = new GroupLayout(wpClonePanel);
      wpClonePanel.setLayout(wpCloneLayout);
    
      wpCloneLayout.setAutoCreateGaps(true);
      wpCloneLayout.setAutoCreateContainerGaps(true);
    
      // Create a sequential group for the horizontal axis.
      {
         GroupLayout.SequentialGroup hGroup = 
            wpCloneLayout.createSequentialGroup();
       
         hGroup.
            addComponent(wplN).
            addComponent(wpfN).
            addGroup(wpCloneLayout.createParallelGroup().
               addGroup(wpCloneLayout.createSequentialGroup().
                  addComponent(wplW).
                  addComponent(wpcW)
                  ).
               addComponent(wpbG)
               );
         wpCloneLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a sequential group for the vertical axis.
      {
         GroupLayout.SequentialGroup vGroup = wpCloneLayout.
                                              createSequentialGroup();
       
         vGroup.addGroup(wpCloneLayout.createParallelGroup(
                                       GroupLayout.Alignment.BASELINE).
            addComponent(wplN).
            addComponent(wpfN).
            addComponent(wplW).
            addComponent(wpcW)
            ).
            addComponent(wpbG);
         wpCloneLayout.setVerticalGroup(vGroup);
      }
 
      // stuffing center panel with components
      // center panel is controlled with BorderLayout
      center.setLayout(new BorderLayout());

      //  +--------center----------+
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    cpScroller      | |
      //  | |      CENTER        | |
      //  | |                    | |
      //  | |                    | |
      //  | +--------------------+ |
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    cpCtlPanel      | |
      //  | |      SOUTH         | |
      //  | +--------------------+ |
      //  +------------------------+
      
      cpUtm = new TaggedDataTableModel("User ID", "Person");
      cpUsers = new JTable(cpUtm);
      cpUsers.getTableHeader().setReorderingAllowed(false);
      cpUsm = cpUsers.getSelectionModel();

      cpScroller = new JScrollPane();
      cpScroller.getViewport().add(cpUsers);
      cpScroller.setBorder(new TitledBorder
         (new EtchedBorder(EtchedBorder.RAISED), " Users in Group "));
      cpScroller.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      center.add(cpScroller, BorderLayout.CENTER);

      cpCtlPanel = new JPanel();
      center.add(cpCtlPanel, BorderLayout.SOUTH);
      
      // create control widgets
      cpfN = new JTextField(12);
      cpbA = new JButton("Add User");
      cpbD = new JButton("Delete Selected");
      
      // create "Add user" pane
      cpAddPanel = createGroup(" Add User ", cpfN, cpbA);
      
      // create layout of the cpCtlPanel panel with all controls
      GroupLayout cpCtlLayout = new GroupLayout(cpCtlPanel);
      cpCtlPanel.setLayout(cpCtlLayout);
    
      cpCtlLayout.setAutoCreateGaps(true);
      cpCtlLayout.setAutoCreateContainerGaps(true);
  
      // Create a parallel group for the horizontal axis
      {
         GroupLayout.ParallelGroup hGroup = 
            cpCtlLayout.createParallelGroup(GroupLayout.Alignment.TRAILING);
       
         hGroup.
            // Delete Selected button
            addComponent(cpbD).
            // Add user pane
            addComponent(cpAddPanel)
            ;
         cpCtlLayout.setHorizontalGroup(hGroup);
      }
    
      // Create a sequential group for the vertical axis:
      {
         GroupLayout.SequentialGroup vGroup = 
            cpCtlLayout.createSequentialGroup();
       
         vGroup.
            // Delete Selected button
            addComponent(cpbD).
            // Add user pane
            addComponent(cpAddPanel)
            ;
         cpCtlLayout.setVerticalGroup(vGroup);
      }

      // create "Group Def" modal dialog panel with Group layout
      cpGroupPanel = new JPanel();

      cplG = new JLabel(" Group Name ");
      cpfG = new JTextField(12);
      cplD = new JLabel(" Description ");
      cpfD = new JTextField(32);
      cpbC = new JButton("Cancel");
      cpbS = new JButton("Save");

      AdminUtils.busy(client);
      authElements = new AuthModeComboBoxes(adm, 
                        AuthModeComboBoxes.TargetScreen.GROUP_ACCOUNT_SCREEN);
      AdminUtils.idle(client);

      JPanel cpGroupPanelContent = new JPanel();
      cpGroupPanelContent.setBorder(new TitledBorder
            (new EtchedBorder(EtchedBorder.RAISED), " Group Definition "));
      GroupLayout cpGroupLayout = new GroupLayout(cpGroupPanelContent);
      cpGroupPanelContent.setLayout(cpGroupLayout);

      // Turn on automatically adding gaps
      cpGroupLayout.setAutoCreateGaps(true);
      cpGroupLayout.setAutoCreateContainerGaps(true);
    
      // Create a sequential group for the horizontal axis.
      {
         GroupLayout.SequentialGroup hGroup = cpGroupLayout.
                                              createSequentialGroup();
       
         hGroup.addGroup(cpGroupLayout.createParallelGroup().
            addComponent(cplG).
            addComponent(cplD).
            addComponent(authElements.getAuthModeLabel()).
            addComponent(authElements.getAuthPluginsLabel())
            );
         hGroup.addGroup(cpGroupLayout.createParallelGroup().
            addComponent(cpfG).
            addComponent(cpfD).
            addComponent(authElements.getAuthModeCombo()).
            addComponent(authElements.getAuthPluginsCombo())
            );
         
         cpGroupLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a sequential group for the vertical axis.
      {
         GroupLayout.SequentialGroup vGroup = cpGroupLayout.
                                              createSequentialGroup();
       
         vGroup.addGroup(cpGroupLayout.createParallelGroup(GroupLayout.
                                                          Alignment.BASELINE).
            addComponent(cplG).
            addComponent(cpfG)
            );
         vGroup.addGroup(cpGroupLayout.createParallelGroup(GroupLayout.
                                                          Alignment.BASELINE).
            addComponent(cplD).
            addComponent(cpfD)
            );
         vGroup.addGroup(cpGroupLayout.createParallelGroup(GroupLayout.
                                                          Alignment.BASELINE).
            addComponent(authElements.getAuthModeLabel()).
            addComponent(authElements.getAuthModeCombo())
            );
         vGroup.addGroup(cpGroupLayout.createParallelGroup(GroupLayout.
                                                          Alignment.BASELINE).
            addComponent(authElements.getAuthPluginsLabel()).
            addComponent(authElements.getAuthPluginsCombo())
            );
         
         cpGroupLayout.setVerticalGroup(vGroup);
      }

      JPanel buttons = AdminUtils.groupComponents(
                         new Component[][]{{cpbC, cpbS}});
      cpGroupPanel = AdminUtils.groupComponents(
                         new Component[][]{{cpGroupPanelContent}, {buttons}});
      ((GroupLayout) cpGroupPanel.getLayout()).
                         setAutoCreateContainerGaps(false);

      // stuffing east panel with components
      // east panel is controlled with BorderLayout
      east.setLayout(new BorderLayout());
      
      //  +---------east-----------+
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    epScroller      | |
      //  | |      CENTER        | |
      //  | |                    | |
      //  | |                    | |
      //  | +--------------------+ |
      //  | +--------------------+ |
      //  | |                    | |
      //  | |    epCtlPanel      | |
      //  | |      SOUTH         | |
      //  | +--------------------+ |
      //  +------------------------+
      
      epUtm = new TaggedDataTableModel("User ID", "Person");
      epUsers = new JTable(epUtm);
      epUsers.getTableHeader().setReorderingAllowed(false);
      epUsm = epUsers.getSelectionModel();
      epSorter = new TableRowSorter<TaggedDataTableModel>(epUtm);
      epSorter.setRowFilter(null);
      epUsers.setRowSorter(epSorter);
      
      epScroller = new JScrollPane();
      epScroller.getViewport().add(epUsers);
      epScrollTitleA = new TitledBorder(new EtchedBorder(EtchedBorder.RAISED), 
                                       " All Users ");
      epScrollTitleF = new TitledBorder(new EtchedBorder(EtchedBorder.RAISED), 
                                       " All Users by Filter");
      epScroller.setBorder(epScrollTitleA);
      epScroller.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      east.add(epScroller, BorderLayout.CENTER);
      
      epCtlPanel = new JPanel();
      east.add(epCtlPanel, BorderLayout.SOUTH);

      // create control widgets
      epbP = new JButton("Populate");
      epbA = new JButton("Add Selected");
      
      epfS = new JTextField(12);
      epbF = new JButton("Filter");

      // create upper pane
      JPanel epUpPane = new JPanel();
      epUpPane.setLayout(new BoxLayout(epUpPane, BoxLayout.LINE_AXIS));
      epUpPane.add(epbP);
      epUpPane.add(Box.createHorizontalGlue());
      epUpPane.add(epbA);
      
      // create "Search" pane
      epSearchPanel = createGroup(" Search ", epfS, epbF);

      // create layout of the epCtlPanel panel with all controls:
      // buttons and the "search" sub-panel
      GroupLayout epCtlLayout = new GroupLayout(epCtlPanel);
      epCtlPanel.setLayout(epCtlLayout);
    
      epCtlLayout.setAutoCreateGaps(true);
      epCtlLayout.setAutoCreateContainerGaps(true);
    
      // Create a parallel group for the horizontal axis
      {
         GroupLayout.ParallelGroup hGroup = 
            epCtlLayout.createParallelGroup();
       
         hGroup.
            addComponent(epUpPane).
            addComponent(epSearchPanel)
            ;
         epCtlLayout.setHorizontalGroup(hGroup);
      }
      
      // Create a sequential group for the vertical axis
      {
         GroupLayout.SequentialGroup vGroup = 
            epCtlLayout.createSequentialGroup();
       
         vGroup.
            addComponent(epUpPane).
            addComponent(epSearchPanel)
            ;
         epCtlLayout.setVerticalGroup(vGroup);
      }

      // create event handlers: west panel

      // Groups table selection events
      wpGsm.addListSelectionListener(new ListSelectionListener() 
      {
         @Override
         public void valueChanged(ListSelectionEvent evt) 
         {
            handleGroupsSelection(evt);
         }
      });

      // Populate button
      wpbR.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handlePopulateGroups();
         }
      });

      // Add button
      wpbA.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleAddGroup();
         }
      });

      // Delete Selected button
      wpbD.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleDeleteSelectedGroups();
         }
      });

      // Edit group button
      wpbE.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleEditGroup();
         }
      });

      // Switch to ACLs button
      wpbS.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleSwitch();
         }
      });

      // Clone group button
      wpbC.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleClone();
         }
      });

      // Clone group button in the modal dialog
      wpbG.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleCloneAs();
         }
      });

      // create event handlers: center panel

      // users table selection events
      cpUsm.addListSelectionListener(new ListSelectionListener() 
      {
         @Override
         public void valueChanged(ListSelectionEvent evt) 
         {
            handleUsersSelection(evt);
         }
      });

      // Add user button
      cpbA.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleAddUser();
         }
      });

      // Delete selected button
      cpbD.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleDeleteSelectedUsers();
         }
      });

      // Cancel editing button
      cpbC.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleCancel();
         }
      });

      // Save definition button
      cpbS.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleSave();
         }
      });

      // create event handlers: east panel

      // all users table selection events
      epUsm.addListSelectionListener(new ListSelectionListener() 
      {
         @Override
         public void valueChanged(ListSelectionEvent evt) 
         {
            handleAllUsersSelection(evt);
         }
      });

      // Populate button
      epbP.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handlePopulateAllUsers();
         }
      });

      // Add selected button
      epbA.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleAddSelectedUsers();
         }
      });

      // Filter button
      epbF.addActionListener(new ActionListener() 
      {
         @Override
         @SuppressWarnings("unused")
         public void actionPerformed(ActionEvent evt) 
         {
            handleFilter();
         }
      });
      
      // create the array of components for the state machine
      state = new JComponent[]
      {
         west,
            wpScroller,
               wpGroups,
            wpCtlPanel,
               wpbR, wpbA, wpbD,
               wpbC, wpbE, wpbS,
         center,
            cpScroller,
               cpUsers,
            cpCtlPanel,
               cpfN, cpbA, cpbD,
          east,
             epScroller,
                epUsers,
             epCtlPanel,
                epbP, epbA,
                epSearchPanel,
                   epfS, epbF
      };
      
      // set initial state of all components
      stateMachine(STATE_INITIAL_EMPTY);
      
      // populate the groups list upfront
      handlePopulateGroups();
   }

   /**
    * Creates print preview panel off the current screen data.
    * This screen prints the current view of the groups table.
    */
   public void print()
   {
      // check the groups table selection
      int[] sel = wpGroups.getSelectedRows();
      int nsel = sel.length;
      int nall = wpGtm.getRowCount();
      final boolean selectOnly;
      
      // no selection means the whole table, otherwise there is a choice
      if (nsel == 0 || nsel == nall)
      {
         // printing the whole table confirmation
         String quest = String.format(
                        "%d group accounts being printed. Continue?",
                        nall);
         if (!AdminUtils.confirmDialog(client, "Printing Groups", quest))
         {
            // declined
            return;
         }
         
         // print all
         selectOnly = false;
      }
      else
      {
         // offering a choice 
         String quest = "Please refine your request";
         String[] choices = new String[2];
         choices[0] = nsel == 1 ? "the selected group account" :
                      String.format("%d selected group accounts", nsel);
         choices[1] = nall == 2 ? "both group accounts" :
                      String.format("all %d group accounts", nall);
         
         int n = AdminUtils.choiceDialog(client, "Printing Groups", quest,
                                         choices, "Print");
         if (n < 0)
         {
            // declined
            return;
         }
         
         // print mode choice
         selectOnly = n == 0;
      }
      
      // busy
      AdminUtils.busy(client);
      
      // build preview main table
      List<GroupInfo> backGl  = new LinkedList<GroupInfo>();
      
      for (int i = 0; i < nall; i++)
      {
         // translate model to view
         int iv = wpGroups.convertRowIndexToView(i);
         
         // honor selection if asked
         if (selectOnly && !wpGsm.isSelectedIndex(iv))
         {
            continue;
         }
         
         // stuff the back list with the group info
         GroupInfo backGi = new GroupInfo();
         backGi.gid    = (String)wpGtm.getValueAt(i, 0);
         backGi.descr  = (String)wpGtm.getValueAt(i, 1);
         
         // get the assigned users
         TaggedName[] list = adm.listGroupUsers(backGi.gid);
         if (list == null)
         {
            // error retrieving group users
            backGi.uid    = "*** error ***";
            backGi.person = "*** error ***";
            backGl.add(backGi);
            continue;
         }
         
         backGi.uid    = "";
         backGi.person = "";
         for (int j = 0; j < list.length; j++)
         {
            backGi.uid    += list[j].getName();
            backGi.person += list[j].getTag();
            
            if (j < list.length - 1)
            {
               backGi.uid    += ",";
               backGi.person += ",";
            }
         }
         
         backGl.add(backGi);
      }
      
      header = new MessageFormat(
             selectOnly ? "Selected Group Accounts" : "All Group Accounts");
      footer = new MessageFormat("page {0, number}");
               
      backTm = new PrintGroupsModel(backGl.toArray(new GroupInfo[] {}));
      backTable = new JTable(backTm);
      
      // idle
      AdminUtils.idle(client);
      
      // display print preview
      client.printOn(backTable, backTm, header, footer);
   }
   
   /**
    * Queries the extended printing capability. If the extended capability is
    * not provided, the printExtended() method should not be called.
    * 
    * @return   <code>true</code> to tell the caller the extended printing is
    *           implemented
    */
   public boolean canPrintExtended()
   {
      return true;
   }

   /**
    * Creates extended print preview table off the current screen data.
    * This screen prints the current view of the groups table.
    */
   public void printExtended()
   {
      // busy
      AdminUtils.busy(client);
      
      frontTm = new PrintGroupsModel(new GroupInfo[] {});
      frontTable = new JTable(frontTm);
      
      refreshView(backTable, backTm, (PrintGroupsModel)frontTm);
      
      // idle
      AdminUtils.idle(client);
      
      client.printExtendedOn(frontTable, header, footer);
   }
      
   /** 
    * Presents the refreshed group accounts management screen.
    */
   void manageGroups() 
   {
      // switch to this screen without changing state of components
      client.show(SCREEN_ACC_GROUPS, TITLE_ACC_GROUPS);
   }
   
   /** 
    * Changes state of the components in response to groups list selections.
    * There are three distinctive states:
    * <ul>
    *   <li>no selection or empty list
    *   <li>single entry selected
    *   <li>multiple entries selected
    * </ul>
    * This method determines the selection state of the list and changes
    * the state of the components accordingly.
    * <p>
    * No selection means that the state should be STATE_INITIAL_EMPTY.
    * Only the "Populate" and "Add group" buttons are enabled.
    * <p>
    * Single selection means that all buttons of the west panel 
    * and "Clone group" sub-panel are enabled, since all these operations make
    * sense for a single selected group.<br>
    * Additionally, the center panel's list of users gets populated with 
    * the users that belong to the selected group. The related  
    * "Add user" button is enabled, too.<br>
    * At last, the east panel, if it is populated, shows the selection
    * of users from the center panel.
    * <p>
    * Multiple selection means that buttons, specific to the single selection,
    * are disabled. The enabled buttons on the west panel are Populate,
    * Add group and Delete selected.<br>
    * The center panel is fully disabled and the east panel's list of all
    * users has no selection and is disabled.
    *
    * @param    evt
    *           event caused this method call
    */
   private void handleGroupsSelection(ListSelectionEvent evt) 
   {
      // do not bother until the dust is fully settled
      if (evt.getValueIsAdjusting())
      {
         return;
      }

      // figure out the selection state
      if (wpGsm.isSelectionEmpty())
      {
         // no selection
         setGroupsNoSelection();
      }
      else if (wpGroups.getSelectedRows().length == 1)
      {
         // single selection
         setGroupsSingleSelection();
      }
      else
      {
         // multiple selection
         setGroupsMultiSelection();
      }
   }

   /**
    * Handle Populate button press in the west panel.
    * <p>
    * Populate button fills the list of groups with the information from
    * the server and enables the groups list component for selections.
    * It also clears the users list of the center panel and any selection in
    * the all users list of the east panel.
    */
   private void handlePopulateGroups() 
   {
      // populate the groups list
      boolean nonEmpty = populateGroupsList();

      // set the state accordingly
      setGroupsNoSelection(nonEmpty);
   }
   
   /**
    * Handle Populate button press in the east panel.
    * <p>
    * Populate button fills the list of all users with the information from
    * the server and enables the all users list component for selections.
    * <p>
    * If the user list is non-empty, the search sub-panel gets enabled as well.
    * <p>
    * If there is a single selected group and the user list is non-empty,
    * the list selection is set to reflect the existing users of the group.
    */
   private void handlePopulateAllUsers() 
   {
      // populate the all users list
      boolean nonEmpty = populateAllUsersList();
      epbP.setText("Refresh");

      // set the state accordingly
      if (nonEmpty)
      {
         // set enabled state of the components
         stateMachine(MASK_EAST, STATE_ALL_USERS);
       
         // check for the single selection in Groups
         if (wpGroups.getSelectedRows().length == 1)
         {
            setAllUsersSelection();
         }
      }
   }
   
   /**
    * Handle Add button press in the west panel.
    * The Group definition panel is enabled and the rest of the west and
    * center panels are disabled till either Cancel or Save button press.
    */
   private void handleAddGroup() 
   {
      // clear the panel
      resetGroupPanel();
               
      // state change to enable a semi-modal group definition creation
      modal = stateMachineModal(MASK_WEST_CENTER, STATE_GROUP_ADD);
      
      // focus on name
      cpfG.requestFocusInWindow();

      // display modal dialog
      dialog = AdminUtils.modalDialog(client, "Group Definition", cpGroupPanel);
      dialog.setVisible(true);

      // restore state
      if (dialog != null)
      {
         // dialog dismissed by closing window
         handleCancel();
      }
   }

   /**
    * Handle Delete Selected button press in the west panel.
    */
   private void handleDeleteSelectedGroups() 
   {
      // get the selection
      int[] sel = wpGroups.getSelectedRows();
      
      // safety check
      if (sel.length == 0)
      {
         return;
      }
      
      // ask confirmation
      String quest = sel.length == 1 ?
                     "1 group account being deleted. Continue?" :
                     Integer.toString(sel.length) + 
                     " group accounts being deleted. Continue?";
      
      if (!AdminUtils.confirmDialog(client, "Deleting Groups", quest))
      {
         // declined
         return;
      }
      
      // busy
      AdminUtils.busy(client);
      
      // sort the array of selected indices to walk backwards
      Arrays.sort(sel);
      
      // walk the array backwards and delete selected groups
      for (int i = sel.length - 1; i >= 0; i--)
      {
         // translate view to model
         int sele = wpGroups.convertRowIndexToModel(sel[i]);
         
         // pick up the name of group being deleted
         String group = (String)wpGtm.getValueAt(sele, 0);
         
         // attempt deletion
         boolean rc = adm.deleteGroup(group);
         if (!rc)
         {
            // attempt has failed
            client.alarm();
         }
         else
         {
            // attempt has succeeded; delete this row
            client.ok();
            wpGtm.deleteRow(sele);
         }
      }
      
      // idle
      AdminUtils.idle(client);
   }

   /**
    * Handle Edit group button press in the west panel.
    */
   private void handleEditGroup() 
   {
      // since the dialog might have been dismissed since the last use,
      // reinitialize its fields
      cpfG.setText(groupName);
      cpfD.setText(editingGroup);
      authElements.setAuthMode(editingAuthMode);
      authElements.setAuthPlugin(editingAuthPlugin);
      
      // state change to enable a semi-modal group definition creation
      modal = stateMachineModal(MASK_WEST_CENTER, STATE_GROUP_EDIT);

      // focus on description
      cpfD.requestFocusInWindow();
      
      // display modal dialog
      dialog = AdminUtils.modalDialog(client, "Group Definition", cpGroupPanel);
      cpfG.setEnabled(false);
      dialog.setVisible(true);
      cpfG.setEnabled(true);

      // restore state
      if (dialog != null)
      {
         // dialog dismissed by closing window
         handleCancel();
      }
   }

   /**
    * Handle Switch to ACLs button press in the west panel.
    * <p>
    * This method should passes the group name to the ACLs screen and initiate 
    * Access Control --&gt; Show By Account menu choice
    */
   private void handleSwitch() 
   {
      client.show(AdminACL.SCREEN_ACLS, AdminACL.TITLE_ACLS);
      AdminACL adminAcl = client.getMenu().getACL().getAclAdmin();
      
      adminAcl.showByAccount(groupName);
   }

   /**
    * Handle Clone group button press in the west panel.
    */
   private void handleClone() 
   {
      // display modal dialog
      dialog = AdminUtils.modalDialog(client, "Clone Group", wpClonePanel);
      dialog.setVisible(true);

      // restore state
      if (dialog != null)
      {
         // dialog dismissed by closing window
         dialog = null;
      }
   }

   /**
    * Handle Clone button press in the Clone As modal dialog
    */
   private void handleCloneAs() 
   {
      // get the source info from selection
      String sourceName = (String)wpGtm.
                          getValueAt(wpGroups.getSelectedRow(), 0);
      String desc = (String)wpGtm.getValueAt(wpGroups.getSelectedRow(), 1);
      
      // get and check the target group name
      String targetName = validateStringField(wpfN, "type clone's group name");
      
      if (targetName == null)
      {
         return;
      }
      
      // get the ACL option
      boolean withACLs = wpcW.isSelected();
      
      // clone the group
      AdminUtils.busy(client);
      boolean rc = adm.cloneGroup(sourceName, targetName, withACLs);
      AdminUtils.idle(client);
      
      // check the result
      if (!rc)
      {
         // set initial focus on the group name field
         wpfN.requestFocusInWindow();

         // alarm
         client.alarm();
         return;
      }
      
      // cloned; clear panel
      client.ok();
      resetClonePanel();
      dialog.dispose();
      dialog = null;
      
      // added to the server; clear selection and add to the list
      wpGsm.clearSelection();
      wpGtm.addRow(new TaggedName(targetName, desc));
   }

   /** 
    * Changes state of the components in response to users list selections.
    * There are two distinctive states:
    * <ul>
    *   <li>no selection or empty list
    *   <li>single or multiple selection
    * </ul>
    * This method determines the selection state of the list and changes
    * the state of the components accordingly.
    * <p>
    * No selection means that the Delete Selected button should be disabled.
    * Otherwise it should be enabled.
    *
    * @param    evt
    *           event caused this method call
    */
   private void handleUsersSelection(ListSelectionEvent evt) 
   {
      // do not bother until the dust is fully settled
      if (evt.getValueIsAdjusting())
      {
         return;
      }

      // figure out the selection state
      if (cpUsm.isSelectionEmpty())
      {
         // no selection
         setUsersNoSelection();
      }
      else
      {
         // some selection
         setUsersAnySelection();
      }
   }
   
   /** 
    * Changes state of the components in response to all users list 
    * selections.
    * The idea is to keep all assigned users of the selected group also
    * in the selected state in the all users list. If extra lines are
    * selected, the Add selected button gets enabled. If some pre-selected
    * lines are deselected, they are reselected to maintain the status.
    *
    * @param   evt
    *          event caused this method call
    */
   private void handleAllUsersSelection(ListSelectionEvent evt) 
   {
      // do not bother until the dust is fully settled
      if (evt.getValueIsAdjusting())
      {
         return;
      }

      if (selecting)
      {
         return;
      }
      selecting = true;
      
      // check the group selection type
      if (!singleGroup)
      {
         epUsm.clearSelection();
         selecting = false;
         
         // disable Add selected button
         epbA.setEnabled(false);
         return;
      }
      
      // get the changed selection range
      int fi = evt.getFirstIndex();
      int ti = evt.getLastIndex();
      
      // inspect the change in selection line by line
      boolean extra = false;
      
      for (int i = (fi < 0 ? 0 : fi); i <= ti; i++)
      {
         // check the selection status
         boolean sel = epUsm.isSelectedIndex(i);
         String user = (String)epUsers.getValueAt(i, 0);
         
         if (sel)
         {
            // this line may have been selected
            if (!cpUtm.isListed(user))
            {
               // extra selection
               extra = true;
            }
            // else do nothing
         }
         else
         {
            // this line may have been deselected
            if (cpUtm.isListed(user))
            {
               // should be reselected
               epUsers.addRowSelectionInterval(i, i);
            }
            else
            {
               // an extra line was deselected
               // rescan the whole selection to detect extra lines
               int[] all = epUsers.getSelectedRows();
               extra = false;
               
               for (int j = 0; j < all.length; j++)
               {
                  if (!cpUtm.isListed((String)epUsers.getValueAt(all[j], 0)))
                  {
                     extra = true;
                  }
               }
            }
         }
      }
      
      // enable or disable Add selected button
      epbA.setEnabled(extra);

      selecting = false;
   }
   
   /**
    * Handle Add user button press in the center panel.
    */
   private void handleAddUser() 
   {
      // clear selection if any
      cpUsm.clearSelection();

      // check the user name field
      String newUser = cpfN.getText().trim();
      
      if (newUser.length() == 0)
      {
         // set initial focus on the group name field
         cpfN.requestFocusInWindow();
         
         // alarm
         client.beep("type name of a user");
         return;
      }
      
      // busy
      AdminUtils.busy(client);
      
      // add this user to the group
      boolean rc = adm.changeGroupUsers(groupName, new String[] {newUser}, 
                                        new String[0]);
      
      if (!rc)
      {
         client.alarm();
         
         // idle
         AdminUtils.idle(client);
         return;
      }
      
      // added; get the user info from the server and add it to the table
      client.ok();
      TaggedName ui = adm.checkUser(newUser);
      cpUtm.addRow(ui);
      
      // clear input field
      cpfN.setText("");
      
      // position the cursor for possible continuation
      cpfN.requestFocusInWindow();
      
      // set selection in all users accordingly
      setAllUsersSelection();

      // idle
      AdminUtils.idle(client);
   }
               
   /**
    * Handle Delete selected button press in the center panel.
    */
   private void handleDeleteSelectedUsers() 
   {
      // get the selection
      int[] sel = cpUsers.getSelectedRows();
      
      // safety check
      if (sel.length == 0)
      {
         return;
      }
      
      // ask confirmation
      String quest = sel.length == 1 ?
                     "1 user account being unassigned. Continue?" :
                     Integer.toString(sel.length) + 
                     " user accounts being unassigned. Continue?";
      
      if (!AdminUtils.confirmDialog(client, "Unassigning Users", quest))
      {
         // declined
         return;
      }
      
      // busy
      AdminUtils.busy(client);
      
      // sort the array of selected indices to walk backwards
      Arrays.sort(sel);
      
      // copy user names
      String[] selNames = new String[sel.length];
      
      // walk the array backwards and add selected users to an array 
      for (int i = sel.length - 1; i >= 0; i--)
      {
         // gather the names of users being deleted
         selNames[i] = (String)cpUtm.getValueAt((sel[i]), 0);
         
         // delete this row
         cpUtm.deleteRow(sel[i]);
      }
      
      // delete users from the group on the server
      boolean rc = adm.changeGroupUsers(groupName, new String[0], selNames);
      
      if (!rc)
      {
         // attempt failed; restore the table
         client.alarm();
         populateUsersList(groupName);
         
         // idle
         AdminUtils.idle(client);
         return;
      }
      
      // refresh the selection in the all users list
      client.ok();
      setAllUsersSelection();

      // idle
      AdminUtils.idle(client);
   }
               
   /**
    * Handle Add selected button press in the east panel.
    */
   private void handleAddSelectedUsers() 
   {
      // scan the whole selection to detect extra lines
      int[] all = epUsers.getSelectedRows();
      
      java.util.List<String> extra = new LinkedList<String>();
      
      for (int j = 0; j < all.length; j++)
      {
         int k = all[j];
         
         String user = (String)epUsers.getValueAt(k, 0);
         if (!cpUtm.isListed(user))
         {
            // add to the list for the server
            extra.add(user);
            
            // add to the table
            TaggedName tn = new TaggedName(user, 
                                           (String)epUsers.getValueAt(k, 1));
            cpUtm.addRow(tn);
         }
      }
      
      // process the additions, if any
      if (!extra.isEmpty())
      {
         String[] additions = extra.toArray(new String[] {});
         
         AdminUtils.busy(client);
         boolean rc = adm.changeGroupUsers(groupName, additions, 
                                           new String[0]);
         AdminUtils.idle(client);
      
         if (!rc)
         {
            client.alarm();
            
            // refresh the users list of center panel
            populateUsersList(groupName);
            return;
         }
         
      }
      
      // disable Add selected button
      client.ok();
      epbA.setEnabled(false);
   }
   
   /**
    * Handle Cancel button press in the center panel.
    * Cancel button terminates the group definition editing, clears inputs
    * and returns to the saved state of the components. 
    */
   private void handleCancel() 
   {
      // determine the mode: add vs edit
      boolean add = cpfG.isEnabled();

      // dispatch the action
      if (add)
      {
         // clear panel on cancel add
         resetGroupPanel();
      }
      else
      {
         // restore panel on cancel edit
         cpfD.setText(editingGroup);
         authElements.setAuthMode(editingAuthMode);
         authElements.setAuthPlugin(editingAuthPlugin);
      }
      
      // hide the dialog
      dialog.setVisible(false);
      dialog = null;

      // restore saved state
      stateMachine(modal);
      
      // enable the groups list explicitly if non-empty
      wpGroups.setEnabled(wpGtm.getRowCount() > 0);
   }
               
   /**
    * Handle Save button press in the center panel.
    * Save button tries and adds the group definition.
    * If the server responds OK, it adds the group name to the groups list
    * and clears editing for the new definition. Otherwise, the editing
    * continues after an alarm.
    * Editing mode is terminated with Cancel button.
    */
   private void handleSave() 
   {
      // determine the mode: add vs edit
      boolean add = cpfG.isEnabled();
      
      // get and check the group name
      String name = validateStringField(cpfG, "type a group name");
      
      if (name == null)
      {
         return;
      }
      
      // get and check the group description
      String desc = validateStringField(cpfD, "type a group description");
      
      if (desc == null)
      {
         return;
      }

      int authMode = authElements.getAuthMode();
      String authPlugin = authElements.getAuthPlugin();

      // dispatch the action
      if (add)
      {
         // add the definition to the server and iterate
         // list remains with no selection
         if (addGroupDef(name, desc, authMode, authPlugin))
         {
            // successfully added to the server; add to the list
            client.ok();
            wpGtm.addRow(new TaggedName(name, desc));

            // clear selection now when the table is modified
            wpGsm.clearSelection();
            
            // refocus on group name field
            cpfG.requestFocusInWindow();
            return;
         }
         
         // adding definition failed; stay in the editing
         cpfG.requestFocusInWindow();
         client.alarm();
      }
      else
      {
         // change the existing definition
         // list retains the selection
         if (setGroupDef(name, desc, authMode, authPlugin))
         {
            // successfully saved changes
            client.ok();
            editingGroup = desc;
            editingAuthMode = authMode;
            editingAuthPlugin = authPlugin;
            wpGtm.refreshRow(new TaggedName(name, desc),
                              wpGroups.getSelectedRow());
            
            // restore saved state
            stateMachine(modal);
            
            // enable the groups list explicitly if non-empty since
            // state machine doesn't do that
            wpGroups.setEnabled(wpGtm.getRowCount() > 0);

            // hide the dialog
            dialog.setVisible(false);
            dialog = null;
            
            return;
         }
         
         // saving definition failed; stay in the editing 
         cpfD.requestFocusInWindow();
         client.alarm();
      }
   }
   
   /**
    * Handle Filter button press in the east panel.
    * When pressed with some contents in the Search text field, the text
    * becomes the filter. Otherwise the existing filter is removed.
    */
   @SuppressWarnings("unchecked")
   private void handleFilter() 
   {
      // clear selection first
      selecting = true;
      epUsm.clearSelection();
      
      // get and check the filter value
      String filter = epfS.getText().trim();
      
      if (filter.length() == 0)
      {
         // remove filtering
         epSorter.setRowFilter(null);
         epScroller.setBorder(epScrollTitleA);
      }
      else
      {
         // install a filter
         epScroller.setBorder(epScrollTitleF);
         epSorter.setRowFilter(AdminUtils.createFilter(filter));
      }
      
      // reshow selection
      selecting = false;
      setAllUsersSelection();
   }
   
   /**
    * Populate the list of groups with fresh data from the server.
    * 
    * @return   <code>true</code> if the list is non-empty
    */
   private boolean populateGroupsList() 
   {
      // clear selection if any
      wpGsm.clearSelection();
      
      // clear the old list
      boolean rc = false;
      wpGtm.clear();
      
      // get the list of groups from the server
      AdminUtils.busy(client);
      TaggedName[] list = adm.listGroups();
      AdminUtils.idle(client);
      
      if (list != null)
      {
         if (list.length > 0)
         {
            // list is non-empty
            rc = true;
            wpGtm.loadAll(list);
         }
      }
      else
      {
         client.alarm();
      }
      
      return rc;
   }
   
   /**
    * Populate the list of users for a given group with fresh data from the
    * server.
    * 
    * @param    groupName
    *           name of the target group
    * 
    * @return   <code>true</code> if the list is non-empty
    */
   private boolean populateUsersList(String groupName) 
   {
      boolean rc = false;
      
      // clear existing records
      cpUtm.clear();
      
      // populate the list of group users
      AdminUtils.busy(client);
      TaggedName[] list = adm.listGroupUsers(groupName);
      AdminUtils.idle(client);
      
      if (list != null)
      {
         if (list.length > 0)
         {
            // list is non-empty
            rc = true;
            cpUtm.loadAll(list);
         }
      }
      else
      {
         client.alarm();
      }
      
      return rc;
   }
   
   /**
    * Populate the list of all defined users.
    * 
    * @return   <code>true</code> if the list is non-empty
    */
   private boolean populateAllUsersList() 
   {
      boolean rc = false;
      
      // clear existing selection if any
      epUsm.clearSelection();

      // clear existing records
      epUtm.clear();
      
      // populate the list of all users
      AdminUtils.busy(client);
      TaggedName[] list = adm.listUsers();
      AdminUtils.idle(client);
      
      if (list != null)
      {
         if (list.length > 0)
         {
            // list is non-empty
            rc = true;
            epUtm.loadAll(list);
         }
      }
      else
      {
         client.alarm();
      }
      
      return rc;
   }

   /**
    * Resets the Clone group panel. 
    */
   private void resetClonePanel()
   {
      wpfN.setText("");
      wpcW.setSelected(false);
   }
   
   /**
    * Resets the center panel after reloading the groups list. 
    * Center panel's list of users is cleared as well as two text fields. 
    */
   private void resetCenterPanel()
   {
      // clear existing records
      cpUtm.clear();
      
      // clear text input
      cpfN.setText("");
      resetGroupPanel();
   }
   
   /**
    * Resets the east panel after reloading the groups list. 
    * East panel's list of all users clears the current selection. 
    */
   private void resetEastPanel()
   {
      // clear existing selection
      epUsm.clearSelection();

      // disable Add selected button
      epbA.setEnabled(false);
   }

   /**
    * Resets the group definition panel after editing session. 
    */
   private void resetGroupPanel()
   {
      cpfG.setText("");
      cpfD.setText("");
      authElements.reset();
   }

   /**
    * Adds new group account definition to the server.
    * 
    * @param    name
    *           new group name
    * 
    * @param    desc
    *           new group description
    *
    * @param    mode
    *           new group auth mode
    *
    * @param    authPlugin
    *           new group auth plugin
    * 
    * @return   <code>true</code> if added successfully
    */
   private boolean addGroupDef(String name, String desc,
                               int mode, String authPlugin)
   {
      GroupDef gd = new GroupDef();
      gd.subjectId   = name;
      gd.description = desc;
      gd.mode = mode;
      gd.authPlugin = authPlugin;
      
      AdminUtils.busy(client);
      boolean rc = adm.addGroup(gd);
      AdminUtils.idle(client);
      
      return rc;
   }

   /**
    * Changes the existing account definition on the server.
    * 
    * @param    name
    *           group name
    * 
    * @param    desc
    *           group description
    *
    * @param    mode
    *           new group auth mode
    *
    * @param    authPlugin
    *           new group auth plugin
    *
    * @return   <code>true</code> if changed successfully
    */
   private boolean setGroupDef(String name, String desc,
                               int mode, String authPlugin)
   {
      GroupDef gd = new GroupDef();
      gd.subjectId   = name;
      gd.description = desc;
      gd.mode = mode;
      gd.authPlugin = authPlugin;
      
      AdminUtils.busy(client);
      boolean rc = adm.setGroup(gd);
      AdminUtils.idle(client);
      
      return rc;
   }

   /**
    * Wraps call to setGroupsNoSelection(true).
    */
   private void setGroupsNoSelection()
   {
      setGroupsNoSelection(true);
   }
   
   /**
    * Sets state of components to no selection in the Groups list
    * <p>
    * No selection means that the state should be STATE_INITIAL_EMPTY. Only
    * the "Populate" and "Add group" buttons are enabled.
    * 
    * @param    enableList
    *           if <code>true</code>, the groups list gets enabled.
    */
   private void setGroupsNoSelection(boolean enableList)
   {
      // reset flags
      singleGroup = false;

      // set enabled state of the list accordingly
      stateMachine(MASK_WEST_CENTER, STATE_INITIAL_EMPTY);
      if (enableList)
      {
         wpGroups.setEnabled(true);         
      }
      
      // clear the Clone group panel
      resetClonePanel();
      
      // clear the center panel
      resetCenterPanel();
      
      // clear selection in east panel, if any
      resetEastPanel();
   }
   
   /**
    * Sets state of components to single selection in the Groups list 
    * <p>
    * Single selection means that all buttons of the west panel 
    * and "Clone group" sub-panel are enabled, since all these operations make
    * sense for a single selected group.<br>
    * Additionally, the center panel's list of users gets populated with 
    * the users that belong to the selected group. The related  
    * "Add user" button is enabled, too.<br>
    * The Group definition panel shows the selected group's details.<br>
    * At last, the east panel, if it is populated, shows the selection
    * of users from the center panel.
    */
   private void setGroupsSingleSelection()
   {
      // busy
      AdminUtils.busy(client);
      
      // set flags
      singleGroup = true;

      // set enabled state of the components
      stateMachine(MASK_WEST_CENTER, STATE_GROUPS_SINGLE);
      
      // populate center panel's users list
      groupName = (String)wpGtm.getValueAt(wpGroups.getSelectedRow(), 0);
      populateUsersList(groupName);
      
      // set proper selection in east panel's all users list
      setAllUsersSelection();

      // show the group definition
      GroupDef gd = adm.getGroup(groupName);
      if (gd == null)
      {
         client.alarm();
      }
      else
      {
         client.ok();
         editingGroup = gd.description;
         editingAuthMode = gd.mode;
         editingAuthPlugin = gd.authPlugin;
         
         cpfG.setText(groupName);
         cpfD.setText(editingGroup);
         authElements.setAuthMode(editingAuthMode);
         authElements.setAuthPlugin(editingAuthPlugin);
      }

      // reset the clone group panel
      resetClonePanel();

      // idle
      AdminUtils.idle(client);
   }
   
   /**
    * Sets state of components to no selection in the Groups list 
    * <p>
    * Multiple selection means that buttons, specific to the single selection,
    * are disabled. The enabled buttons on the west panel are Populate,
    * Add group and Delete selected.<br>
    * The center panel is fully disabled and the east panel's list of all
    * users has no selection and is disabled.
    */
   private void setGroupsMultiSelection()
   {
      // reset flags
      singleGroup = false;
      
      // set enabled state of the components
      stateMachine(MASK_WEST_CENTER, STATE_GROUPS_MULTI);
      
      // clear the Clone group panel
      resetClonePanel();
      
      // clear the center panel
      resetCenterPanel();
      
      // clear selection in east panel, if any
      resetEastPanel();
   }
   
   /**
    * Sets state of components to no selection in the Users list
    * No selection means that the Delete Selected button should be disabled.
    */
   private void setUsersNoSelection()
   {
      cpbD.setEnabled(false);
   }
   
   /**
    * Sets state of components to any selection in the Users list
    * Any selection means that the Delete Selected button should be enabled.
    */
   private void setUsersAnySelection()
   {
      cpbD.setEnabled(true);
   }
   
   /**
    * Sets selection of the all users list so that all users of the
    * currently selected single group are selected in the all users list.
    */
   private void setAllUsersSelection()
   {
      // clear existing selection if any
      epUsm.clearSelection();
      
      // get all users of the selected group and select them one by one
      int nusers = cpUtm.getRowCount();
      
      nextUser:
      for (int i = 0; i < nusers; i++)
      {
         // get user name at this position
         String user = (String)cpUtm.getValueAt(i, 0);
         
         // locate the user in the all users list
         for (int j = 0; j < epUtm.getRowCount(); j++)
         {
            if (epUtm.getValueAt(j, 0).equals(user))
            {
               // convert the model index to view index
               int iv = epUsers.convertRowIndexToView(j);
               
               // select this index
               if (iv != -1)
               {
                  epUsers.addRowSelectionInterval(iv, iv);
               }
               continue nextUser;
            }
         }
      }

      // disable Add selected button
      epbA.setEnabled(false);
   }
   
   /**
    * Refreshes the front table of the print preview panel with the data
    * from the sorted back table.
    */
   private void refreshView(JTable backTable, TableModel backTm,
                            PrintGroupsModel frontTm)
   {
      // build preview front table
      List<GroupInfo> frontGl = new LinkedList<GroupInfo>();
      int nall = backTable.getRowCount();
      
      for (int i = 0; i < nall; i++)
      {
         // translate view index to model
         int im = backTable.convertRowIndexToModel(i);
         
         // stuff the front list with the group info
         GroupInfo frontGi = new GroupInfo();
         frontGi.gid    = (String)backTm.getValueAt(im, 0);
         frontGi.descr  = (String)backTm.getValueAt(im, 1);
         
         // get the assigned users
         TaggedName[] list = adm.listGroupUsers(frontGi.gid);
         if (list == null)
         {
            // error retrieving group users
            frontGi.uid    = "*** error ***";
            frontGi.person = "*** error ***";
            frontGl.add(frontGi);
            continue;
         }
       
         for (int j = 0; j < list.length; j++)
         {
            frontGi.uid    = list[j].getName();
            frontGi.person = list[j].getTag();
            frontGl.add(frontGi);
            frontGi = new GroupInfo();
            frontGi.gid    = "";
            frontGi.descr  = "";
         }
      }
      
      frontTm.reload(frontGl.toArray(new GroupInfo[] {}));
   }   
   
   /**
    * Group information record for printing.
    */
   private static class GroupInfo
   {
      /** group id */
      String gid;
      
      /** group description */
      String descr;
      
      /** user id */
      String uid;
      
      /** person's name */
      String person;
   }
   
   /**
    * Provides a table data model to represent printable group accounts.
    */
   private static class PrintGroupsModel
   extends AbstractTableModel
   {
      /** Column names. */
      private final String[] columnNames =
      {
         "Group ID",
         "Description",
         "User ID",
         "Person"
      };
      
      /** Default column sizes. */
      private final int columnSizes[] =
      {
         50,
         100,
         50,
         100
      };
      
      /** Column data types. */
      private final Class<?>[] columnTypes =
      {
         String.class,
         String.class,
         String[].class,
         String[].class
      };
      
      /** Core data for the table. */
      private GroupInfo[] data = null;
   
      /**
       * Construct an instance.
       * 
       * @param    data
       *           The data to provide to the table.
       */
      public PrintGroupsModel(GroupInfo[] data)
      {
         this.data = data;
      }
      
      /**
       * Gets the number of columns in the table model.
       * 
       * @return   The number of columns.
       */
      public int getColumnCount()
      {
         return columnNames.length;
      }
   
      /**
       * Gets the width of the given column.
       * 
       * @param    col
       *           The column index (0-based).
       * 
       * @return   The width in pixels.
       */
      @SuppressWarnings("unused")
      public int getColumnWidth(int col)
      {
         return columnSizes[col];
      }
   
      /**
       * Gets the number of rows in the table model.
       * 
       * @return   The number of rows.
       */
      public int getRowCount()
      {
         return data.length;
      }
   
      /**
       * Gets the name of a specific column.
       * 
       * @param    col
       *           The column index (0-based).
       * 
       * @return   The column name.
       */
      public String getColumnName(int col)
      {
         return columnNames[col];
      }
   
      /**
       * Get the cell value to be displayed.
       * 
       * @param    col
       *           The column index (0-based).
       * @param    row
       *           The row index (0-based).
       * 
       * @return   The value in specified cell.
       */
      public Object getValueAt(int row, int col)
      {
         Object datum = null;
         GroupInfo info  = data[row];
         
         switch (col)
         {
            case 0:
               datum = info.gid;
               break;
            case 1:
               datum = info.descr;
               break;
            case 2:
               datum = info.uid;
               break;
            case 3:
               datum = info.person;
               break;
         }
         
         return datum;
      }
   
      /**
       * Gets the data type of a specific column.
       * 
       * @param    col
       *           The column index (0-based).
       * 
       * @return   The column type.
       */
      public Class<?> getColumnClass(int col)
      {
         return columnTypes[col];
      }
      
      /**
       * Reloads the entire table with new data.
       * 
       * @param    data
       *           The data to provide to the table.
       */
      public void reload(GroupInfo[] data)
      {
         this.data = data;
         fireTableDataChanged();
      }
   }
}