/*
** Module   : AdminJars.java
** Abstract : "Customer libraries" screen
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ------------------Description--------------------
** 001 SVL 20111209 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import com.goldencode.p2j.admin.*;

/**
 * "Customer libraries" screen.
 */
public class AdminJars
extends AdminScreen
{
   /** Screen switcher - "customer libraries" screen ID. */
   public static final String SCREEN_JARS = "adm_jars";
   
   /** Screen title. */
   protected static final String TITLE_JARS = "Customer Libraries";
   
   /** Jar table model. */
   protected TaggedDataTableModel jarTableModel = null;

   /** API table model. */
   protected TaggedDataTableModel apiTableModel = null;

   /** Jar table. */
   protected JTable jarTable = null;

   /** API table. */
   protected JTable apiTable = null;

   /** Refresh jars button. */
   protected JButton btnRefresh = null;

   /** Register jar button. */
   protected JButton btnRegJar = null;

   /** Deregister jar button. */
   protected JButton btnDeregJar = null;

   /** Reload jar button. */
   protected JButton btnReloadJar = null;

   /** Register API button. */
   protected JButton btnRegApi = null;

   /** Deregister API button. */
   protected JButton btnDeregApi = null;

   /** Edit jar hooks button. */
   protected JButton btnEditHooks = null;

   /** Use server-wide hooks radio button. */
   protected JRadioButton useServerWideHooks = null;

   /** Use global hooks radio button. */
   protected JRadioButton useGlobalHooks = null;

   /** Label with managed libraries directory information. */
   protected JLabel lblDirInfo = null;

   /** Defines whether hooks are server-wide or global. */
   protected boolean isServerWideHooks = true;

   /** 
    * Constructor.
    *
    * @param    client
    *           Core UI module of which we are a part.
    */
   AdminJars(final AdminClient client)
   {
      // store useful references
      this.client = client;
      adm = client.getExports();

      // radio buttons
      useServerWideHooks = new JRadioButton("Use server-wide hooks");
      useGlobalHooks = new JRadioButton("Use global hooks");

      ButtonGroup group = new ButtonGroup();
      group.add(useServerWideHooks);
      group.add(useGlobalHooks);
      
      // jar panel components
      jarTableModel = new TaggedDataTableModel("Jar File Name",
                                               "Active Hook");
      JScrollPane jarScroller = new JScrollPane();
      jarTable = AdminUtils.createBorderedScrolledTable(
            "Registered Customer Libraries",
            jarTableModel, jarScroller);
      TableColumn col = jarTable.getColumnModel().getColumn(0);
      col.setMaxWidth(220);
      col.setPreferredWidth(220);
      col.setWidth(220);

      btnRefresh = new JButton("Refresh");
      btnRegJar = new JButton("Register New Customer Lib");
      btnDeregJar = new JButton("Deregister Customer Lib");
      btnReloadJar = new JButton("Reload Customer Lib");
      btnEditHooks = new JButton("Edit Hook Class");

      // api panel components
      apiTableModel = new TaggedDataTableModel("Interface", "Implementation Class");
      JScrollPane apiScroller = new JScrollPane();
      apiTable = AdminUtils.createBorderedScrolledTable(
            "APIs Registered for the Customer Library",
            apiTableModel, apiScroller);

      btnRegApi = new JButton("Register New API");
      btnDeregApi = new JButton("Deregister API");

      lblDirInfo = new JLabel();
      lblDirInfo.setFont(new Font("Dialog", Font.PLAIN, 12));
      lblDirInfo.setToolTipText("<html>You can change the directory by specifying the <i>-Dmanaged.libs.dir=some/path/</i> java option or<br>" +
                                      "by setting the <i>/server/default/managed-libs-dir</i> parameter in the configuration directory.</html>");

      // create control panels
      JPanel jarControlPanel = AdminUtils.groupComponents(
         new Component[][]{{btnRefresh, btnRegJar, btnEditHooks},
                           {btnDeregJar, btnReloadJar, new JLabel()}},
         true);
      JPanel apiControlPanel = AdminUtils.groupComponents(
         new Component[][]{{btnRegApi, btnDeregApi}});

      JPanel jarRadioButtons = AdminUtils.groupComponents(
         new Component[][]{{useServerWideHooks, useGlobalHooks}});
      ((GroupLayout) jarRadioButtons.getLayout()).
                           setAutoCreateContainerGaps(false);

      // create the main screen and add it to the swapper
      JPanel mainPanel = AdminUtils.groupComponents(
         new Component[][]{{lblDirInfo, new JLabel()},
                           {jarRadioButtons, new JLabel()},
                           {jarScroller, apiScroller},
                           {jarControlPanel, apiControlPanel}});
      client.add(mainPanel, SCREEN_JARS);

      // assign action listeners
      btnRefresh.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent evt)
         {
            refreshJarList(true);
         }
      });

      jarTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
      {
         @Override
         public void valueChanged(ListSelectionEvent evt)
         {
            refreshJarButtonsState();
            refreshApiList();
         }
      });

      apiTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
      {
         @Override
         public void valueChanged(ListSelectionEvent evt)
         {
            refreshApiButtonsState();
         }
      });

      btnRegJar.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            handleRegisterJar();
         }
      });

      btnRegApi.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            handleRegisterApi();
         }
      });

      btnDeregJar.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            handleDeregisterJar();
         }
      });

      btnDeregApi.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            handleDeregisterAPI();
         }
      });

      btnReloadJar.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            handleReloadJar();
         }
      });

      btnEditHooks.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            handleEditHooks();
         }
      });

      // create the array of components for the state machine
      state = new JComponent[]
      {
            jarScroller,
            jarTable,
            jarControlPanel,
            btnRefresh, btnRegJar, btnDeregJar, btnReloadJar, btnEditHooks,
            apiScroller,
            apiTable,
            apiControlPanel,
            btnRegApi, btnDeregApi
      };

      // get hooks type
      loadHooksType();

      useServerWideHooks.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            handleChangeHooksType();
         }
      });

      useGlobalHooks.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            handleChangeHooksType();
         }
      });

      // populate list up front
      refreshJarList(false);

      loadManagedLibsDirInfo();
   }

   /**
    * Get the hooks type and select a proper radio button.
    */
   protected void loadHooksType()
   {
      AdminUtils.busy(client);
      Boolean res = adm.isUsingServerWideHooks();
      AdminUtils.idle(client);

      if (res == null)
      {
         JOptionPane.showMessageDialog(client,
               "Cannot get hooks type",
               "Error",
               JOptionPane.ERROR_MESSAGE);
         return;
      }

      isServerWideHooks = res;
      useServerWideHooks.setSelected(res);
      useGlobalHooks.setSelected(!res);
   }

   /**
    * Handle "use server-wide hooks" and "use global hooks" radio buttons.
    */
   protected void handleChangeHooksType()
   {
      boolean toServerWide;

      if (!isServerWideHooks && useServerWideHooks.isSelected())
      {
         // "use server-wide hooks?" prompt
         toServerWide = true;
      }
      else if (isServerWideHooks && useGlobalHooks.isSelected())
      {
         // "use global hooks?" prompt
         toServerWide = false;
      }
      else
         return;

      JButton btnOK1 = new JButton(toServerWide ?
            "<html><center><b>Yes</b>, copy all hooks from global scope to<br>" +
                  "server-wide scope</center></html>" :
            "<html><center><b>Yes</b>, move all server-wide hooks to global scope <br>" +
                  "(overwriting global hooks)</center></html>");
      btnOK1.setFont(new Font("Dialog", Font.PLAIN, 12));
      btnOK1.setVerticalAlignment(SwingConstants.TOP);

      JButton btnOK2 = new JButton(toServerWide ?
            "<html><b>Yes</b>, use empty initial set of server-wide hooks</html>" :
            "<html><b>Yes</b>, just remove server-wide hooks</html>");
      btnOK2.setFont(new Font("Dialog", Font.PLAIN, 12));

      JButton btnCancel = new JButton("Cancel");

      JTextArea question = AdminUtils.createReadOnlyTextArea(toServerWide ?
         "Are you sure you want to use server-wide hooks instead of global hooks?" :
         "Are you sure you want to use global hooks instead of server-wide hooks? " +
               "All server-wide hooks will be removed.");
      question.setFont(new Font("Dialog", Font.BOLD, 12));

      JPanel controlPanel = AdminUtils.groupComponents(
         new Component[][]{{btnOK1, btnOK2, btnCancel}},
         null, false, GroupLayout.Alignment.CENTER);

      GroupLayout layout = (GroupLayout) controlPanel.getLayout();
      layout.linkSize(SwingConstants.VERTICAL, btnOK1, btnOK2, btnCancel);

      JPanel dialogPanel = AdminUtils.groupComponents(
            new Component[][]{{question}, {controlPanel}});

      final JDialog dialog = AdminUtils.modalDialog(client,
                                    toServerWide ?
                                       "Use Server-wide Hooks" :
                                       "Use Global Hooks",
                                    dialogPanel);

      final Boolean[] answer = {null};

      btnOK1.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            answer[0] = true;
            dialog.setVisible(false);
         }
      });

      btnOK2.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            answer[0] = false;
            dialog.setVisible(false);
         }
      });

      btnCancel.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            dialog.setVisible(false);
         }
      });

      // switch selection back
      useServerWideHooks.setSelected(!toServerWide);
      useGlobalHooks.setSelected(toServerWide);

      // display dialog
      dialog.setVisible(true);
      dialog.dispose();

      if (answer[0] == null)
         return;

      // store new type
      AdminUtils.busy(client);
      JarOpErrorCode result = adm.changeJarHooksType(toServerWide,
                                                         answer[0]);
      AdminUtils.idle(client);

      if (result != JarOpErrorCode.SUCCESS)
      {
         displayErrorMessage("An error occured while storing new hooks type.",
                             result);
         return;
      }

      // accept choice by reloading information from server
      loadHooksType();
   }

   /**
    * Display error message which consist of text which describes the failed
    * action and the reason of failure.
    *
    * @param client
    *        Related instance of AdminClient.
    * @param actionDescriptionText
    *        Description of the failed action.
    * @param result
    *        The reason of failure.
    */
   static void displayErrorMessage(AdminClient client,
                                   String actionDescriptionText,
                                   JarOpErrorCode result)
   {
      StringBuilder sb = new StringBuilder(actionDescriptionText);
      sb.append(" ");
      sb.append(result);

      JOptionPane.showMessageDialog(client,
           sb.toString(),
           "Error",
           JOptionPane.ERROR_MESSAGE);
   }

   /**
    * Checks whether MultiClassLoader is used as the system class loader.
    * If not, displays the error message.
    *
    * @param  client
    *         Related instance of AdminClient.
    *
    * @return <code>true</code> if MultiClassLoader is used as the system
    *         class loader.
    */
   static boolean checkMultiClassLoader(AdminClient client)
   {
      AdminUtils.busy(client);
      Boolean res = client.getExports().isUsingMultiClassLoader();
      AdminUtils.idle(client);

      if (res == null)
      {
         JOptionPane.showMessageDialog(client,
               "Cannot get class loader type",
               "Error",
               JOptionPane.ERROR_MESSAGE);
         return false;
      }
      else if (!res)
      {
         JOptionPane.showMessageDialog(client,
                  "<html>In order to be able to manage customer libraries you should use MultiClassLoader as the system class loader.<br>" +
                        "For this, specify -Djava.system.class.loader=com.goldencode.p2j.classloader.MultiClassLoader java option<br>" +
                        "for the P2J server.",
                  "Invalid Class Loader",
                  JOptionPane.INFORMATION_MESSAGE);
         return false;
      }

      return true;
   }

   /**
    * Presents the refreshed customer libraries screen.
    */
   void manageJars()
   {
      // switch to this screen without changing state of components
      client.show(SCREEN_JARS, TITLE_JARS);
   }

   /**
    * Handle "deregister API" button.
    */
   private void handleDeregisterAPI()
   {
      String targetJar = getSelectedJar();
      TaggedName targetApi = getSelectedApi();

      if (targetJar == null || targetApi == null)
         return;

      boolean res = AdminUtils.confirmDialog(client,
            "Deregister API",
            "Are you sure you want to deregister the selected API?");

      if (!res)
         return;

      AdminUtils.busy(client);
      JarOpErrorCode result = adm.deregisterJarAPI(targetJar,
                                                   targetApi.getName(),
                                                   false);
      AdminUtils.idle(client);

      if (result == JarOpErrorCode.ACTIVE_SESSIONS_PRESENT)
      {
         // there are active session related to the selected API, display
         // the "terminate sessions" dialog.
         AdminTerminateSessions dialog
               = new AdminTerminateSessions(client, adm,
               AdminTerminateSessions.JarActionType.API_DEREGISTER,
               targetJar, targetApi.getName());

         dialog.runDialog();
      }
      else if (result != JarOpErrorCode.SUCCESS)
      {
         displayErrorMessage("An error occured while deregistering the API.",
               result);
      }

      refreshApiList();
   }

   /**
    * Handle "deregister jar" button.
    */
   private void handleDeregisterJar()
   {
      String targetJar = getSelectedJar();

      if (targetJar == null)
         return;

      boolean res = AdminUtils.confirmDialog(client,
            "Deregister Customer Library",
            "Are you sure you want to deregister the selected customer library?");

      if (!res)
         return;

      AdminUtils.busy(client);
      JarOpErrorCode result = adm.deregisterJar(targetJar, false, true);
      AdminUtils.idle(client);

      if (result == JarOpErrorCode.ACTIVE_SESSIONS_PRESENT)
      {
         // there are active session related to the selected jar, display
         // the "terminate sessions" dialog.
         AdminTerminateSessions dialog
               = new AdminTerminateSessions(client, adm,
               AdminTerminateSessions.JarActionType.JAR_DEREGISTER,
               targetJar, null);

         dialog.runDialog();

      }
      else if (result != JarOpErrorCode.SUCCESS)
      {
         displayErrorMessage("An error occured while deregistering the jar.",
               result);
      }

      refreshJarList(false);
   }

   /**
    * Refresh the list of jars.
    *
    * @param preserveSelection
    *        <code>true</code> if current selection should be preserved.
    */
   private void refreshJarList(boolean preserveSelection)
   {
      // get the list of jars from the server
      AdminUtils.busy(client);
      TaggedName[] jars = adm.getRegisteredJars();
      AdminUtils.idle(client);

      if (jars != null)
      {
         String selectedJar = null;
         if (preserveSelection)
         {
            selectedJar = getSelectedJar();
         }

         jarTableModel.loadAll(jars);
         Enumeration<TableColumn> cols = jarTable.getColumnModel().getColumns();
         while (cols.hasMoreElements())
         {
            cols.nextElement().setCellRenderer(new JarHookCellRenderer());
         }

         if (preserveSelection && selectedJar != null)
         {
            int idx = -1;

            int size = jarTableModel.data.size();
            for (int i = 0; i < size; i++)
            {
               if (selectedJar.equals(jarTableModel.data.get(i).getName()))
               {
                  idx = i;
                  break;
               }
            }
            if (idx != -1)
               jarTable.addRowSelectionInterval(idx, idx);
         }
      }
      else
      {
         JOptionPane.showMessageDialog(client,
                        "Cannot get the list of registered jars.",
                        "Error",
                        JOptionPane.ERROR_MESSAGE);
      }

      refreshJarButtonsState();
      refreshApiList();
   }

   /**
    * Refresh API list.
    */
   private void refreshApiList()
   {
      String selectedJar = getSelectedJar();

      if (selectedJar != null)
      {
         // get the list of APIs from the server
         AdminUtils.busy(client);
         TaggedName[] apiList = adm.getRegisteredJarAPIs(selectedJar);
         AdminUtils.idle(client);

         if (apiList != null)
         {
            apiTableModel.loadAll(apiList);
         }
         else
         {
            JOptionPane.showMessageDialog(client,
                        "Cannot get the list of registered APIs.",
                        "Error",
                        JOptionPane.ERROR_MESSAGE);
         }
      }
      else
      {
         apiTableModel.clear();
      }

      refreshApiButtonsState();
   }
   
   /**
    * Get selected jar.
    *
    * @return the name of the selected jar or <code>null</code> if there is
    *         no selected jar.
    */
   private String getSelectedJar()
   {
      int selectedRow = jarTable.getSelectedRow();

      String selection = null;
      if (selectedRow >= 0)
      {
         selection = (String) jarTableModel.getValueAt(selectedRow, 0);
      }

      return selection;
   }

   /**
    * Get selected API.
    *
    * @return the TaggedName which represents the selected API or
    *         <code>null</code> if there is no selected API.
    */
   private TaggedName getSelectedApi()
   {
      int selectedRow = apiTable.getSelectedRow();

      TaggedName selection = null;
      if (selectedRow >= 0)
      {
         selection = apiTableModel.data.get(selectedRow);
      }

      return selection;
   }

   /**
    * Update the state of jar handling buttons.
    */
   private void refreshJarButtonsState()
   {
      boolean selected = getSelectedJar() != null;

      btnDeregJar.setEnabled(selected);
      btnReloadJar.setEnabled(selected);
      btnEditHooks.setEnabled(selected);
   }

   /**
    * Update the state of API handling buttons.
    */
   private void refreshApiButtonsState()
   {
      if (getSelectedJar() == null)
      {
         btnRegApi.setEnabled(false);
         btnDeregApi.setEnabled(false);
      }
      else
      {
         btnRegApi.setEnabled(true);
         btnDeregApi.setEnabled(getSelectedApi() != null);
      }
   }

   /**
    * Convert an array of strings to the array of TaggedNames objects which
    * names match the specified strings.
    *
    * @param  strings
    *         Source array of strings.
    * @return array of TaggedNames objects which names match the specified
    *         strings.
    */
   private TaggedName[] toTaggedNames(String[] strings)
   {
      TaggedName[] dest = new TaggedName[strings.length];
      for (int i = 0; i < strings.length; i++)
      {
         dest[i] = new TaggedName(strings[i]);
      }
      return dest;
   }

   /**
    * Handle "register customer library" button.
    */
   private void handleRegisterJar()
   {
      // jar selection screen
      AdminUtils.busy(client);
      String[] jars = adm.getUnregisteredJars();
      AdminUtils.idle(client);

      String targetJar = null;

      if (jars != null)
      {
         if (jars.length == 0)
         {
            AdminUtils.busy(client);
            TaggedName dirInfo = adm.getManagedLibsDirInformation();
            AdminUtils.idle(client);

            if (dirInfo == null)
            {
               JOptionPane.showMessageDialog(client,
                     "Cannot get managed libraries directory information.",
                     "Error",
                  JOptionPane.ERROR_MESSAGE);
               return;
            }

            StringBuilder sb = new StringBuilder();
            if (dirInfo.getName() == null)
            {
               sb.append("<html><span style=\"color:red\">Customer libraries directory is not specified!</span><br>");
               sb.append("You can set the directory by specifying the <i>-Dmanaged.libs.dir=some/path/</i> java option or<br>");
               sb.append("by setting the <i>/server/default/managed-libs-dir</i> parameter in the configuration directory.<br>");
            }
            else
            {
               sb.append("<html>There are no unregistered customer libraries in the managed customer libraries directory!<br>");
               sb.append("The current path to the directory is <i>");
               sb.append(dirInfo.getName());
               sb.append("true".equals(dirInfo.getTag()) ? "</i>.<br>" :
                         "</i>, but it <span style=\"color:red\"> does not exist or is inaccessible</span>!<br>");
               sb.append("You can change the directory by specifying the <i>-Dmanaged.libs.dir=some/path/</i> java option or<br>");
               sb.append("by setting the <i>/server/default/managed-libs-dir</i> parameter in the configuration directory.<br>");
            }
            sb.append("Also, managed customer libraries should not present in the server class path.</html>");

            JOptionPane.showMessageDialog(client,
                                          sb.toString(),
                                          "No Unregistered Libraries",
                                          JOptionPane.INFORMATION_MESSAGE);
            return;
         }
         else
         {
            TaggedName[] data = toTaggedNames(jars);

            AdminSelectionListDialog dialog = new AdminSelectionListDialog(
                  client,
                  "Register New Customer Library (Step 1 / 2)",
                  "Customer libraries that can be registered",
                  new String[]{"Jar file name"},
                  "Select the appropriate customer library. If you are not seeing the expected library " +
                        "then it may be already registered or presents in the server class path.",
                  "Select a customer library!",
                  "No Customer Library Is Selected");

            TaggedName tn = dialog.runDialog(data);
            if (tn != null)
               targetJar = tn.getName();

            // dialog was cancelled
            if (targetJar == null)
               return;
         }
      }
      else
      {
         JOptionPane.showMessageDialog(client,
               "Cannot get the list of unregistered jars.",
               "Error",
               JOptionPane.ERROR_MESSAGE);
         return;
      }

      // hook class selection screen
      AdminUtils.busy(client);
      String[] hooks = adm.getAvailableJarHookClasses(targetJar);
      AdminUtils.idle(client);

      String targetHook = null;

      if (hooks != null)
      {
         if (hooks.length == 0)
         {
            JOptionPane.showMessageDialog(client,
                  "<html>There are no classes in the selected library that can serve as a hook class which<br>" +
                        "performs jar initialization. No hook class will be defined for this library.</html>",
                  "No Hook Classes Found for the Selected Jar",
                  JOptionPane.INFORMATION_MESSAGE);
         }
         else
         {
            TaggedName[] data = toTaggedNames(hooks);

            final HookSelectionDialog dialog = new HookSelectionDialog(
                  client,
                  "Register New Customer Library (Step 2 / 2)",
                  "Hook classes availible in the selected jar",
                  new String[]{"Hook class name"},
                  "Select an appropriate hook class. This class will be used for jar initialization. " +
                     "Note that you can skip this step.",
                  "Select a hook class or press the \"Skip this step\" button!",
                  "No Hook Class Is Selected",
                  "OK",
                  "Skip this step");

            dialog.btnCancel.addActionListener(new ActionListener()
            {
               @Override
               public void actionPerformed(ActionEvent e)
               {
                  // tagged name with null name means that hook class
                  // selection was skipped
                  dialog.result = new TaggedName((String) null, null);
               }
            });

            TaggedName tn = dialog.runDialog(data);
            if (tn != null)
               targetHook = tn.getName();
            else
               return;
         }
      }
      else
      {
         JOptionPane.showMessageDialog(client,
               "Cannot get the list of jar hook classes.",
               "Error",
               JOptionPane.ERROR_MESSAGE);
         return;
      }

      AdminUtils.busy(client);
      JarOpErrorCode result = adm.registerJar(targetJar, targetHook, true);
      AdminUtils.idle(client);

      refreshJarList(false);

      if (result == JarOpErrorCode.SUCCESS)
      {
         JOptionPane.showMessageDialog(client,
               "The customer library was successfully registered.",
               "Operation Completed",
               JOptionPane.INFORMATION_MESSAGE);
      }
      else
      {
         displayErrorMessage("An error occured while registering the customer library.",
               result);
      }
   }

   /**
    * Handle "register API" button.
    */
   private void handleRegisterApi()
   {
      String targetJar = getSelectedJar();

      if (targetJar == null)
         return;

      // interface selection screen
      AdminUtils.busy(client);
      String[] interfaces = adm.getAvailableJarAPIInterfaces(targetJar,
                                                             false);
      AdminUtils.idle(client);

      String targetInterface = null;

      if (interfaces != null)
      {
         if (interfaces.length == 0)
         {
            JOptionPane.showMessageDialog(client,
                  "In the selected jar there are no unregistered interfaces which can define an API.",
                  "No Interfaces Were Found",
                  JOptionPane.INFORMATION_MESSAGE);
            return;
         }
         else
         {
            TaggedName[] data = toTaggedNames(interfaces);

            AdminSelectionListDialog dialog = new AdminSelectionListDialog(
                  client,
                  "Register New API (Step 1 / 2)",
                  "Interfaces",
                  new String[]{"Interface name"},
                  "Select the interface which defines the API you want to register (this list contains " +
                        "only unregistered interfaces).",
                  "Select an interface!",
                  "No Interface Is Selected");

            TaggedName tn = dialog.runDialog(data);
            if (tn != null)
               targetInterface = tn.getName();

            if (targetInterface == null)
               return;
         }
      }
      else
      {
         JOptionPane.showMessageDialog(client,
               "Cannot get the list of interfaces.",
               "Error",
               JOptionPane.ERROR_MESSAGE);
         return;
      }

      // implementation class selection screen
      AdminUtils.busy(client);
      String[] implementations = adm.getAvailableJarAPIImplClasses(
            targetJar,
            targetInterface);
      AdminUtils.idle(client);

      String targetImplementation;

      if (implementations != null)
      {
         if (implementations.length == 0)
         {
            JOptionPane.showMessageDialog(client,
                  "<html>In the selected customer library there are no classes which statically<br>" +
                        "implement the selected interface. API registration was cancelled.</html>",
                  "No Implementation Classes Were Found",
                  JOptionPane.INFORMATION_MESSAGE);
            return;
         }
         else
         {
            TaggedName[] data = toTaggedNames(implementations);

            AdminSelectionListDialog dialog = new AdminSelectionListDialog(
                  client,
                  "Register New API (Step 2 / 2)",
                  "Implementation classes",
                  new String[]{"Class name"},
                  "Select the class which implements the API you want to register.",
                  "Select an implementation class!",
                  "No Implementation Class Is Selected");

            TaggedName tn = dialog.runDialog(data);
            if (tn != null)
               targetImplementation = tn.getName();
            else
               return;
         }
      }
      else
      {
         JOptionPane.showMessageDialog(client,
               "Cannot get the list of implementation classes.",
               "Error",
               JOptionPane.ERROR_MESSAGE);
         return;
      }

      AdminUtils.busy(client);
      JarOpErrorCode result = adm.registerJarAPI(targetInterface,
            targetImplementation);
      AdminUtils.idle(client);

      refreshApiList();

      if (result == JarOpErrorCode.SUCCESS)
      {
         JOptionPane.showMessageDialog(client,
               "The API was successfully added.",
               "Operation Completed",
               JOptionPane.INFORMATION_MESSAGE);
      }
      else
      {
         displayErrorMessage("An error occured while registering the API.",
               result);
      }
   }

   /**
    * Handle "edit hook class" button.
    */
   private void handleEditHooks()
   {
      final String targetJar = getSelectedJar();
      if (targetJar == null)
         return;

      JButton btnEditHookClass = new JButton("Edit");
      JButton btnRemoveHookClass = new JButton("Remove");
      JButton btnClose = new JButton("Close");
      final JTextField txtHookClass
            = AdminUtils.createReadOnlyTextField(null, 35);

      reloadHookClass(targetJar, txtHookClass);

      JTextArea help = AdminUtils.createReadOnlyTextArea(
            "You are editing the hook class specified in the directory, so it may not match " +
            "the active hook class displayed in the customer libraries table. " +
            "Hook is reinitialized when the customer library is reloaded.");
      help.setForeground(Color.RED);
      help.setRows(3);

      JPanel panel = AdminUtils.groupComponents(
            new Component[][]{
                  {new JLabel("Hook class"), txtHookClass, btnEditHookClass, btnRemoveHookClass},
                  {new JLabel(), help, new JLabel(), new JLabel()},
                  {btnClose, new JLabel(), new JLabel(), new JLabel()}});

      final JDialog dialog = AdminUtils.modalDialog(client,
                                    "Edit Customer Library Hook Class",
                                    panel);
      btnClose.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            dialog.setVisible(false);
         }
      });

      btnEditHookClass.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            if (handleEditHookClass(txtHookClass.getText(), targetJar))
            {
               reloadHookClass(targetJar, txtHookClass);
               refreshJarList(true);
            }
         }
      });

      btnRemoveHookClass.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent e)
         {
            if (handleRemoveHookClass(targetJar))
            {
               reloadHookClass(targetJar, txtHookClass);
               refreshJarList(true);
            }
         }
      });

      dialog.setVisible(true);
      dialog.dispose();
   }

   /**
    * Get the jar hook class from server and display it in the specified
    * text field.
    *
    * @param  targetJar
    *         Target jar file.
    * @param  txtHookClass
    *         Text field for displaying server-wide hook class.
    */
   private void reloadHookClass(String targetJar,
                                JTextField txtHookClass)
   {
      AdminUtils.busy(client);
      String hookClass = adm.getStoredJarHookClass(targetJar);
      AdminUtils.idle(client);

      txtHookClass.setText(hookClass);
   }

   /**
    * Handle "edit hook class" buttons.
    *
    * @param  currentHook
    *         Current hook class (server-wide or global) for the target jar
    *         file.
    * @param  targetJar
    *         Target jar file.
    *
    * @return <code>true</code> if the hook class was successfully updated.
    */
   protected boolean handleEditHookClass(String currentHook,
                                         String targetJar)
   {
      // get hook classes available for selection
      AdminUtils.busy(client);
      String[] availHooks = adm.getAvailableJarHookClasses(targetJar);
      AdminUtils.idle(client);

      if (availHooks == null)
      {
         JOptionPane.showMessageDialog(client,
                        "Cannot get available hook classes.",
                        "Error",
                        JOptionPane.ERROR_MESSAGE);
         return false;
      }
      else if (availHooks.length == 0)
      {
         JOptionPane.showMessageDialog(client,
                  "<html>There are no classes in the selected jar that can serve as a hook class which<br>" +
                  "performs jar initialization.</html>",
                  "No Hook Classes Found for the Selected Jar",
                  JOptionPane.INFORMATION_MESSAGE);
         return false;
      }

      AdminSelectionListDialog dialog = new AdminSelectionListDialog(
                  client,
                  "Edit hook class",
                  "Hook classes availible in the selected jar",
                  new String[]{"Hook class name"},
                  null,
                  "Select a hook class!",
                  "No Hook Class Is Selected");

      // load data into dialog
      dialog.tableModel.loadAll(toTaggedNames(availHooks));

      // if the current hook class present in the list available hook classes,
      // it will be selected by default
      int idx = -1;
      if (currentHook != null && !currentHook.isEmpty())
      {
         int size = dialog.tableModel.data.size();
         for (int i = 0; i < size; i++)
         {
            if (currentHook.equals(dialog.tableModel.data.get(i).getName()))
            {
               idx = i;
               break;
            }
         }
      }
      if (idx != -1)
         dialog.table.addRowSelectionInterval(idx, idx);

      // display selection dialog
      String targetHook = null;
      TaggedName tn = dialog.runDialog(null);
      if (tn != null)
         targetHook = tn.getName();

      if (targetHook == null)
         return false;

      // update value on server
      AdminUtils.busy(client);
      JarOpErrorCode result = adm.changeJarHookClass(targetJar,
                                                     targetHook);
      AdminUtils.idle(client);

      if (result == JarOpErrorCode.SUCCESS)
      {
         return true;
      }
      else
      {
         displayErrorMessage("Cannot change hook class.", result);
         return false;
      }
   }

   /**
    * Handle "remove hook class" buttons.
    *
    * @param  targetJar
    *         Target jar file.
    *
    * @return <code>true</code> if the hook class was successfully removed.
    */
   private boolean handleRemoveHookClass(String targetJar)
   {
      boolean res = AdminUtils.confirmDialog(client, "Remove Hook",
                              "Are you sure you want to remove the hook?");
      if (!res)
         return false;

      AdminUtils.busy(client);
      JarOpErrorCode result = adm.changeJarHookClass(targetJar,
                                                     null);
      AdminUtils.idle(client);

      if (result == JarOpErrorCode.SUCCESS)
      {
         return true;
      }
      else
      {
         displayErrorMessage("Cannot change hook class.", result);
         return false;
      }
   }

   /**
    * Handle "reload customer library" button.
    */
   private void handleReloadJar()
   {
      String targetJar = getSelectedJar();

      if (targetJar == null)
         return;

      boolean res = AdminUtils.confirmDialog(client,
               "Reload Customer Library",
               "<html>Are you sure you want to reload the selected customer library?<br> " +
               "Note that you should replace the customer library on the disk only<br>" +
               "when you will be asked for it on the next step.</html>");

      if (!res)
         return;

      // first, get the stored hook class and deregister jar
      AdminUtils.busy(client);
      String storedHookClass = adm.getStoredJarHookClass(targetJar);
      JarOpErrorCode result  = adm.deregisterJar(targetJar, false, false);
      AdminUtils.idle(client);

      if (result == JarOpErrorCode.ACTIVE_SESSIONS_PRESENT)
      {
         // there are active session related to the selected jar, display
         // the "terminate sessions" dialog, it will attempt to deregister jar
         AdminTerminateSessions dialog
               = new AdminTerminateSessions(client, adm,
                  AdminTerminateSessions.JarActionType.JAR_RELOAD,
                  targetJar, null);

         if (dialog.runDialog())
         {
            // go to further steps
            result = JarOpErrorCode.SUCCESS;
         }
         else
         {
            refreshJarList(true);
            return;
         }
      }

      if (result == JarOpErrorCode.SUCCESS)
      {
         // second, replace the customer library on disk (if necessary)
         Object[] options = new Object[]{"Continue"};

         JOptionPane.showOptionDialog(client,
               "Now you can replace the customer library on the disk\n" +
               "(unless you just want to reload the hook).",
               "Replace Customer Library",
               JOptionPane.OK_OPTION,
               JOptionPane.PLAIN_MESSAGE,
               null,
               options,
               options[0]);

         // third, register jar
         AdminUtils.busy(client);
         result = adm.registerJar(targetJar, storedHookClass, false);
         AdminUtils.idle(client);

         if (result == JarOpErrorCode.SUCCESS)
         {
            JOptionPane.showMessageDialog(client,
                  "The customer library was successfully reloaded.",
                  "Operation Completed",
                  JOptionPane.INFORMATION_MESSAGE);
         }
         else
         {
            displayErrorMessage(
                  "An error occured while registering customer library.",
                  result);

            // fourthly, if we have failed to re-register jar, remove its
            // registration
            AdminUtils.busy(client);
            result = adm.deregisterJarFromDirectory(targetJar,
                                                    storedHookClass);
            AdminUtils.idle(client);

            if (result == JarOpErrorCode.SUCCESS)
            {
               StringBuilder sb = new StringBuilder();
               sb.append("<html>Because of re-registration error, registration has been removed for<br>");
               sb.append("customer library ");
               sb.append(targetJar);
               if (storedHookClass == null)
                  sb.append(" (no hook class).</html>");
               else
               {
                  sb.append(" (hook class ");
                  sb.append(storedHookClass);
                  sb.append(").</html>");
               }

               JOptionPane.showMessageDialog(client,
                     sb.toString(),
                     "Operation Completed",
                     JOptionPane.INFORMATION_MESSAGE);
            }
            else
            {
               displayErrorMessage(
                  "An error occured while deregistering customer library after re-registration error.",
                  result);
            }
         }
      }
      else
      {
         displayErrorMessage("An error occured while deregistering customer library.",
               result);
      }

      refreshJarList(true);
   }

   /**
    * Display error message which consist of text which describes the failed
    * action and the reason of failure.
    *
    * @param actionDescriptionText
    *        Description of the failed action.
    * @param result
    *        The reason of failure.
    */
   private void displayErrorMessage(String actionDescriptionText,
                                      JarOpErrorCode result)
   {
      displayErrorMessage(client, actionDescriptionText, result);
   }

   /**
    * Load information about customer libraries directory from server and
    * display it.
    */
   private void loadManagedLibsDirInfo()
   {
      AdminUtils.busy(client);
      TaggedName info = adm.getManagedLibsDirInformation();
      AdminUtils.idle(client);

      if (info == null)
      {
         JOptionPane.showMessageDialog(client,
                     "Cannot get managed libraries directory information.",
                     "Error",
                     JOptionPane.ERROR_MESSAGE);
         return;
      }

      StringBuilder sb = new StringBuilder("<html><b>Managed Libraries Directory:</b> ");
      if (info.getName() == null)
      {
         sb.append("<span style=\"color:red\"> not specified</span>");
      }
      else
      {
         sb.append(info.getName());
         if (!"true".equals(info.getTag()))
            sb.append("<span style=\"color:red\"> (doesn't exist or inaccessible)</span>");
      }
      sb.append("</html>");

      lblDirInfo.setText(sb.toString());
   }

   /**
    * Dialog which allows to select a single hook class or skip its
    * selection.
    */
   private class HookSelectionDialog
   extends AdminSelectionListDialog
   {
      /**
       * Create a dialog.
       *
       * @param client
       *        Related instance of AdminClient.
       * @param dialogTitle
       *        Dialog title.
       * @param tableTitle
       *        Table title.
       * @param columns
       *        Array of column names.
       * @param helpText
       *        Help text to be displayed above the table or <code>null</code>
       *        if it is not needed.
       * @param selectOneErrorText
       *        Text of the message which is displayed if an item is not
       *        selected.
       * @param selectOneErrorTitle
       *        Title of the message which is displayed if an item is not
       *        selected.
       * @param okButtonCaption
       *        Custom caption for the "OK" button.
       * @param cancelButtonCaption
       *        Custom caption for the "Cancel" button.
       */
      public HookSelectionDialog(AdminClient client,
                                 String dialogTitle,
                                 String tableTitle,
                                 String[] columns,
                                 String helpText,
                                 String selectOneErrorText,
                                 String selectOneErrorTitle,
                                 String okButtonCaption,
                                 String cancelButtonCaption)
      {
         super(client, dialogTitle, tableTitle, columns, helpText,
               selectOneErrorText, selectOneErrorTitle,
               okButtonCaption, cancelButtonCaption);
      }

      /**
       * Build main panel of the dialog.
       */
      @Override
      protected void buildMainPanel()
      {
         mainPanel = AdminUtils.groupComponents(
               txtHelp == null ?
                     new Component[][]{{tableScroller},
                                       {controlPanel}} :
                     new Component[][]{{txtHelp},
                                       {tableScroller},
                                       {controlPanel}});
      }
   }

   /**
    * Table cell renderer for columns of the jars table which highlights the
    * cell and adds the tooltip if the active hook does not match the stored
    * hook for the given jar.
    */
   private class JarHookCellRenderer
   extends DefaultTableCellRenderer
   {
     /**
      *
      * Returns the table cell renderer.
      *
      * @param table      the <code>JTable</code>
      * @param value      the value to assign to the cell at
      *			           <code>[row, column]</code>
      * @param isSelected true if cell is selected
      * @param hasFocus   true if cell has focus
      * @param row        the row of the cell to render
      * @param column     the column of the cell to render
      *
      * @return           the default table cell renderer
      */
      @Override
      public Component getTableCellRendererComponent(JTable table,
                                                     Object value,
                                                     boolean isSelected,
                                                     boolean hasFocus,
                                                     int row,
                                                     int column)
      {
         Component comp = super.getTableCellRendererComponent(table,
                                                              value,
                                                              isSelected,
                                                              hasFocus,
                                                              row,
                                                              column);
         TaggedName tn = jarTableModel.data.get(row);
         if (!"true".equals(tn.get(2)))
         {
            // active hook does not match stored hook
            comp.setForeground(Color.RED);
            ((JarHookCellRenderer) comp).setToolTipText(
               "Active hook class does not match stored hook class, reload " +
               "the library in order to run the stored hook class.");
         }
         else
         {
            // active hook matches stored hook
            comp.setForeground(Color.BLACK);
            ((JarHookCellRenderer) comp).setToolTipText(null);
         }
         return comp;
      }
   }
}