/*
** Module   : AdminRuntimeSettings.java
** Abstract : "Runtime settings" menu.
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SVL 20111209 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.admin.client;

import javax.swing.*;
import java.awt.event.*;

/**
 * "Runtime settings" menu.
 */
public class AdminRuntimeSettings
{
   /** Related instance of AdminClient */
   private AdminClient client = null;

   /** "Customer Libraries" screen. */
   private AdminJars jarsScr = null;

   /** "Customer Libraries" menu item. */
   private JMenuItem rtsJars = null;

   public void init(final AdminClient client, JMenu rts)
   {
      this.client = client;

      // "Per Server" submenu
      JMenu psr = new JMenu("Per Server", false);
      JMenuItem mi = new JMenuItem("Logging");
      mi.setEnabled(false);                // temporary - item not implemented
      psr.add(mi);
      mi = new JMenuItem("Startup");
      mi.setEnabled(false);                // temporary - item not implemented
      psr.add(mi);
      mi = new JMenuItem("Exports");
      mi.setEnabled(false);                // temporary - item not implemented
      psr.add(mi);
      mi = new JMenuItem("Persistence");
      mi.setEnabled(false);                // temporary - item not implemented
      psr.add(mi);
      mi = new JMenuItem("Database");
      mi.setEnabled(false);                // temporary - item not implemented
      psr.add(mi);
      rts.add(psr);

      // "Per Account" submenu
      JMenu par = new JMenu("Per Account", false);
      mi = new JMenuItem("Parameters");
      mi.setEnabled(false);                // temporary - item not implemented
      par.add(mi);
      rts.add(par);

      // top-level menu
      rtsJars = new JMenuItem("Customer Libraries");
      rtsJars.setEnabled(true);
      rts.add(rtsJars);

      // adding action listeners
      rtsJars.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent evt)
         {
            if (AdminJars.checkMultiClassLoader(client))
            {
               lazyJars();
            }
         }
      });
   }

   /**
    * Handles lazy "customer libs" list menu initialization.
    */
   private void lazyJars()
   {
      // initialize jars screen
      jarsScr = new AdminJars(client);

      // allow direct action handler invocation
      ActionListener[] lst = rtsJars.getActionListeners();

      for (ActionListener aLst : lst)
      {
         rtsJars.removeActionListener(aLst);
      }

      rtsJars.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent evt)
         {
            jarsScr.manageJars();
         }
      });

      // handle this action once
      jarsScr.manageJars();
   }
}