/*
** Module   : AdminUtils.java
** Abstract : Place for various utility methods
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20090116   @41621 Created the first version of the file.
** 002 NVS 20090428   @42043 Added busy/idle cursors.
** 003 CA  20090512   @42164 Added a confirmation dialog with Yes/No/Cancel 
**                           buttons.
** 004 CA  20090522   @42430 Made some methods public to allow plugin access.
** 005 CA  20090604   @42560 Added errorDialog(), to display an error dialog 
**                           with a specified message..
** 006 NVS 20090609   @42626 Drop 14 milestone.
** 007 GES 20090617   @42760 Added createFilter().
** 008 CA  20090714   @43159 Added APIs related to removing action listeners 
**                           and getting menu items from a menu.
** 009 NVS 20090717   @43232 Drop 15 milestone.
** 010 CA  20090720   @43248 Drop 16 milestone.
** 011 NVS 20090723   @43361 Drop 17 milestone. Added isListed() method.
** 012 NVS 20090727   @43398 Drop 18 milestone.
** 013 NVS 20090729   @43488 Added choiceDialog() method. Drop 19 milestone.
** 014 NVS 20090804   @43516 Added nested classes AccountInfo and
**                           PrintAccountModel for accounts printing.
** 015 NVS 20090804   @43524 Added nested class PrintExtendedAccountModel.
** 016 NVS 20090807   @43554 Refactored in support of custom menu extensions.
** 017 NVS 20090807   @43563 Drop 19.05
** 018 NVS 20090811   @43594 Drop 20 milestone.
** 019 NVS 20090816   @43650 Drop 21 milestone.
** 020 NVS 20090820   @43707 Version 21.02 with shutdown notification.
** 021 CA  20091119   @44438 Drop 21.03 (modified the record lock info to 
**                           display subject names too).
** 022 SIY 20091211   @44506 Minor cleanups.
** 023 SVL 20111209          createBorderedScrolledTable,
**                           createReadOnlyTextArea/Field functions have been
**                           added.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

/**
 * 
 * @author NVS
 */
public class AdminUtils 
{
   /**
    * Sets the busy type mouse cursor.
    * 
    * @param    client
    *           instance of AdminClient to use
    */
   public static void busy(AdminClient client)
   {
      client.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
   }
      
   /**
    * Sets the busy type mouse cursor.
    * 
    * @param    client
    *           instance of AdminClient to use
    */
   public static void idle(AdminClient client)
   {
      client.setCursor(Cursor.getDefaultCursor());
   }
   
   /**
    * Displays a modal dialog with the specified contents.
    * 
    * @param    client
    *           instance of AdminClient to use
    * 
    * @param    title
    *           dialog title
    * 
    * @param    panel
    *           prepared panel to use in the dialog
    * 
    * @return   the newly created and displayed dialog
    */
   public static JDialog modalDialog(AdminClient client, 
                                     String title, 
                                     JPanel panel)
   {
      Frame f = client.getFrame();
      JDialog dlg = new JDialog(f, title, true);
      
      dlg.getContentPane().add(panel, BorderLayout.CENTER); 
      dlg.pack();
      dlg.setLocationRelativeTo(client);
      
      return dlg;
   }

   /**
    * Displays Logoff confirmation dialog.
    * 
    * @param    client
    *           instance of AdminClient to use
    * 
    * @param    title
    *           dialog title
    * 
    * @param    question
    *           confirmation question assuming continuation on YES answer
    * 
    * @return   <code>true</code> if the user confirms the action
    */
   public static boolean confirmDialog(AdminClient client, 
                                       String title, 
                                       String question)
   {
      int rc = JOptionPane.showConfirmDialog(client, question, title,
                                             JOptionPane.YES_NO_OPTION, 
                                             JOptionPane.WARNING_MESSAGE);
      if (rc == 0)
      {
         return true;
      }
      
      return false;
   }
   
   /**
    * Displays choices as a set of radio button and returns the choice.
    * 
    * @param    client
    *           instance of AdminClient to use
    * 
    * @param    title
    *           dialog title
    * 
    * @param    question
    *           confirmation question assuming continuation on YES answer
    * 
    * @param    choices
    *           array of labels for the radio buttons
    * 
    * @param    goButton
    *           a label for the button that confirms the choice
    * 
    * @return   choice index if confirmed or <code>-1</code> otherwise
    */
   public static int choiceDialog(AdminClient client, 
                                  String title, 
                                  String question,
                                  String[] choices,
                                  String goButton)
   {
      // required array of components
      int nr = choices.length;
      Component[][] comps = new Component[nr][1];
      final ButtonGroup radio = new ButtonGroup();
      
      // radio button rows
      for (int i = 0; i < nr; i++)
      {
         JRadioButton choice = new JRadioButton(choices[i]);
         comps[i][0] = choice;
         radio.add(choice);
      }
      ((JRadioButton)comps[0][0]).setSelected(true);
      
      // radio buttons layout
      JPanel rpanel = AdminUtils.groupComponents(comps, question, false);
      
      // buttons
      JButton bGo  = new JButton(goButton);
      JButton bCan = new JButton("Cancel");
      
      // buttons layout
      JPanel bpanel = AdminUtils.borderComponents(
                      new Component[] {bGo, new JLabel(""), bCan}, false);
      
      // whole panel
      JPanel panel = AdminUtils.borderComponents(
                                      new Component[] {rpanel, bpanel}, true);
      
      // create modal dialog
      final JDialog dlg = modalDialog(client, title, panel);
      
      // attach event handlers
      // Cancel button
      bCan.addActionListener(new ActionListener() 
      {
         @SuppressWarnings("unused") 
         public void actionPerformed(ActionEvent evt) 
         {
            // handle cancel
            dlg.setVisible(false);
            radio.clearSelection();
         }
      });
      
      // Go button
      bGo.addActionListener(new ActionListener() 
      {
         @SuppressWarnings("unused") 
         public void actionPerformed(ActionEvent evt) 
         {
            // handle go
            dlg.setVisible(false);
         }
      });
      
      // display modal dialog
      dlg.setVisible(true);
      
      // extracting the choice
      for (int i = 0; i < nr; i++)
      {
         if (((JRadioButton)comps[i][0]).isSelected())
         {
            // return the choice
            return i;
         }
      }
      
      // return cancel
      return -1;
   }
   
   /**
    * Displays an error dialog.
    * 
    * @param    client
    *           instance of AdminClient to use
    * 
    * @param    title
    *           dialog title
    * 
    * @param    message
    *           error message to be displayed
    */
   public static void errorDialog(AdminClient client, 
                                  String title, 
                                  String message)
   {
      JOptionPane.showMessageDialog(client, message, title, 
                                    JOptionPane.ERROR_MESSAGE);
   }
   
   /**
    * Displays confirmation dialog, with Yes/No/Cancel buttons.
    * 
    * @param    client
    *           instance of AdminClient to use
    * 
    * @param    title
    *           dialog title
    * 
    * @param    question
    *           confirmation question assuming continuation on YES answer
    * 
    * @return   <code>null</code> if Cancel is selected,
    *           <code>Boolean.TRUE</code> if Yes is selected,
    *           <code>Boolean.FALSE</code> else.
    */
   public static Boolean confirmCancelDialog(AdminClient client, 
                                             String title, 
                                             String question)
   {
      int rc = JOptionPane.showConfirmDialog(client, question, title,
            JOptionPane.YES_NO_CANCEL_OPTION, 
            JOptionPane.WARNING_MESSAGE);

      if (rc == JOptionPane.YES_OPTION)
      {
         return Boolean.TRUE;
      }
      if (rc == JOptionPane.NO_OPTION)
      {
         return Boolean.FALSE;
      }
      
      return null;
   }

   /**
    * Displays About dialog.
    * 
    * @param    client
    *           instance of AdminClient to use
    */
   static void aboutDialog(AdminClient client)
   {
      String[] about = new String[]
      {
         "P2J Admin Client Applet drop 21.03  ",
         "                                    ",
         "A graphical interface for P2J server",
         "adminstration tasks.                "         
      };
      
      JOptionPane.showMessageDialog(client,
                                    about,
                                    "About Admin",
                                    JOptionPane.PLAIN_MESSAGE);
   }
   
   /**
    * Create a table row filter based on the specific text to be matched.
    *
    * @param    filter
    *           Text to match on a case-insensitive basis.
    *
    * @return   The row filter.
    */
   @SuppressWarnings("unchecked")
   static RowFilter createFilter(String filter)
   {
      StringBuilder sb = new StringBuilder(".*");
      
      for (int i = 0; i < filter.length(); i++)
      {
         char c = filter.charAt(i);
         
         sb.append("[");
         
         if (Character.isUpperCase(c))
         {
            sb.append(Character.toLowerCase(c));
            sb.append(c);
         }
         else
         {
            sb.append(c);
            sb.append(Character.toUpperCase(c));
         }
         
         sb.append("]");
      }
      sb.append(".*");
      
      return RowFilter.regexFilter(sb.toString());
   }

   /**
    * Remove all action listeners for the specified component.
    * 
    * @param    comp
    *           the component for which the action listeners need to be
    *           removed.
    */
   static void removeActionListeners(AbstractButton comp)
   {
      removeActionListeners(new AbstractButton[] { comp });
   }
   
   /**
    * Remove all action listeners for the specified components.
    * 
    * @param    comps
    *           the components for which the action listeners need to be
    *           removed.
    */
   static void removeActionListeners(AbstractButton[] comps)
   {
      for (int i = 0; i < comps.length; i++)
      {
         ActionListener[] lst = comps[i].getActionListeners();
         for (int j = 0; j < lst.length; j++)
         {
            comps[i].removeActionListener(lst[j]);
         }
      }
   }

   /**
    * Queries the menu and returns all the items.
    * 
    * @param    menu
    *           menu of interest
    * 
    * @return   array of menu items
    */
   static JMenuItem[] getMenuItems(JMenu menu)
   {
      JMenuItem[] items = new JMenuItem[menu.getItemCount()];
      for (int i = 0; i < items.length; i++)
      {
         items[i] = menu.getItem(i);
      }
      
      return items;
   }
   
   /**
    * Looks up a string in an array and returns its index.
    * 
    * @param    what
    *           string to look up
    * 
    * @param    where
    *           array of strings
    * 
    * @return   index of the string or <code>-1</code> if not found
    */
   public static int isListed(String what, String[] where)
   {
      // protection
      if (what == null || where == null)
      {
         return -1;
      }
      
      // lookup
      for (int i = 0; i < where.length; i++)
      {
         if (what.equals(where[i]))
         {
             return i;
         }
      }
      
      // not found
      return -1;
   }
   
   /**
    * Layout the specified components using group layout.
    * 
    * @param    comps
    * 
    * @return   panel with provided components grouped on it.
    */
   public static JPanel groupComponents(Component[][] comps)
   {
      return groupComponents(comps, null, false);
   }

   /**
    * Layout the specified components using group layout.
    * 
    * @param    comps
    *           The specified components.
    * @param    title
    *           If not null, it will set a titled-border to the panel.
    * 
    * @return   panel with provided components grouped on it.
    */
   public static JPanel groupComponents(Component[][] comps, 
                                        String title)
   {
      return groupComponents(comps, title, false);
   }

   /**
    * Layout the specified components using group layout.
    * 
    * @param    comps
    *           The specified components.
    * @param    linkSize
    *           <code>true</code> if widget size should be linked.
    *           
    * @return   panel with provided components grouped on it.
    */
   public static JPanel groupComponents(Component[][] comps, 
                                        boolean linkSize)
   {
      return groupComponents(comps, null, linkSize);
   }

   /**
    * Layout the specified components using group layout.
    *
    * @param    comps
    *           The specified components.
    * @param    title
    *           If not null, it will set a titled-border to the panel.
    * @param    linkSize
    *           <code>true</code> if widget size should be linked.
    *
    * @return   panel with provided components grouped on it.
    */
   public static JPanel groupComponents(Component[][] comps,
                                        String title,
                                        boolean linkSize)
   {
      return groupComponents(comps, title, linkSize,
                             GroupLayout.Alignment.BASELINE);
   }

   /**
    * Layout the specified components using group layout.
    * 
    * @param    comps
    *           The specified components.
    * @param    title
    *           If not null, it will set a titled-border to the panel.
    * @param    linkSize
    *           <code>true</code> if widget size should be linked.
    * @param    verticalAlign
    *           Vertical alignment for components.
    *
    * @return   panel with provided components grouped on it.
    */
   public static JPanel groupComponents(Component[][] comps, 
                                        String title,
                                        boolean linkSize,
                                        GroupLayout.Alignment verticalAlign)
   {
      if (title != null)
      {
         title = " "  + title.trim() + " ";
      }

      JPanel pnl = new JPanel();
      
      GroupLayout layout = new GroupLayout(pnl);
      if (title != null)
      {
         pnl.setBorder(new TitledBorder
               (new EtchedBorder(EtchedBorder.RAISED), title));
      }
      pnl.setLayout(layout);

      // Turn on automatically adding gaps
      layout.setAutoCreateGaps(true);
      layout.setAutoCreateContainerGaps(true);
      
      // Create a sequential group for the horizontal axis.
      {
         GroupLayout.SequentialGroup hGroup = layout.
                                              createSequentialGroup();
         
         for (int j = 0; j < comps[0].length; j++)
         {
            GroupLayout.ParallelGroup pGroup = layout.createParallelGroup();
            
            for (int i = 0; i < comps.length; i++)
            {
               pGroup.addComponent(comps[i][j]);
            }
            
            hGroup.addGroup(pGroup);
         }

         layout.setHorizontalGroup(hGroup);
      }

      // Create a sequential group for the vertical axis.
      {
         GroupLayout.SequentialGroup vGroup = layout.
                                              createSequentialGroup();
       
         for (int i = 0; i < comps.length; i++)
         {
            GroupLayout.ParallelGroup pGroup = 
               layout.createParallelGroup(verticalAlign);
            
            for (int j = 0; j < comps[i].length; j++)
            {
               pGroup.addComponent(comps[i][j]);
            }
            
            vGroup.addGroup(pGroup);
         }
         
         layout.setVerticalGroup(vGroup);
      }
      
      if (linkSize)
      {
         for (int j = 0; j < comps[0].length; j++)
         {
            Component[] vcomps = new Component[comps.length];
            
            for (int i = 0; i < comps.length; i++)
            {
               vcomps[i] = comps[i][j];
            }
            
            layout.linkSize(vcomps);
         }
      }
      
      return pnl;
   }

   /**
    * Layout the specified components using border layout.
    * 
    * @param    comps
    *           The specified components.
    * @param    vert
    *           <code>true</code> if the components should layout vertically;
    *           <code>false</code> otherwise.
    * 
    * @return   panel with provided components grouped on it.
    */
   public static JPanel borderComponents(Component[] comps, boolean vert)
   {
      JPanel pnl = new JPanel();

      Object dir1 = vert ? BorderLayout.NORTH : BorderLayout.WEST;
      Object dir2 = vert ? BorderLayout.SOUTH : BorderLayout.EAST;
      
      pnl.setLayout(new BorderLayout());
         
      for (int j = 0; j < comps.length; j++)
      {
         Object pos = BorderLayout.NORTH;
         
         switch (comps.length)
         {
            case 1:
               pos = BorderLayout.CENTER;
               break;
            case 2:
               switch (j)
               {
                  case 0:
                     pos = dir1;
                     break;
                  case 1:
                     pos = BorderLayout.CENTER;
                     break;
               }                  
               break;
            case 3:
               switch (j)
               {
                  case 0:
                     pos = dir1;
                     break;
                  case 1:
                     pos = BorderLayout.CENTER;
                     break;
                  case 2:
                     pos = dir2;
                     break;
               }                  
               break;
         }
         
         pnl.add(comps[j], pos);
      }
         
      return pnl;
   }

   /**
    * Create a bordered and scrolled table.
    *
    * @param  borderTitle
    *         Border title.
    * @param  dataModel
    *         Table data model.
    * @param  parentScrollPane
    *         Scroll pane used for the table.
    *
    * @return bordered and scrolled table.
    */
   public static JTable createBorderedScrolledTable(String borderTitle,
                                              TableModel dataModel,
                                              JScrollPane parentScrollPane)
   {
      JTable table = new JTable(dataModel);
      table.getTableHeader().setReorderingAllowed(false);
      table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

      parentScrollPane.getViewport().add(table);
      parentScrollPane.setBorder(new TitledBorder
               (new EtchedBorder(EtchedBorder.RAISED), borderTitle));

      parentScrollPane.setViewportBorder(new EmptyBorder(5, 5, 5, 5));
      return table;
   }

   /**
    * Create read-only text area.
    *
    * @param  text
    *         Text to be displayed in the text area.
    *
    * @return read-only text area with the specified text.
    */
   public static JTextArea createReadOnlyTextArea(String text)
   {
      JTextArea res = new JTextArea(text);
      res.setLineWrap(true);
      res.setWrapStyleWord(true);
      res.setEditable(false);
      res.setCursor(null);
      res.setOpaque(false);
      res.setFocusable(false);
      return res;
   }

   /**
    * Create read-only text field.
    *
    * @param  text
    *         Text to be displayed in the text field.
    * @param  size
    *         Text field size.
    *
    * @return read-only text field with the specified text.
    */
   public static JTextField createReadOnlyTextField(String text, int size)
   {
      JTextField res = new JTextField(text, size);
      res.setEditable(false);
      res.setCursor(null);
      res.setOpaque(false);
      res.setFocusable(false);
      return res;
   }
   
   /**
    * Account information record for printing.
    * The number of extended attributes is determined at runtime.
    */
   static class AccountInfo
   {
      /** account id */
      String aid;
      
      /** account description */
      String descr;
      
      /** disabled account flag */
      String disabled;
      
      /** assigned certificate */
      String alias;
      
      /**
       * account attributes 1:
       * password protection status for users;
       * application vs server process for processes
       */
      String attr1;
      
      /**
       * account attributes 2:
       * groups for this user
       * master process for processes
       */
      String attr2;
      
      /** extended account attributes */
      String[] ext;
   }
   
   /**
    * Provides a table data model to represent printable user and process
    * accounts.
    */
   static class PrintAccountModel
   extends AbstractTableModel
   {
      /** Standard column names are enumerated here */
      private final String[] stdColumnNames =
      {
         "Account ID",
         "Description",
         "Disabled",
         "Certificate",
         "Status",
         "Groups",
      };
      
      /** Extended column names will be stored here */
      private final String[] extColumnNames;
      
      /** Standard column sizes are enumerated here */
      private final int stdColumnSizes[] =
      {
         35,
         70,
         20,
         35,
         45,
         45,
      };
      
      /** Extended column sizes are all the same size specified here */
      private final int extColumnSize = 50;
      
      /** Core data for the table. */
      private AccountInfo[] data = null;
   
      /** calculated total number of columns in the table */
      private final int columns;
      
      /**
       * Construct an instance.
       *
       * @param    extraHeaders
       *           additional headers for extended columns; may be an empty
       *           array or even <code>null</code>
       *
       * @param    data
       *           The data to provide to the table.
       */
      public PrintAccountModel(String[] extraHeaders, AccountInfo[] data)
      {
         // check the extra headers and calculate the total columns
         int extras = extraHeaders != null && extraHeaders.length > 0 ?
                      extraHeaders.length : 0;
                      
         columns = stdColumnNames.length + extras;
         
         // store the extra headers
         if (extras > 0)
         {
            extColumnNames = extraHeaders;
         }
         else
         {
            extColumnNames = new String[] {};
         }
         
         // store the data
         this.data = data;
      }
      
      /**
       * Gets the number of columns in the table model.
       *
       * @return   The number of columns.
       */
      public int getColumnCount()
      {
         return columns;
      }
   
      /**
       * Gets the width of the given column.
       *
       * @param    col
       *           The column index (0-based).
       *
       * @return   The width in pixels.
       */
      public int getColumnWidth(int col)
      {
         if (col < stdColumnNames.length)
         {
            return stdColumnSizes[col];
         }
         
         return extColumnSize;
      }
   
      /**
       * Gets the number of rows in the table model.
       *
       * @return   The number of rows.
       */
      public int getRowCount()
      {
         return data.length;
      }
   
      /**
       * Gets the name of a specific column.
       * 
       * @param    col
       *           The column index (0-based).
       * 
       * @return   The column name.
       */
      public String getColumnName(int col)
      {
         if (col < stdColumnNames.length)
         {
            return stdColumnNames[col];
         }
         
         if (col < columns)
         {
            return extColumnNames[col - stdColumnNames.length];
         }
         
         return "";
      }
   
      /**
       * Get the cell value to be displayed.
       * 
       * @param    col
       *           The column index (0-based).
       * @param    row
       *           The row index (0-based).
       * 
       * @return   Cell value.
       */
      public Object getValueAt(int row, int col)
      {
         Object datum = null;
         AccountInfo info  = data[row];
         
         if (col >= stdColumnNames.length)
         {
            return info.ext[col - stdColumnNames.length];
         }

         switch (col)
         {
            case 0:
               datum = info.aid;
               break;
            case 1:
               datum = info.descr;
               break;
            case 2:
               datum = info.disabled;
               break;
            case 3:
               datum = info.alias;
               break;
            case 4:
               datum = info.attr1;
               break;
            case 5:
               datum = info.attr2;
         }
         
         return datum;
      }
   
      /**
       * Get the cell value to be displayed.
       *
       * @param    value
       *           value to be stored in the cell
       *
       * @param    col
       *           The column index (0-based).
       *
       * @param    row
       *           The row index (0-based).
       */
      public void setValueAt(Object value, int row, int col)
      {
         AccountInfo info = data[row];
         
         if (col >= stdColumnNames.length)
         {
            info.ext[col - stdColumnNames.length] = (String)value;
         }

         switch (col)
         {
            case 0:
               info.aid = (String)value;
               break;
            case 1:
               info.descr = (String)value;
               break;
            case 2:
               info.disabled = (String)value;
               break;
            case 3:
               info.alias = (String)value;
               break;
            case 4:
               info.attr1 = (String)value;
               break;
            case 5:
               info.attr2 = (String)value;
         }
      }
   
      /**
       * Gets the data type of a specific column.
       *
       * @param    col
       *           The column index (0-based).
       *
       * @return   The column type, which is always String
       */
      public Class<?> getColumnClass(int col)
      {
         return String.class;
      }
      
      /**
       * Reloads the entire table with new data.
       *
       * @param    data
       *           The data to provide to the table.
       */
      public void reload(AccountInfo[] data)
      {
         this.data = data;
         fireTableDataChanged();
      }
   }

   /**
    * Provides a table data model to represent extended printable user and 
    * process accounts.
    */
   static class PrintExtendedAccountModel
   extends AbstractTableModel
   {
      /** Standard column names */
      private final String[] stdColumnNames;
      
      /** Extended column names */
      private final String[] extColumnNames;
      
      /** Core data for the table. */
      private String[][] data = null;
   
      /** calculated total number of columns in the table */
      private final int columns;
      
      /**
       * Construct an instance.
       *
       * @param    stdHeaders
       *           headers for the standard part of the table
       *
       * @param    extHeaders
       *           additional headers for extended columns; may be an empty
       *           array or even <code>null</code>
       *
       * @param    data
       *           The data to provide to the table.
       */
      public PrintExtendedAccountModel(String[] stdHeaders, String[] extHeaders, 
                                       String[][] data)
      {
         // check the extra headers and calculate the total columns
         int extras = extHeaders != null && extHeaders.length > 0 ?
                      extHeaders.length : 0;
                      
         columns = stdHeaders.length + extras;
         
         // store the headers
         stdColumnNames = stdHeaders;
         
         // store the extra headers
         if (extras > 0)
         {
            extColumnNames = extHeaders;
         }
         else
         {
            extColumnNames = new String[] {};
         }
         
         // store the data
         this.data = data;
      }
      
      /**
       * Gets the number of columns in the table model.
       *
       * @return   The number of columns.
       */
      public int getColumnCount()
      {
         return columns;
      }
   
      /**
       * Gets the width of the given column.
       *
       * @param    col
       *           The column index (0-based).
       *
       * @return   The width in pixels.
       */
      public int getColumnWidth(int col)
      {
         // hard coded value
         return 100;
      }
   
      /**
       * Gets the number of rows in the table model.
       *
       * @return   The number of rows.
       */
      public int getRowCount()
      {
         return data.length;
      }
   
      /**
       * Gets the name of a specific column.
       *
       * @param    col
       *           The column index (0-based).
       *
       * @return   The column name.
       */
      public String getColumnName(int col)
      {
         if (col < stdColumnNames.length)
         {
            return stdColumnNames[col];
         }
         
         if (col < columns)
         {
            return extColumnNames[col - stdColumnNames.length];
         }
         
         return "";
      }
   
      /**
       * Get the cell value to be displayed.
       * 
       * @param    col
       *           The column index (0-based).
       * @param    row
       *           The row index (0-based).
       * 
       * @return   Cell value.
       */
      public Object getValueAt(int row, int col)
      {
         return data[row][col];
      }
   
      /**
       * Get the cell value to be displayed.
       * 
       * @param    value
       *           value to be stored in the cell
       * 
       * @param    col
       *           The column index (0-based).
       * 
       * @param    row
       *           The row index (0-based).
       */
      public void setValueAt(Object value, int row, int col)
      {
         data[row][col] = (String)value;
      }
   
      /**
       * Gets the data type of a specific column.
       * 
       * @param    col
       *           The column index (0-based).
       * 
       * @return   The column type, which is always String
       */
      public Class<?> getColumnClass(int col)
      {
         return String.class;
      }
      
      /**
       * Reloads the entire table with new data.
       * 
       * @param    data
       *           The data to provide to the table.
       */
      public void reload(String[][] data)
      {
         this.data = data;
         fireTableDataChanged();
      }
   }
}