/*
** Module   : AttributeMethodDescriptor.java
** Abstract : small container class used to store conversion mapping info for 4GL attributes and
**            4GL methods
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description--------------------------------
** 001 GES 20141012 First version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.convert;

/**
 * This is a small container class used to store conversion mapping info for 4GL attributes and
 * 4GL methods.
 */
public class AttributeMethodDescriptor
{
   /** The type of the resource used for unwrapping handles. */
   public String wrapper = null;
   
   /** Java method used as a getter for attributes and the only thing called for 4GL methods. */
   public String getter = null;
   
   /** Java method used as a setter for attribute assignment. */
   public String setter = null;
   
   /** Flag to denote if the getter/setters are static or instance methods. */
   public boolean instance = true;
   
   /**
    * Obtain the type of the resource used for unwrapping handles.
    *
    * @return   The resource type.
    */
   public String getWrapper()
   {
      return wrapper;
   }
   
   /**
    * Assign the type of the resource used for unwrapping handles.
    *
    * @param   wrapper
    *          The resource type.
    */
   public void setWrapper(String wrapper)
   {
      this.wrapper = wrapper; 
   }
   
   /**
    * Obtain the Java method used as a getter for attributes and the only thing called for 4GL
    * methods.
    *
    * @return   The method name.
    */
   public String getGetter()
   {
      return getter; 
   }
   
   /**
    * Assign the Java method used as a getter for attributes and the only thing called for 4GL
    * methods.
    *
    * @param   getter
    *          The method name.
    */
   public void setGetter(String getter)
   {
      this.getter = getter; 
   }
   
   /**
    * Obtain the Java method used as a setter for attributes.
    *
    * @return   The method name.
    */
   public String getSetter()
   {
      return setter;
   }
   
   /**
    * Assign the Java method used as a setter for attributes.
    *
    * @param   setter
    *          The method name.
    */
   public void setSetter(String setter)
   {
      this.setter = setter;
   }

   /**
    * Obtain the flag that denotes if the getter/setters are static or instance methods.
    *
    * @return   <code>true</code> if this is an instance method.
    */
   public boolean getInstance()
   {
      return instance;
   }
   
   /**
    * Assign the flag that denotes if the getter/setters are static or instance methods.
    *
    * @param   instance
    *          <code>true</code> if this is an instance method.
    */
   public void setInstance(boolean instance)
   {
      this.instance = instance;
   }
}