/*
** Module   : NativeSignedInt32.java
** Abstract : descriptor for a signed 32-bit integer 
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20131212 First version.
** 002 GES 20140212 Non-scalar instances must emit as a sign-extended value of the platform's
**                  word-size instead of the actual size of the instance.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.library;

import com.goldencode.p2j.util.*;

/**
 * A descriptor for a signed short.
 */
public class NativeSignedInt32 
extends NativeInteger
{
   /**
    * Default constructor.
    */
   public NativeSignedInt32()
   {
      super();
      type = LibraryManager.TYPE_SIGNED_INT32;
   }
   
   /**
    * Report the size (in bytes) of the native memory needed to store this instance.
    * <p>
    * This should never be called on the server, only on the client side.
    *
    * @return   The number of bytes needed.
    */
   public int size()
   {
      return isInArray() ? 4 : LibraryManager.ptrSize();
   }
   
   /**
    * Worker method to read the primitive value from the first index position of the given
    * memory buffer.
    *
    * @param    ptr
    *           The memory buffer to reference.
    */
   protected void readNative(memptr ptr)
   {
      integer num = ptr.getLong(1);
      
      if (!num.isUnknown())
      {
         value = num.longValue();
      }
   }
   
   /**
    * Worker method to write the primitive value to the first index position of the given
    * memory buffer.
    *
    * @param    ptr
    *           The memory buffer to reference.
    */
   protected void writeNative(memptr ptr)
   {
      if (isInArray())
      {
         ptr.setLong(value, 1);
      }
      else
      {
         writeNativeWordSize(ptr);
      }
   }
}