/*
** Module   : TrustedSpawnerResource.java
** Abstract : plugin to manage access to trusted spawning
**
** Copyright (c) 2016-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------------------Description-----------------------------------
** 001 GES 20160222 Created the first version.
** 002 CA  20161123 Fixed package for getRightsEditorName().
*/ 
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.main;

import com.goldencode.p2j.security.*;

/**
 * Implements the trusted spawner target account resource. This controls which OS account names 
 * can be used for spawning clients in trusted mode.
 * <p>
 * When using exact matching, this must match the target OS account name.  It is also valid to
 * implement inexact (regex) matches as well.  This would allow all possible matches (very 
 * insecure!):
 * <p>
 * <pre>
 * .*
 * </pre>
 */
public class TrustedSpawnerResource
extends StringConditionResource
{
   /**
    * Default constructor. 
    */
   public TrustedSpawnerResource()
   {
   }

   /**
    * Returns the plugin resource type name as a string.
    *
    * @return   The plugin resource type name.
    */
   public String getTypeName()
   {
      return "trustedspawner";
   }

   /**
    * Returns the name of a class that implements the <code>RightsEditor</code> interface for
    * this resource type.
    *
    * @return   The name of the class implementing <code>RightsEditor</code> for this resource.
    */
   public String getRightsEditorName()
   {
      return "com.goldencode.p2j.main.TrustedSpawnerRightsEditor";
   }
    
   /**
    * Instantiates a plugin's class that implements the <code>Rights</code> interface, using
    * the array of objects representing a set of access rights fields.
    *
    * @param    rights
    *           The objects needed to create an access rights instance.
    *           
    * @return   The newly created rights instance.
    */
   public Rights getRightsInstance(Object[] rights)
   {
      return new TrustedSpawnerRights((String)rights[0]);
   }
}