/*
** Module   : NodeInfo.java
** Abstract : describes a node in the network
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 SIY 20050120 ADD   @19499 Created initial version.
** 002 SIY 20050208 CHG   @19710 Changed some typos in comments.
** 003 SIY 20050315 CHG   @20384 Minor cleanups.
** 004 GES 20061222 CHG   @31791 More cleanup.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.net;

import java.net.*;
import java.io.*;

/**
 * This class is a simple container that describes a single node in the
 * network.  At this time it contains the node address, name and a list
 * of internet protocol addresses.
 * <p>
 * <b>WARNING: this class is not yet done.  At a minimum, port information
 * will need to be added to this class for each IP address.  Otherwise this
 * information is really of no use for making connections.  Likewise, it
 * is probable that a list of queues would be useful to allow routing
 * decisions to be made.  Even the above referenced data does not easily
 * provide for mapping an entire network from a single location as is
 * the concept behind the address server (which is the primary user of this
 * class). For this reason, this class may be removed or changed in the
 * future.</b>
 */
public final class NodeInfo
implements Serializable
{
   /** Node address. */
   private int address;

   /** List of internet addresses assigned to node. */
   private InetAddress[] ipList;

   /** Node name. */
   private String name;

   /**
    * Constructs an empty <code>NodeInfo</code> instance.
    */
   public NodeInfo()
   {
      this(0, null, null);
   }

   /**
    * Constructs an instance of <code>NodeInfo</code> with assigned node
    * address.
    * 
    * @param address
    *           Node address.
    */
   public NodeInfo(int address)
   {
      this(address, null, null);
   }

   /**
    * Constructs an instance of <code>NodeInfo</code> with assigned node
    * name.
    * 
    * @param    name
    *           Node name.
    */
   public NodeInfo(String name)
   {
      this(0, name, null);
   }

   /**
    * Constructs a completely filled instance of <code>NodeInfo</code>.
    * 
    * @param    address
    *           Node address.
    * @param    name
    *           Node name.
    * @param    ipList
    *           List of internet protocol addresses.
    */
   public NodeInfo(int address, String name, InetAddress[] ipList)
   {
      this.address = address;
      this.name    = name;
      this.ipList  = ipList;
   }

   /**
    * Provides access to the node address.
    * 
    * @return   The node address.
    */
   public int getAddress()
   {
      return address;
   }

   /**
    * Assign new node address.
    * 
    * @param    address
    *           Node address.
    */
   public void setAddress(int address)
   {
      this.address = address;
   }

   /**
    * Provides access to the node name.
    * 
    * @return   The node name.
    */
   public String getName()
   {
      return name;
   }
   
   /**
    * Assign new node name.
    * 
    * @param    name
    *           Node name.
    */
   public void setName(String name)
   {
      this.name = name;
   }
   
   /**
    * Provides access to list of internet protocol addresses assigned to node.
    * 
    * @return   The list of internet protocol addresses.
    */
   public InetAddress[] getIpList()
   {
      return ipList;
   }

   /**
    * Assign new list of internet addresses.
    * 
    * @param    ipList
    *           List of internet addresses.
    */
   public void setIpList(InetAddress[] ipList)
   {
      this.ipList = ipList;
   }
}