/*
** Module   : RuleListElement.java
** Abstract : represents a generic element in a list of rules/rule containers 
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 GES 20050602 ADD   @21384 Created initial version which represents a
**                               generic element in a list of rules/rule
**                               containers.
** 002 ECF 20080730 CHG   @39265 Added cleanup() method. Provides a hook for
**                               termination processing.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.pattern;

/**
 * Represents a generic element in a list of rules/rule containers. This
 * allows each <code>RuleContainer</code> to maintain an ordered list of
 * elements that can be either a <code>Rule</code> or another
 * <code>RuleContainer</code>.  This makes <code>RuleContainer</code>
 * instances arbitrarily nestable.
 */
interface RuleListElement
{
   /**
    * Indicates whether this object either represents a rule of the specified
    * type OR contains any rules of the specified type.
    *
    * @param   type
    *          Type of rule for which to search: init, walk, or post, using
    *          the <code>RULE_*</code> constants.
    *
    * @return  <code>true</code> if the object is a rule of the specified
    *          type or if the object's list of rules contains any rule of
    *          the specified type, else <code>false</code>.
    */
   public boolean hasType(int type);
   
   /**
    * Reports whether this instance is a rule container.
    *
    * @return  <code>true</code> if this instance is a rule container.
    */
   public boolean isContainer();
   
   /**
    * Apply every rule of the specified type, in the order they were added
    * to this container OR if this instance is a rule of the given type,
    * apply it and return.
    *
    * @param   type
    *          Type of rule to apply: init, walk, or post, using the
    *          <code>RULE_*</code> constants.
    */
   public void apply(int type);
   
   /**
    * Termination hook to allow resources to be cleaned up when this object is
    * about to go out of service.
    */
   public void cleanup();
}