/*
** Module   : OffEnd.java
** Abstract : Enumeration of the various query off-end states
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description---------------------
** 001 ECF 20071204  @36263  Created initial version. Enumeration of the
**                           various query off-end states.
** 002 SVL 20080418  @38039  Added inverse() function.
** 003 SVL 20091030  @44292  The enumeration was made public.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.persist;

/**
 * An enumeration of the various query off-end states:
 * <ul>
 *   <li>not off-end
 *   <li>off the front end of a result list
 *   <li>off the back end of a result list
 * </ul>
 * <p>
 * This information is used in combination with a query's last used sort
 * index, to ensure dynamic queries start their searches at the appropriate
 * record.
 * 
 * @see  SortIndex
 * @see  RecordBuffer#getOffEnd(SortIndex, boolean)
 */
public enum OffEnd
{
   /** Query is not off-end */
   NONE,
   
   /** Query is off front end of results */
   FRONT,
   
   /** Query is off back end of results */
   BACK;
   
   /**
    * Returns inverse off-end.
    *
    * @return <code>FRONT</code> if the off-end is <code>BACK</code>.
    *         <code>BACK</code> if the off-end is <code>FRONT</code>.
    *         <code>NONE</code> otherwise.
    */
   public OffEnd inverse()
   {
      if (this == FRONT)
      {
         return BACK;
      }
      else if (this == BACK)
      {
         return FRONT;
      }
      
      return NONE;
   }
}