/*
** Module   : ErrorHandler.cs
** Abstract : Delegates the CLR decorated methods to overloaded operators from converted assembly.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -------------------------------- Description --------------------------------
** 001 OM  20140811 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

/* In this version we are using some deprecated API. Disable the warnings for the moment. */
#pragma warning disable 0612

using System;

/**
 * This class delegates two calls to converted classes.
 */
public class ErrorHandler
{
    /**
     * The constructor is private. No instances of this class are allowed. All access is done
     * using the static methods.
     */
    private ErrorHandler()
    {
    }

   /**
    * Forwards the call to <code>com.goldencode.p2j.persist.pl.ErrorHandler</code> which will 
    * initialize the error flag stack for the current function scope by pushing the given error
    * state onto the stack.
    * 
    * @param   state
    *          Error state to push onto the stack.
    * 
    * @return  <code>True</code>.
    */
    public static Boolean initError_b(Boolean state)
    {
        return com.goldencode.p2j.persist.pl.ErrorHandler.initError(
                  Common.toBoolean(state)).booleanValue();
    }

    /**
     * A special wrapper function for a function implemented in Java.
     * for details see com.goldencode.p2j.persist.pl.ErrorHandler.checkError(Boolean, Boolean).
     * 
     * @param   init
     *          The result of the <code>initError(false)</code> function called as the
     *          first parameter; not used.
     * @param   result
     *          The result of executing the boolean expression containing the Java function or
     *          operator. This result indicates the success/failure of the enclosed, boolean 
     *          sub-expression.
     * 
     * @return  <code>false</code> if an error occurred in this scope; otherwise, the result of
     *          the boolean sub-expression.
     */
    public static Boolean checkError_bb(Boolean init, Boolean? result)
    {
        return com.goldencode.p2j.persist.pl.ErrorHandler.checkError(
                  Common.toBoolean(init), Common.toBoolean(result)).booleanValue();
    }
}