/*
** Module   : HintsTags.java
** Abstract : Defines tags used in preprocessor hints XML documents.
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 NVS 20050527 ADD   @21317 Created. Initial implementation.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.preproc;

/**
 * Defines tags used in preprocessor hints XML documents.
 * <p>
 * @author NVS
 * @version 1.0.0
 */
public interface HintsTags
{
   /** XML tag for the root element */
   static final String ELEM_TAG_ROOT = "hints";

   /** XML tag for the subroot element */
   static final String ELEM_TAG_SUBROOT = "preprocessor-output";

   /** XML tag for the include hint element */
   static final String ELEM_TAG_INCL = "include";

   /** XML tag for the file atribute of include hint element */
   static final String ATTR_TAG_FILE = "file";

   /** XML tag for the start-line atribute of include hint element */
   static final String ATTR_TAG_START_LINE = "start-line";

   /** XML tag for the start-column atribute of include hint element */
   static final String ATTR_TAG_START_COL = "start-column";

   /** XML tag for the end-line atribute of include hint element */
   static final String ATTR_TAG_END_LINE = "end-line";

   /** XML tag for the end-column atribute of include hint element */
   static final String ATTR_TAG_END_COL = "end-column";

   /** XML tag for the argument hint element */
   static final String ELEM_TAG_ARG = "argument";

   /** XML tag for the name atribute of argument hint element */
   static final String ATTR_TAG_ARG_NAME = "name";

   /** XML tag for the position atribute of argument hint element */
   static final String ATTR_TAG_ARG_POS = "position";

   /** XML tag for the value atribute of argument hint element */
   static final String ATTR_TAG_ARG_VAL = "value";

   /** XML tag for the used atribute of argument hint element */
   static final String ATTR_TAG_ARG_USED = "used";

   /** XML tag for the reference hint element */
   static final String ELEM_TAG_REF = "reference";

   /** XML tag for the name atribute of reference hint element */
   static final String ATTR_TAG_REF_NAME = "name";

   /** XML tag for the value atribute of reference hint element */
   static final String ATTR_TAG_REF_VAL = "value";

   /** XML tag for the type atribute of reference hint element */
   static final String ATTR_TAG_REF_TYPE = "type";

   /** XML tag for the line atribute of reference hint element */
   static final String ATTR_TAG_REF_LINE = "line";

   /** XML tag for the column atribute of reference hint element */
   static final String ATTR_TAG_REF_COL = "column";
}