/*
** Module   : AdminRightsEditor.java
** Abstract : Rights editor for the "admin" type resources.
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20090707  @44095  Created initial version.
** 002 SIY 20091216  @44516  Added support for "refresh" type.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.security;

import com.goldencode.p2j.directory.*;
import com.goldencode.p2j.admin.client.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Edits the "admin" rights for the admin client.
 * 
 * @author NVS
 */
public class AdminRightsEditor
implements RightsEditor
{ 
   /** instance of AdminClient */
   private AdminClient client = null;
   
   /** generic path permissions editor panel */
   private JPanel editPath = null;
   
   /** Unlimited access check box */
   private JCheckBox pUnlimited = null;
   
   /** Denied access check box */
   private JCheckBox pDenied = null;
   
   /** Save button */
   private JButton pbsave = null;
   
   /** Cancel button */
   private JButton pbcanc = null;
   
   /** refresh permissions editor panel */
   private JPanel editRefresh = null;
   
   /** Allow refresh check box */
   private JCheckBox rAllowed = null;

   /** Denied access check box */
   private JCheckBox rDenied = null;

   /** Save button */
   private JButton rbcanc = null;

   /** Cancel button */
   private JButton rbsave = null;
   
   /** user account permissions editor panel */
   private JPanel editUser = null;
   
   /** UA Enumerate permission check box */
   private JCheckBox uEnumerate = null;
   
   /** UA Password permission check box */
   private JCheckBox uPassword = null;
   
   /** UA Group permission check box */
   private JCheckBox uGroup = null;
   
   /** UA Read permission check box */
   private JCheckBox uRead = null;
   
   /** UA Write permission check box */
   private JCheckBox uWrite = null;
   
   /** UA Create permission check box */
   private JCheckBox uCreate = null;
   
   /** UA Delete permission check box */
   private JCheckBox uDelete = null;
   
   /** UA Denied access check box */
   private JCheckBox uDenied = null;
   
   /** Save button */
   private JButton ubsave = null;
   
   /** Cancel button */
   private JButton ubcanc = null;
   
   /** dialog */
   private JDialog dialog = null;
   
   /** save or cancel flag */
   private boolean saving = false;

   /**
    * Gives chance to editors to initialize their data structures.
    * <p>
    * This is a one time call for this kind of resource for the life time 
    * of the admin session. 
    *
    * @param ac
    *        instance of AdminClient the editor is called from
    *
    * @param description
    *        rights structure description
    */
   public void initialize(AdminClient ac, Description[] description)
   {
      // save the useful reference
      client = ac;
      
      // create the path permissions edit panel
      Component[][] pcomps = new Component[3][2];
      
      // 1st row - Unlimited permission
      pUnlimited = new JCheckBox("U");
      pUnlimited.setToolTipText("allows unlimited access to this level " + 
                                "and below");
      pcomps[0][0] = new JLabel("unlimited");
      pcomps[0][1] = pUnlimited;
      
      // 2nd row - Denied permission
      pDenied = new JCheckBox("N");
      pDenied.setToolTipText("denies any and all accesses to this level " + 
                             "and below");
      pcomps[1][0] = new JLabel("denied");
      pcomps[1][1] = pDenied;
      
      // last row - buttons
      pbcanc = new JButton("Cancel");
      pbsave = new JButton("Save");
      pcomps[2][0] = pbcanc;
      pcomps[2][1] = pbsave;
      
      // layout
      editPath = AdminUtils.groupComponents(pcomps, "path rights", false);

      // create the refresh permissions edit panel
      Component[][] rcomps = new Component[3][2];
      
      // 1st row - Unlimited permission
      rAllowed = new JCheckBox("R");
      rAllowed.setToolTipText("allows access to refresh function");
      rcomps[0][0] = new JLabel("refresh");
      rcomps[0][1] = rAllowed;
      
      // 2nd row - Denied permission
      rDenied = new JCheckBox("N");
      rDenied.setToolTipText("denies accesses to refresh function");
      rcomps[1][0] = new JLabel("denied");
      rcomps[1][1] = rDenied;
      
      // last row - buttons
      rbcanc = new JButton("Cancel");
      rbsave = new JButton("Save");
      rcomps[2][0] = rbcanc;
      rcomps[2][1] = rbsave;
      
      // layout
      editRefresh = AdminUtils.groupComponents(rcomps, "refresh rights", false);
      
      // create the user account permissions edit panel
      Component[][] ucomps = new Component[9][2];
      
      // 1st row - Enumerate permission
      uEnumerate = new JCheckBox("E");
      uEnumerate.setToolTipText("allows enumeration of user accounts");
      ucomps[0][0] = new JLabel("enumerate");
      ucomps[0][1] = uEnumerate;
      
      // 2nd row - Password permission
      uPassword = new JCheckBox("P");
      uPassword.setToolTipText("allows changing passwords");
      ucomps[1][0] = new JLabel("password");
      ucomps[1][1] = uPassword;
      
      // 3rd row - Group permission
      uGroup = new JCheckBox("G");
      uGroup.setToolTipText("allows assigning groups");
      ucomps[2][0] = new JLabel("group");
      ucomps[2][1] = uGroup;
      
      // 4th row - Read access
      uRead = new JCheckBox("R");
      uRead.setToolTipText("allows reading account details");
      ucomps[3][0] = new JLabel("read");
      ucomps[3][1] = uRead;
      
      // 5th row - Write access
      uWrite = new JCheckBox("W");
      uWrite.setToolTipText("allows writing account details");
      ucomps[4][0] = new JLabel("write");
      ucomps[4][1] = uWrite;

      // 6th row - Create access
      uCreate = new JCheckBox("C");
      uCreate.setToolTipText("allows creating new users");
      ucomps[5][0] = new JLabel("create");
      ucomps[5][1] = uCreate;

      // 7th row - Delete access
      uDelete = new JCheckBox("D");
      uDelete.setToolTipText("allows deletion of users");
      ucomps[6][0] = new JLabel("delete");
      ucomps[6][1] = uDelete;

      // 8th row - Denied access
      uDenied = new JCheckBox("N");
      uDenied.setToolTipText("denies any and all accesses to user accounts");
      ucomps[7][0] = new JLabel("denied");
      ucomps[7][1] = uDenied;
      
      // last row - buttons
      ubcanc = new JButton("Cancel");
      ubsave = new JButton("Save");
      ucomps[8][0] = ubcanc;
      ucomps[8][1] = ubsave;
      
      // layout
      editUser = AdminUtils.groupComponents(ucomps, "user account rights", 
                                            false);
      
      // event handlers:
      
      // Path
      
      // Path: unlimited path access check box
      pUnlimited.addItemListener(new ItemListener()
         {
            public void itemStateChanged(ItemEvent evt) 
            {
               handleUnlimitedPath();
            }
         });
      
      // Path: denied path access check box
      pDenied.addItemListener(new ItemListener()
         {
            public void itemStateChanged(ItemEvent evt) 
            {
               handleDeniedPath();
            }
         });
      
      // Path: Cancel button
      pbcanc.addActionListener(new ActionListener() 
         {
            public void actionPerformed(ActionEvent evt) 
            {
               handleCancel();
            }
         });
      
      // Path: Save button
      pbsave.addActionListener(new ActionListener() 
         {
            public void actionPerformed(ActionEvent evt) 
            {
               handleSave();
            }
         });

      // Refresh
      
      // Refresh: unlimited path access check box
      rAllowed.addItemListener(new ItemListener()
         {
            public void itemStateChanged(ItemEvent evt) 
            {
               handleAllowRefresh();
            }
         });
      
      // Refresh: denied path access check box
      rDenied.addItemListener(new ItemListener()
         {
            public void itemStateChanged(ItemEvent evt) 
            {
               handleDeniedRefresh();
            }
         });
      
      // Refresh: Cancel button
      rbcanc.addActionListener(new ActionListener() 
         {
            public void actionPerformed(ActionEvent evt) 
            {
               handleCancel();
            }
         });
      
      // Refresh: Save button
      rbsave.addActionListener(new ActionListener() 
         {
            public void actionPerformed(ActionEvent evt) 
            {
               handleSave();
            }
         });
      
      // User
      
      // User: denied path access check box
      uDenied.addItemListener(new ItemListener()
         {
            public void itemStateChanged(ItemEvent evt) 
            {
               handleDeniedUser();
            }
         });
      
      // User: Cancel button
      ubcanc.addActionListener(new ActionListener() 
         {
            public void actionPerformed(ActionEvent evt) 
            {
               handleCancel();
            }
         });
      
      // User: Save button
      ubsave.addActionListener(new ActionListener() 
         {
            public void actionPerformed(ActionEvent evt) 
            {
               handleSave();
            }
         });
   }

   /**
    * Creates an instance of the <code>Rights</code>, which does not take
    * any input and is considered a default starting point in editing.
    *
    * @return an instance of <code>Rights</code>
    */
   public Rights createDefaultRights()
   {
      return new AdminRights();
   }
   
   /**
    * Edits the given instance of the <code>Rights</code>. 
    * The implementation should prepare its modal dialog using the initial state
    * from the passed instance of the <code>Rights</code>, show the dialog, 
    * provide event processing until the Save or Cancel action is applied, then
    * hide the dialog and return the appropriate result.
    *
    * @param name
    *        name of the resource instance being edited
    *
    * @param exact
    *        nature of the name
    
    * @param rights
    *        instance of <code>Rights</code> to be edited
    *
    * @return the resulting instance of <code>Rights</code> if the user saved
    *         the results, otherwise <code>null</code>
    */
   public Rights edit(String name, boolean exact, Rights rights)
   {
      // determine the nature of the rights and replace the passed instance
      // with the proper one if the default doesn't fit
      AdminResource.Node[] path = AdminResource.findPath(name);
      if (path == null)
      {
         return rights;
      }
      
      int reqType = path[path.length - 1].getType();
      int type = ((AdminRights)rights).getType(); 
      
      AdminRights ar = reqType == type ? (AdminRights)rights :
                                         new AdminRights(reqType);
      type = reqType;
      
      // prefill the edit panel with the current rights
      JPanel panel = null;
      BitField perms = ar.getPermissions();
      
      switch (type)
      {
         case AdminRights.ADMT_PATH:
            {
               boolean unlimited = perms.get(AdminRights.ADTP_UNLIMITED_ACCESS);
               boolean denied = perms.get(AdminRights.ADTP_DENIED_ACCESS);
               if (denied)
               {
                  unlimited = false;
               }
               pUnlimited.setSelected(perms.get(AdminRights.
                                                ADTP_UNLIMITED_ACCESS));
               pDenied.setSelected(denied);
               pUnlimited.setEnabled(!denied);
               pDenied.setEnabled(!unlimited);
               
               panel = editPath;
            }
            break;
            
         case AdminRights.ADMT_LEAF_REFRESH:         
            {
               boolean refresh = perms.get(AdminRights.ADLR_REFRESH_ACCESS);
               boolean denied = perms.get(AdminRights.ADLR_DENIED_ACCESS);
               if (denied)
               {
                  refresh = false;
               }
               rAllowed.setSelected(perms.get(AdminRights.ADLR_REFRESH_ACCESS));
               rDenied.setSelected(denied);
               rAllowed.setEnabled(!denied);
               rDenied.setEnabled(!refresh);
               
               panel = editRefresh;
            }
            break;
            
         case AdminRights.ADMT_LEAF_USER:
            {
               uEnumerate.setSelected(perms.get(AdminRights.
                                                ADLU_ENUMERATE_ACCESS));
               uPassword.setSelected(perms.get(AdminRights.
                                               ADLU_PASSWORD_ACCESS));
               uGroup.setSelected(perms.get(AdminRights.ADLU_GROUP_ACCESS));
               uRead.setSelected(perms.get(AdminRights.ADLU_READ_ACCESS));
               uWrite.setSelected(perms.get(AdminRights.ADLU_WRITE_ACCESS));
               uCreate.setSelected(perms.get(AdminRights.ADLU_CREATE_ACCESS));
               uDelete.setSelected(perms.get(AdminRights.ADLU_DELETE_ACCESS));
               boolean denied = perms.get(AdminRights.ADLU_DENIED_ACCESS);
               uDenied.setSelected(denied);
               uEnumerate.setEnabled(!denied);
               uPassword.setEnabled(!denied);
               uGroup.setEnabled(!denied);
               uRead.setEnabled(!denied);
               uWrite.setEnabled(!denied);
               uCreate.setEnabled(!denied);
               uDelete.setEnabled(!denied);
               uDenied.setEnabled(true);
               
               panel = editUser;
            }
            break;
      }
      
      // display the dialog
      dialog = AdminUtils.modalDialog(client, "Admin Rights", panel);
      dialog.setVisible(true);
      
      // process the results
      dialog = null;
      
      if (!saving)
      {
         return null;
      }
      
      switch (type)
      {
         case AdminRights.ADMT_PATH:
            perms.set(AdminRights.ADTP_UNLIMITED_ACCESS, 
                      pUnlimited.isSelected());
            perms.set(AdminRights.ADTP_DENIED_ACCESS, pDenied.isSelected());
            if (pDenied.isSelected())
            {
               perms.clear(AdminRights.ADTP_UNLIMITED_ACCESS);
            }
            break;
         
         case AdminRights.ADMT_LEAF_REFRESH:
            perms.set(AdminRights.ADLR_REFRESH_ACCESS, rAllowed.isSelected());
            perms.set(AdminRights.ADLR_DENIED_ACCESS, rDenied.isSelected());
            if (rDenied.isSelected())
            {
               perms.clear(AdminRights.ADLR_REFRESH_ACCESS);
            }
            break;
            
         case AdminRights.ADMT_LEAF_USER:
            perms.set(AdminRights.ADLU_ENUMERATE_ACCESS, 
                      uEnumerate.isSelected());
            perms.set(AdminRights.ADLU_PASSWORD_ACCESS, 
                      uPassword.isSelected());
            perms.set(AdminRights.ADLU_GROUP_ACCESS, uGroup.isSelected());
            perms.set(AdminRights.ADLU_READ_ACCESS, uRead.isSelected());
            perms.set(AdminRights.ADLU_WRITE_ACCESS, uWrite.isSelected());
            perms.set(AdminRights.ADLU_CREATE_ACCESS, uCreate.isSelected());
            perms.set(AdminRights.ADLU_DELETE_ACCESS, uDelete.isSelected());
            perms.set(AdminRights.ADLU_DENIED_ACCESS, uDenied.isSelected());
            if (uDenied.isSelected())
            {
               perms.clear(AdminRights.ADLU_ENUMERATE_ACCESS);
               perms.clear(AdminRights.ADLU_PASSWORD_ACCESS);
               perms.clear(AdminRights.ADLU_GROUP_ACCESS);
               perms.clear(AdminRights.ADLU_READ_ACCESS);
               perms.clear(AdminRights.ADLU_WRITE_ACCESS);
               perms.clear(AdminRights.ADLU_CREATE_ACCESS);
               perms.clear(AdminRights.ADLU_DELETE_ACCESS);
            }
            break;
      }
      
      AdminRights nar = new AdminRights(type, perms);
      return nar;
   }
   
   /**
    * Handle Unlimited Path check box selection.
    */
   private void handleUnlimitedPath()
   {
      pDenied.setEnabled(!pUnlimited.isSelected());
   }
   
   /**
    * Handle Refresh check box selection.
    */
   protected void handleAllowRefresh()
   {
      rDenied.setEnabled(!rAllowed.isSelected());
   }
   
   /**
    * Handle Denied Path check box selection.
    */
   private void handleDeniedPath()
   {
      pUnlimited.setEnabled(!pDenied.isSelected());
   }
   
   /**
    * Handle Denied Refresh check box selection.
    */
   protected void handleDeniedRefresh()
   {
      rAllowed.setEnabled(!rDenied.isSelected());
   }
   
   /**
    * Handle Denied User check box selection.
    */
   private void handleDeniedUser()
   {
      boolean denied = uDenied.isSelected();
      uDenied.setSelected(denied);
      uEnumerate.setEnabled(!denied);
      uPassword.setEnabled(!denied);
      uGroup.setEnabled(!denied);
      uRead.setEnabled(!denied);
      uWrite.setEnabled(!denied);
      uCreate.setEnabled(!denied);
      uDelete.setEnabled(!denied);
      uDenied.setEnabled(true);
   }
   
   /**
    * Handle Save button press.
    */
   private void handleSave()
   {
      saving = true;      
      dialog.setVisible(false);
   }
   
   /**
    * Handle Cancel button press.
    */
   private void handleCancel()
   {
      saving = false;
      dialog.setVisible(false);
   }
}   