/*
** Module   : CommonListWidget.java
** Abstract : Define all list widgets-related APIs available via a widget handle instance
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 IAS 20150209        Created initial version.
** 022 IAS 20150225        Added support for the DELIMITER attribute,
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

public interface CommonListWidget<T extends ControlSetConfig> extends CommonWidget
{

   /**
    * Implements the ENTRY() widget method.  This method returns items from
    * the set by 1-based index.
    *
    * @param    index
    *           The index of an item.
    *
    * @return   The list item indexed or <code>unknown value</code>.
    */
   @LegacyMethod(name = "ENTRY")
   public character entry(int64 index);

   /**
    * Implements the ENTRY() widget method.  This method returns items from
    * the set by 1-based index.
    *
    * @param    index
    *           The index of an item.
    *
    * @return   The list item indexed or <code>unknown value</code>.
    */
   @LegacyMethod(name = "ENTRY")
   public character entry(long index);

   /**
    * Implements the LOOKUP() widget method.  This method returns the 1-based
    * item's index in the set.
    *
    * @param    text
    *           The text of an item.
    *
    * @return   The item's index or <code>unknown value</code>.
    */
   public integer lookup(character text);

   /**
    * Implements the LOOKUP() widget method.  This method returns the 1-based
    * item's index in the set.
    *
    * @param    text
    *           The text of an item.
    *
    * @return   The item's index or <code>unknown value</code>.
    */
   public integer lookup(String text);

   /**
    * Implements the ADD-FIRST() widget method, which adds a list of labels
    * to the beginning of the list.
    *
    * @param    itemList
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(character itemList);

   /**
    * Implements the ADD-FIRST() widget method, which adds a list of labels
    * to the beginning of the list.
    *
    * @param    itemList
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(String itemList);

   /**
    * Implements the ADD-FIRST() widget method, which adds a single label
    * and value pair to the beginning of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(character label, BaseDataType value);

   /**
    * Implements the ADD-FIRST() widget method, which adds a single label
    * and value pair to the beginning of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(String label, BaseDataType value);

   /**
    * Implements the ADD-FIRST() widget method, which adds a single label
    * and value pair to the beginning of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(character label, String value);

   /**
    * Implements the ADD-FIRST() widget method, which adds a single label
    * and value pair to the beginning of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-FIRST")
   public logical addFirst(String label, String value);

   /**
    * Implements the ADD-LAST() widget method, which adds a list of labels
    * to the end of the list.
    *
    * @param    itemList
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(character itemList);

   /**
    * Implements the ADD-LAST() widget method, which adds a list of labels
    * to the end of the list.
    *
    * @param    itemList
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(String itemList);

   /**
    * Implements the ADD-LAST() widget method, which adds a single label
    * and value pair to the end of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(character label, BaseDataType value);

   /**
    * Implements the ADD-LAST() widget method, which adds a single label
    * and value pair to the end of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(String label, BaseDataType value);

   /**
    * Implements the ADD-LAST() widget method, which adds a single label
    * and value pair to the end of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(character label, String value);

   /**
    * Implements the ADD-LAST() widget method, which adds a single label
    * and value pair to the end of the list.
    *
    * @param    label
    *           The label of the item to add.
    * @param    value
    *           The value of the item to add.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "ADD-LAST")
   public logical addLast(String label, String value);

   /**
    * Implements the DELETE() widget method, which removes the list item or items specified.
    *
    * @param    item
    *           The item or items to delete.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "DELETE")
   public logical delete(character item);
   
   /**
    * Implements the DELETE() widget method, which removes the list item or items specified.
    *
    * @param    item
    *           The item or items to delete.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "DELETE")
   public logical delete(String item);
   
   /**
    * Implements the DELETE() widget method, which removes the list item at the given index.
    *
    * @param    idx
    *           The index of the item to delete.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "DELETE")
   public logical delete(int64 idx);
   
   /**
    * Implements the DELETE() widget method, which removes the list item at the given index.
    *
    * @param    idx
    *           The index of the item to delete.
    *
    * @return   <code>true</code> if the method succeeds.
    */
   @LegacyMethod(name = "DELETE")
   public logical delete(long idx);
   
   /**
    * Store array of items into config.
    * 
    * @param    items
    *           Array of new items.
    * @param    pairs
    *           true if LIST_ITEM_PAIRS, RADIO_BUTTONS is set
    *           false if LIST_ITEMs IS set
    */
   public void setItems(ControlSetItem[] items, boolean pairs);

   /**
    * Sets the LIST-ITEMS writeable attribute.
    * 
    * @param    list
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.  These
    *           items will be set as the full list of items in this widget.
    */
   @LegacyAttribute(name = "LIST-ITEMS", setter = true)
   public void setListItems(character list);

   /**
    * Setter for the LIST-ITEMS writeable attribute.
    *
    * @param    list
    *           The new value for the LIST-ITEMS.
    */
   @LegacyAttribute(name = "LIST-ITEMS", setter = true)
   public void setListItems(String list);

   /**
    * Common implementation list item pair attributes (LIST-ITEM-PAIRS and
    * RADIO-BUTTONS) attribute. The parameter contains string representation
    * of all labels and values separated by a delimiter (comma is the default
    * delimiter).
    * <p>
    * Example: <code>Label1,value1,Label2,value2,...,LabelN,valueN</code>
    * 
    * @param    list
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.  These
    *           items will be set as the full list of items in this widget.
    */
   @LegacyAttribute(name = "LIST-ITEM-PAIRS", setter = true)
   public void setListItemPairs(character list);

   /**
    * Common implementation list item pair attributes (LIST-ITEM-PAIRS and
    * RADIO-BUTTONS) attribute. The parameter contains string representation
    * of all labels and values separated by a delimiter (comma is the default
    * delimiter).
    * <p>
    * Example: <code>Label1,value1,Label2,value2,...,LabelN,valueN</code>
    * 
    * @param    list
    *           A list of delimited items using the configured delimiter for
    *           this widget (a comma by default) to split the items.  These
    *           items will be set as the full list of items in this widget.
    */
   @LegacyAttribute(name = "LIST-ITEM-PAIRS", setter = true)
   public void setListItemPairs(String list);

   /**
    * Gets the LIST-ITEMS writeable attribute.
    * 
    * @return   The current value of LIST-ITEMS attribute.
    */
   @LegacyAttribute(name = "LIST-ITEMS")
   public character getListItems();

   /**
    * Gets the LIST-ITEM-PAIRS writeable attribute.
    *
    * @return   The current value of LIST-ITEMS attribute.
    */
   @LegacyAttribute(name = "LIST-ITEM-PAIRS")
   public character getListItemPairs();
   
   /**
    * Gets the DELIMITER writeable attribute.
    *
    * @return   The current value of DELIMITER attribute.
    */
   @LegacyAttribute(name = "DELIMITER")
   public character getDelimiter();

   /**
    * Sets the DELIMITER writeable attribute.
    *
    * @param value
    *       The new value of DELIMITER attribute.
    */
   @LegacyAttribute(name = "DELIMITER", setter = true )
   public void setDelimiter(character value);
   
   /**
    * Sets the DELIMITER writeable attribute.
    *
    * @param value
    *       The new value of DELIMITER attribute.
    */
   @LegacyAttribute(name = "DELIMITER", setter = true )
   public void setDelimiter(String value);

   /**
    * Tell widget that data format should be honored.
    * 
    * @param    b
    *           If <code>true</code>, the format string will be honored.
    */
   public void setHonorFormat(boolean b);
}