/* 
** Module   : ControlTextConfig.java
** Abstract : stores the common text widget type variables
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 EVL 20050725   @21810 Creation based on general concept.
** 002 SIY 20050831   @22497 Refactoring (see @@22394-22409) 
** 003 SIY 20050907   @22629 Fixed use of @inheritDoc tag in javadoc
**                           Fixed NullPointerException
** 004 SIY 20050909   @22663 Fixed sideLabel handling.
** 005 SIY 20050913   @22833 Renamed some boolean methods from getXXX to 
**                           isXXX, added support for remaining attributes.
** 006 SIY 20060203   @24238 Added NO-LABELS flag.
** 007 SIY 20060209   @24479 Major refactoring.
** 008 SIY 20060316   @25127 Added support for header flag.
** 009 SIY 20060401   @25311 Moved header flag into ComponentConfig.
** 010 SIY 20060428   @26079 Moved some common members from FillInConfig 
**                           and EditorConfig.
** 011 SIY 20060710   @27982 Added static widget content flag.
** 012 SIY 20060717   @28092 Added @ widget flag.
** 013 SIY 20060824   @28149 Removed getIndex() method.
** 014 GES 20060803   @28403 Made externalizable for better performance.
**                           Many code standard cleanups.
** 015 CA  20081215   @40902 If F4 is pressed during a pause generated by
**                           setFormat, the exception must not be hidden
**                           into a RuntimeException - it must be thrown.
** 016 SIY 20090127   @41232 Do not change format if it is the same as 
**                           before.
** 017 SIY 20100619          Switching to own implementation of character UI.
** 018 SIY 20100804          Changes in imports, some refactorings.
** 019 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 020 SIY 20111202          Updated imports.
** 021 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 022 CA  20150105          Changes for SIDE-LABEL-HANDLE runtime support.
** 023 CA  20150323          The widget's @ format length must be stored separately.
** 024 HC  20150323          Removed business logic from config classes.
** 025 CA  20150722          In LABEL case, the server and client-side configs are different 
**                           types (TextConfig vs LabelConfig) - so, applyConfig is protected to
**                           ignore instances which are not of this type.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * The config class which describes the common attribute set for text related
 * CHUI widget classes.
 */
public abstract class ControlTextConfig
extends ControlConfig
{
   /** The Progress BLANK attribute. */
   public boolean blank = false;
   
   /** The Progress FORMAT attribute. */
   public String format = null;

   /** The Progress READ-ONLY attribute. */
   public boolean readOnly = false;

   /** Static widget content flag. */
   public boolean flagStatic = false;

   /** AT (@) field flag. */
   public boolean flagAt = false;

   /** Flag tracking the maximum format from widget's @ phrases. */
   public int atFormatLength = 0;

   /**
    * Default constructor (only used in de-serialization).
    */
   public ControlTextConfig()
   {
      possibleSideLabel = true;
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected ControlTextConfig(int id)
   {
      super(id);

      possibleSideLabel = true;
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected ControlTextConfig(WidgetId id)
   {
      super(id);

      possibleSideLabel = true;
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      if (!(config instanceof ControlTextConfig))
      {
         return;
      }
      
      ControlTextConfig cfg = (ControlTextConfig) config;
      
      blank = cfg.blank;
      format = cfg.format;
      readOnly = cfg.readOnly;
      flagStatic = cfg.flagStatic;
      flagAt = cfg.flagAt;
      atFormatLength = cfg.atFormatLength;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      blank          = in.readBoolean();
      readOnly       = in.readBoolean();
      flagStatic     = in.readBoolean();
      flagAt         = in.readBoolean();
      atFormatLength = in.readInt();
      format         = (String) in.readObject();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeBoolean(blank);
      out.writeBoolean(readOnly);
      out.writeBoolean(flagStatic);
      out.writeBoolean(flagAt);
      out.writeInt(atFormatLength);
      out.writeObject(format);
  }
}