/*
** Module   : EditorHelper.java
** Abstract : common Editor implementation code used by both server and client
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description--------------------------------
** 001 GES 20150103 First version, with common cursor positioning code. 
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import java.util.*;

/**
 * Common Editor implementation code used by both server and client.
 */
public class EditorHelper
{
   /**
    * Don't allow instances of this class to be constructed.
    */
   private EditorHelper()
   {
   }
   
   /**
    * Calculate and set the cursor X position into the given configuration instance. If the
    * position is not near the edge, then the cursor will be positioned about at the middle of
    * the screen.
    *
    * @param    config
    *           The editor's config instance.
    * @param    pos
    *           New cursor position (0-based).
    */
   public static void setCursorX(EditorConfig config, int pos)
   {
      if (pos < config.scrollX || pos >= config.scrollX + config.innerChars)
      {
         // scrolling is necessary
         config.cursorX = (config.innerChars - 1)/2 - 1;
         config.scrollX = pos - config.cursorX;
         
         if (config.scrollX < 0 || (config.scrollX + config.innerChars) >= config.bufferChars)
         {
            if (config.scrollX < 0)
            {
               config.cursorX -= config.scrollX;
               config.scrollX = 0;
            }
            else
            {
               config.cursorX = config.innerChars - config.bufferChars + pos - 1;
               config.scrollX = pos - config.cursorX;
            }
         }
      }
      else
      {
         config.cursorX = pos - config.scrollX;
      }
   }
}