/*
** Module   : MenuConfig.java
** Abstract : Menu widget attribute container.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------------------Description-----------------------------
** 001 VIG 20141029 Created initial version. 
** 002 VIG 20150119 Added externalization methods implementation.
** 003 VIG 20150809 Added property "displayed".
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.ui;

import java.io.*;

/**
 * MENU widget configuration container.
 */
public class MenuConfig
extends PaneConfig
{
   /**  
    * Progress POPUP-ONLY attribute (Graphical interfaces only). Specifies whether this 
    * menu owned by a widget (<code>true</code>) or by window (<code>false</code>).
    * Set by MENUBAR option in menu definition. 
    */
   public boolean popupOnly = true;
   
   /**
    * OWNER property: window for menubar or other widget for pop-up menu, 
    * or 0 if not assigned. 
    */
   public int ownerId = 0;
   
   /**
    * This flag is used to indicate the client state of widget.
    * <code>true</code> means the menu is dropped down (become visible).
    * Make sense only for pop-up menus.
    */
   public boolean displayed = false;
   
   /**
    * Default constructor (only used in deserialization).
    */
   public MenuConfig()
   {
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   public MenuConfig(WidgetId id)
   {
      super(id);
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      MenuConfig cfg = (MenuConfig) config;
      
      popupOnly = cfg.popupOnly;
      displayed = cfg.displayed;
      ownerId   = cfg.ownerId;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      
      popupOnly = in.readBoolean();
      displayed = in.readBoolean();
      ownerId   = in.readInt(); 
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      
      out.writeBoolean(popupOnly);
      out.writeBoolean(displayed);
      out.writeInt(ownerId);
   }
}