/*
** Module   : RadioSetInterface.java
** Abstract : Define all RADIO-SET widget-related APIs available via a widget handle instance
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description--------------------------------
** 001 IAS 20150212 Created initial version.
** 002 IAS 20150312 Implemented ENABLE/DISABLE method and NUM-BUTTONS attribute.
** 003 GES 20150901 Fixed javadoc and comment formatting.
** 004 EVL 20150911 Moving orientation related API to separate interface.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Defines all attributes and methods that are specific to RADIO-SET.  Attributes and methods
 * shared by multiple widget types will be defined elsewhere.
 */
public interface RadioSetInterface
extends OrientationInterface
{
   /**
    * Resets the value of the EXPAND attribute for radio-set.
    * 
    * @param   value
    *          The new value of the HORIZONTAL attribute.
    */
   @LegacyAttribute(name = "EXPAND", setter = true)
   public void setExpand(boolean value);

   /**
    * Resets the value of the EXPAND attribute for radio-set.
    * 
    * @param   value
    *          The new value of the HORIZONTAL attribute.
    */
   @LegacyAttribute(name = "EXPAND", setter = true)
   public void setExpand(logical value);
   
   /**
    * Get the value of the EXPAND attribute for radio-set.
    * 
    * @return
    *          The value of the EXPAND attribute.
    */
   @LegacyAttribute(name = "EXPAND")
   public logical isExpand();

   /**
    * Get RADIO-BUTTONS attribute.
    * 
    * @return  Current value of the attribute.
    */
   @LegacyAttribute(name = "RADIO-BUTTONS")
   public character getRadioButtons();
   
   /**
    * Set RADIO-BUTTONS attribute. The parameter contains string
    * representation of all radio button labels and values separated by
    * delimiter (comma is the default value).
    * <p>
    * Example: <code>Label1,value1,Label2,value2,..., LabelN,valueN</code>
    * 
    * @param   c
    *          New value of the attribute.
    */
   @LegacyAttribute(name = "RADIO-BUTTONS", setter = true)
   public void setRadioButtons(character c);

   /**
    * Set RADIO-BUTTONS attribute. The parameter contains string
    * representation of all radio button labels and values separated by
    * delimiter (comma is the default value).
    * <p>
    * Example: <code>Label1,value1,Label2,value2,..., LabelN,valueN</code>
    * 
    * @param   c
    *          New value of the attribute.
    */
   @LegacyAttribute(name = "RADIO-BUTTONS", setter = true)
   public void setRadioButtons(String c);

   /**
    * Enable RADIO-BUTTON in a RADIO-SET widget. 
    * 
    * @param   label
    *          New label of the button to be enabled.
    *  
    * @return
    *          <code>true</code> if successful
    */
   @LegacyMethod(name = "ENABLE")
   public logical enable(String label);

   /**
    * Enable RADIO-BUTTON in a RADIO-SET widget. 
    * 
    * @param   label
    *          New label of the button to be enabled.
    *  
    * @return
    *          <code>true</code> if successful
    */
   @LegacyMethod(name = "ENABLE")
   public logical enable(character label);

   /**
    * Disable RADIO-BUTTON in a RADIO-SET widget. 
    * 
    * @param   label
    *          New label of the button to be disabled.
    *  
    * @return
    *          <code>true</code> if successful
    */
   @LegacyMethod(name = "DISABLE")
   public logical disable(String label);

   /**
    * Disable RADIO-BUTTON in a RADIO-SET widget. 
    * 
    * @param   label
    *          New label of the button to be disabled.
    *  
    * @return
    *          <code>true</code> if successful
    */
   @LegacyMethod(name = "DISABLE")
   public logical disable(character label);

   /**
    * Get the value of the NUM-BUTTONS attribute which is the number of buttions in the
    * RADIO-SET.
    *
    * @return   The number of buttons in the widget.
    */
   @LegacyAttribute(name = "NUM-BUTTONS")
   public integer getNumButtons();
}