/*
** Module   : ReplaceInterface.java
** Abstract : Define REPLACE methods available via a handle instance 
**            for the EDITOR/COMBO-BOX/SELECTION-LIST/RADIO-SET widgets
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 IAS 20150604 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

public interface ReplaceInterface
{
   
   /**
.    * REPLACE (old-string, new-string, flag ) method for EDITOR
    * 
    * @param    from
    *           The source text.
    * @param    to
    *           The replacement text.
    * @param    flag
    *          the search mode
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character from, character to, integer flag);

   /**
.    * REPLACE (old-string, new-string, flag ) method for EDITOR
    * 
    * @param    from
    *           The source text.
    * @param    to
    *           The replacement text.
    * @param    flag
    *          the search mode
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String from, character to, integer flag);

//   /**
//.    * REPLACE (old-string, new-string, flag ) method for EDITOR
//    * 
//    * @param    from
//    *           The source text.
//    * @param    to
//    *           The replacement text.
//    * @param    flag
//    *          the search mode
//    * @return   <code>true</code> if the operation was successful.
//    */
//   @LegacyMethod(name = "REPLACE")
//   public logical replace(character from, String to, integer flag);

   /**
.    * REPLACE (old-string, new-string, flag ) method for EDITOR
    * 
    * @param    from
    *           The source text.
    * @param    to
    *           The replacement text.
    * @param    flag
    *          the search mode
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character from, character to, int flag);

   /**
.    * REPLACE (old-string, new-string, flag ) method for EDITOR
    * 
    * @param    from
    *           The source text.
    * @param    to
    *           The replacement text.
    * @param    flag
    *          the search mode
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String from, character to, int flag);

   /**
.    * REPLACE (old-string, new-string, flag ) method for EDITOR
    * 
    * @param    from
    *           The source text.
    * @param    to
    *           The replacement text.
    * @param    flag
    *          the search mode
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character from, String to, int flag);


   /**
    * REPLACE (new-label, new-value, list-index) method for COMBO-BOX/SELECTION-LIST
    * REPLACE (old-string, new-string, flag ) method for EDITOR
    * 
    * @param arg0
    *          new item's label for COMBO-BOX/SELECTION-LIST,
    *          the source text for EDITOR
    * @param arg1         
    *          new item's value for COMBO-BOX/SELECTION-LIST,
    *          replacement test for editor
    * @param arg2
    *          index of the item to be replaced for COMBO-BOX/SELECTION-LIST,
    *          the search mode for EDITOR
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String arg0, String arg1, integer arg2);
   
   /**
    * REPLACE (new-label, new-value, list-index) method for COMBO-BOX/SELECTION-LIST
    * REPLACE (old-string, new-string, flag ) method for EDITOR
    * 
    * @param arg0
    *          new item's label for COMBO-BOX/SELECTION-LIST,
    *          the source text for EDITOR
    * @param arg1         
    *          new item's value for COMBO-BOX/SELECTION-LIST,
    *          replacement test for editor
    * @param arg2
    *          index of the item to be replaced for COMBO-BOX/SELECTION-LIST,
    *          the search mode for EDITOR
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String arg0, String arg1, int arg2);

   /**
    * REPLACE(REPLACE (new-item-list, list-item )) method
    * (for COMBO-BOX and SELECTION-LIST)
    * 
    * @param newItemList
    *          new item list
    * @param listItem
    *          old item's value
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newItemList, character listItem);

   /**
    * REPLACE(REPLACE (new-item-list, list-item )) method
    * (for COMBO-BOX and SELECTION-LIST)
    * 
    * @param newItemList
    *          new item list
    * @param listItem
    *          old item's value
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newItemList, String listItem);

   /**
    * REPLACE(REPLACE (new-item-list, list-item )) method
    * (for COMBO-BOX and SELECTION-LIST)
    * 
    * @param newItemList
    *          new item list
    * @param listItem
    *          old item's value
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newItemList, character listItem);

   /**
    * REPLACE(REPLACE (new-item-list, list-item )) method
    * (for COMBO-BOX and SELECTION-LIST)
    * 
    * @param newItemList
    *          new item list
    * @param listItem
    *          old item's value
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newItemList, String listItem);

   /**
    * REPLACE (new-item-list, list-index ) method
    * (for COMBO-BOX and SELECTION-LIST)
    * 
    * @param newItemList
    *          new item list
    * @param listIndex
    *          old item's index
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newItemList, integer listIndex);

   /**
    * REPLACE (new-item-list, list-index ) method
    * (for COMBO-BOX and SELECTION-LIST)
    * 
    * @param newItemList
    *          new item list
    * @param listIndex
    *          old item's index
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newItemList, int listIndex);

   /**
    * REPLACE (new-item-list, list-index ) method
    * (for COMBO-BOX and SELECTION-LIST)
    * 
    * @param newItemList
    *          new item list
    * @param listIndex
    *          old item's index
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newItemList, integer listIndex);

   /**
    * REPLACE (new-item-list, list-index ) method
    * (for COMBO-BOX and SELECTION-LIST)
    * 
    * @param newItemList
    *          new item list
    * @param listIndex
    *          old item's index
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newItemList, int listIndex);

   /**
    * REPLACE (new-label, new-value, list-item) method for COMBO-BOX/SELECTION-LIST,
    * REPLACE (new-label, new-value, old-label) method for RADIO-SET
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param oldItem
    *          oldItem (old-label for the READIO-SET, and list-item for the COMBO-BOX/SELECTION-LIST)
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newLabel, BaseDataType  newValue, character oldItem);

   /**
    * REPLACE (new-label, new-value, list-item) method for COMBO-BOX/SELECTION-LIST,
    * REPLACE (new-label, new-value, old-label) method for RADIO-SET
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param oldItem
    *          oldItem (old-label for the READIO-SET, and list-item for the COMBO-BOX/SELECTION-LIST)
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newLabel, BaseDataType  newValue, character oldItem);

   /**
    * REPLACE (new-label, new-value, list-item) method for COMBO-BOX/SELECTION-LIST,
    * REPLACE (new-label, new-value, old-label) method for RADIO-SET
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param oldItem
    *          oldItem (old-label for the READIO-SET, and list-item for the COMBO-BOX/SELECTION-LIST)
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newLabel, String  newValue, character oldItem);

   /**
    * REPLACE (new-label, new-value, list-item) method for COMBO-BOX/SELECTION-LIST,
    * REPLACE (new-label, new-value, old-label) method for RADIO-SET
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param oldItem
    *          oldItem (old-label for the READIO-SET, and list-item for the COMBO-BOX/SELECTION-LIST)
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newLabel, String  newValue, character oldItem);

   /**
    * REPLACE (new-label, new-value, list-item) method for COMBO-BOX/SELECTION-LIST,
    * REPLACE (new-label, new-value, old-label) method for RADIO-SET
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param oldItem
    *          oldItem (old-label for the READIO-SET, and list-item for the COMBO-BOX/SELECTION-LIST)
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newLabel, BaseDataType  newValue, String oldItem);

   /**
    * REPLACE (new-label, new-value, list-item) method for COMBO-BOX/SELECTION-LIST,
    * REPLACE (new-label, new-value, old-label) method for RADIO-SET
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param oldItem
    *          oldItem (old-label for the READIO-SET, and list-item for the COMBO-BOX/SELECTION-LIST)
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newLabel, BaseDataType  newValue, String oldItem);

   /**
    * REPLACE (new-label, new-value, list-item) method for COMBO-BOX/SELECTION-LIST,
    * REPLACE (new-label, new-value, old-label) method for RADIO-SET
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param oldItem
    *          oldItem (old-label for the READIO-SET, and list-item for the COMBO-BOX/SELECTION-LIST)
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newLabel, String  newValue, String oldItem);

   /**
    * REPLACE (new-label, new-value, list-item) method for COMBO-BOX/SELECTION-LIST,
    * REPLACE (new-label, new-value, old-label) method for RADIO-SET
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param oldItem
    *          oldItem (old-label for the READIO-SET, and list-item for the COMBO-BOX/SELECTION-LIST)
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newLabel, String  newValue, String oldItem);

   /**
    * REPLACE (new-label, new-value, list-index) method for COMBO-BOX/SELECTION-LIST
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param listIndex
    *          index of the item to be replaced
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newLabel, BaseDataType  newValue, integer listIndex);

   /**
    * REPLACE (new-label, new-value, list-index) method for COMBO-BOX/SELECTION-LIST
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param listIndex
    *          index of the item to be replaced
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newLabel, BaseDataType newValue, integer listIndex);

   /**
    * REPLACE (new-label, new-value, list-index) method for COMBO-BOX/SELECTION-LIST
    * REPLACE (old-string, new-string, flag ) method for EDITOR
    * 
    * @param arg0
    *          new item's label for COMBO-BOX/SELECTION-LIST,
    *          the source text for EDITOR
    * @param arg1         
    *          new item's value for COMBO-BOX/SELECTION-LIST,
    *          replacement test for editor
    * @param arg2
    *          index of the item to be replaced for COMBO-BOX/SELECTION-LIST,
    *          the search mode for EDITOR
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character arg0, String arg1, integer arg2);

   /**
    * REPLACE (new-label, new-value, list-index) method for COMBO-BOX/SELECTION-LIST
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param listIndex
    *          index of the item to be replaced
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(character newLabel, BaseDataType newValue, int listIndex);

   /**
    * REPLACE (new-label, new-value, list-index) method for COMBO-BOX/SELECTION-LIST
    * 
    * @param newLabel
    *          new item's label
    * @param newValue         
    *          new item's value
    * @param listIndex
    *          index of the item to be replaced
    * @return   <code>true</code> if the operation was successful.
    */
   @LegacyMethod(name = "REPLACE")
   public logical replace(String newLabel, BaseDataType  newValue, int listIndex);
}