/*
** Module   : SliderInterface.java
** Abstract : Define slider specific widget-related APIs available via a widget handle instance
**            and used by SLIDER widget.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description--------------------------------
** 001 EVL 20150911 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Defines all slider specific attributes and methods that are used by SLIDER.
 */
public interface SliderInterface
extends OrientationInterface
{
   /**
    * Gets the MIN-VALUE writable attribute.
    *
    * @return   The current value of the MIN-VALUE attribute.
    */
   @LegacyAttribute(name = "MIN-VALUE")
   public integer getMinValue();

   /**
    * Sets the MIN-VALUE writable attribute.
    *
    * @param    minValue
    *           The new value for the MIN-VALUE attribute.
    */
   @LegacyAttribute(name = "MIN-VALUE", setter = true)
   public void setMinValue(NumberType minValue);
   
   /**
    * Sets the MIN-VALUE writable attribute.
    *
    * @param    minValue
    *           The new value for the MIN-VALUE attribute.
    */
   @LegacyAttribute(name = "MIN-VALUE", setter = true)
   public void setMinValue(int minValue);

   /**
    * Gets the MAX-VALUE writable attribute.
    *
    * @return   The current value of the MAX-VALUE attribute.
    */
   @LegacyAttribute(name = "MAX-VALUE")
   public integer getMaxValue();

   /**
    * Sets the MAX-VALUE writable attribute.
    *
    * @param    maxValue
    *           The new value for the MAX-VALUE attribute.
    */
   @LegacyAttribute(name = "MAX-VALUE", setter = true)
   public void setMaxValue(NumberType maxValue);
   
   /**
    * Sets the MAX-VALUE writable attribute.
    *
    * @param    maxValue
    *           The new value for the MAX-VALUE attribute.
    */
   @LegacyAttribute(name = "MAX-VALUE", setter = true)
   public void setMaxValue(int maxValue);

   /**
    * Get the value of the LARGE-TO-SMALL attribute.
    *
    * @return   the value of the attribute.
    */
   @LegacyAttribute(name = "LARGE-TO-SMALL")
   public logical isLargeToSmall();
   
   /**
    * Set the value of the LARGE-TO-SMALL attribute.
    *
    * @param   value
    *          the new value of the attribute.
    */
   @LegacyAttribute(name = "LARGE-TO-SMALL", setter = true)
   public void setLargeToSmall(logical value);

   /**
    * Set the value of the LARGE-TO-SMALL attribute.
    *
    * @param   value
    *          the new value of the attribute.
    */
   @LegacyAttribute(name = "LARGE-TO-SMALL", setter = true)
   public void setLargeToSmall(boolean value);

   /**
    * Get the value of the NO-CURRENT-VALUE attribute.
    *
    * @return   the value of the attribute.
    */
   @LegacyAttribute(name = "NO-CURRENT-VALUE")
   public logical isNoCurrentValue();
   
   /**
    * Set the value of the NO-CURRENT-VALUE attribute.
    *
    * @param   value
    *          the new value of the attribute.
    */
   @LegacyAttribute(name = "NO-CURRENT-VALUE", setter = true)
   public void setNoCurrentValue(logical value);

   /**
    * Set the value of the NO-CURRENT-VALUE attribute.
    *
    * @param   value
    *          the new value of the attribute.
    */
   @LegacyAttribute(name = "NO-CURRENT-VALUE", setter = true)
   public void setNoCurrentValue(boolean value);

   /**
    * Get the value of the TIC-MARKS attribute.
    *
    * @return   the value of the attribute.
    */
   @LegacyAttribute(name = "TIC-MARKS")
   public character getTicMarks();
   
   /**
    * Set the value of the TIC-MARKS attribute.
    *
    * @param   tMarks
    *          the new value of the attribute.
    */
   @LegacyAttribute(name = "TIC-MARKS", setter = true)
   public void setTicMarks(String tMarks);
   
   /**
    * Set the value of the TIC-MARKS attribute.
    *
    * @param   tMarks
    *          the new value of the attribute.
    */
   @LegacyAttribute(name = "TIC-MARKS", setter = true)
   public void setTicMarks(character tMarks);
   
   /**
    * Get the value of the FREQUENCY attribute.
    *
    * @return   the value of the attribute.
    */
   @LegacyAttribute(name = "FREQUENCY")
   public integer getFrequency();
   
   /**
    * Set the value of the FREQUENCY attribute.
    *
    * @param   freq
    *          the new value of the attribute.
    */
   @LegacyAttribute(name = "FREQUENCY", setter = true)
   public void setFrequency(int freq);
   
   /**
    * Set the value of the FREQUENCY attribute.
    *
    * @param   freq
    *          the new value of the attribute.
    */
   @LegacyAttribute(name = "FREQUENCY", setter = true)
   public void setFrequency(NumberType freq);
}