/*
** Module   : SystemHelpAction.java
** Abstract : Help management actions.
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description--------------------------------
** 001 SVL 20101006 Created initial version.
** 002 GES 20170313 Added constants to finish the set.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

/**
 * Help management actions.
 */
public enum SystemHelpAction
{
   /** Displays the named topic in the alternate keyword index. Only works for .chm files. */
   ALTERNATE_KEY,
   
   /** Execute a help macro.  Only works for .hlp files. */
   COMMAND,
   
   /** Display the default topic in the contents pane. Only works for .hlp files. */
   CONTENTS,
   
   /** Display the topic associated with the given numeric id. */
   CONTEXT,
   
   /** Display the Windows Help Topics dialog box. Only works for .hlp files. */
   FINDER,
   
   /** Forces the help file to be open and visible. Only works for .hlp files. */
   FORCE_FILE,

   /** Displays the contents of the ABL help file. Only works for .hlp files.  */
   HELP,
   
   /** Displays the topic matching the given string in the keyword index. */
   KEY,
   
   /** Force an already existing help window to maximize. */
   MAXIMIZE,
   
   /** Displays the named topic in the alternate keyword index. Only works for .hlp files. */
   MULTIPLE_KEY,
   
   /** Matches a topic by partial string in the keyword index. Only works for .hlp files. */
   PARTIAL_KEY,
   
   /** Displays a popup window with the topic from a context id. Only works for .hlp files. */
   POPUP,
   
   /** Positions and sizes an existing help window. */
   POSITION,
   
   /** Close down the help system if it is running. */
   QUIT,
   
   /** Remaps contents to the given context id. Only works for .hlp files. */
   SET_CONTENTS,
   
   /** Displays the named topic in the content pane. Only works for .chm files. */
   TOPIC
}