/*
** Module   : MessageLine.java
** Abstract : Message line widget.
**
** Copyright (c) 2006-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20060131   @24196 Created initial version
** 002 SIY 20060704   @27839 Label can be set to null.
** 003 NVS 20060731   @28259 Labels draw directly and do not produce paint
**                           events. repaint() override is an empty method
** 004 NVS 20060801   @28284 Labels draw padding blanks now.
** 005 SVL 20071127   @36147 Widget is not painted then color = -1 
**                           specified. Ref: #36030
** 006 SIY 20071127   @36148 Rolled back #005. 
** 007 SIY 20080507   @38229 Do not wipe out message area when some text
**                           is present.
** 008 NVS 20080828   @39618 Added repaintBackDoor() method which is a way
**                           to call the overridden repaint(). 
** 009 SIY 20090225   @41377 Don't clear entire area if text is not null.
** 010 GES 20090424   @41966 Import change.
** 011 SIY 20100619          Switching to own implementation of character UI.
** 012 SIY 20100804          Changes in imports, some refactorings.
** 013 SIY 20100822          Refactored AttributeContainer support.
** 014 SIY 20100912          Changed hierarchy in order to avoid need for 
**                           repaintBackdoor(). Renamed from P2JLabel.
** 015 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 016 SIY 20111202          Changed base class.
** 017 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.*;

/**
 * Message line implementation. Its purpose is to adjust color/size
 * handling to make it consistent with the other widgets and message/status
 * lines requirements.
 * 
 * @author  SIY
 * @version 1.0.0
 */
public class MessageLine
extends StatusLineImpl
{
   /**
    * Create a new message line.
    * 
    * @param   id
    *          The widget's ID.
    */
   public MessageLine(WidgetId id)
   {
      super(id);
   }
   
   /**
    * Blocks repaint requests.
    */
   @Override
   public void repaint() 
   {
   }
}