/*
** Module   : StatusLineImpl.java
** Abstract :
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SIY 20111124 Extracted from StatusLine
** 002 SIY 20111221 Fixed constructor visibility.
** 003 CA  20140711 Some refactoring for GUI support.
** 004 CA  20140805 Do not allow ChUI impl classes access to APIs outside of ChuiOutputManager.
** 005 HC  20140911 GUI coordinate support - data types refactoring.
** 006 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 007 CA  20141119 Refactored drawing code to enable multiple OS windows, for GUI support.  Misc
**                  changes/fixes related to drawing on multiple windows.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.*;
import com.goldencode.util.*;

/**
 * CHUI-specific version of StatusLine
 */
@SuppressWarnings("unchecked")
public class StatusLineImpl
extends StatusLine<ChuiOutputManager>
{
   /**
    * Hide base class public constructor.
    * 
    * @param   id
    *          The widget's ID.
    */
   StatusLineImpl(WidgetId id)
   {
      super(id);
   }
   
   /**
    * Widget drawing method.
    */
   @Override
   public void draw()
   {
      // get the screen location
      Point origin = screenLocation();
   
      screen().at(origin);
   
      // first,output the available text using this instance color
      StringBuffer buf = new StringBuffer(getLabelText());
      int textlength = getLabelText().length();
      if (textlength > 0)
      {
         if (maxLen > 0 && textlength > maxLen)
            buf.setLength(maxLen); // truncate
   
         screen().append(buf.toString(), color);
      }
   
      if (textlength == 0 && isNull)
      {
         String padding = StringHelper.repeatChar(' ', (int) width());
         screen().append(padding, Color.DEFAULT);
      }
   }

   /**
    * Attach this widget to the specified window.
    * 
    * @param    window
    *           The parent window.
    */
   @Override
   public void attach(Window<ChuiOutputManager> window)
   {
      window.getContentPane().add(this);
      
      Dimension screenSize = window.screenDimension();
      
      this.setLocation(0, screenSize.height - 1);
      
      this.setLength((int) screenSize.width - Window.INSERT_TEXT.length());
      this.setColor(Color.NORMAL);
   }
}