/*
** Module   : Dimension.java
** Abstract : Area dimension descriptor.
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100629          Created initial version
** 002 SIY 20100804          Changes in imports, some refactorings.
** 003 HC  20140911          GUI coordinate support - data types refactoring.
** 004 HC  20141021          Implemented window move/position support.
** 005 HC  20150913          Added human-friendly toString implementation.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.Coordinate;

/**
 * Area dimension descriptor.
 * The coordinate values are expressed in character units.
 */
public class Dimension
{
   /** Area height. */
   public double height;
   /** Area width.*/
   public double width;

   /**
    * Constructor.
    * 
    * @param   width
    *          Area width.
    * @param   height
    *          Area height.
    */
   public Dimension(double width, double height)
   {
      this.width = Coordinate.scale(width);
      this.height = Coordinate.scale(height);
   }

   /**
    * Create a copy of this dimension object.
    * 
    * @return  dimension object with the same with and height.
    */
   public Dimension copy()
   {
      return new Dimension(width, height);
   }

   /**
    * Increase dimension by width and height deltas from provided 
    * {@link Insets} instance.
    *  
    * @param   insets
    *          Source insets.
    *          
    * @return  new dimension object enlarged by width and height deltas.
    */
   public Dimension enlargedBy(Insets insets)
   {
      return new Dimension(width + insets.widthDelta(), 
                           height + insets.heightDelta());
   }

   /**
    * Reduce dimension by width and height deltas from provided 
    * {@link Insets} instance.
    *  
    * @param   insets
    *          Source insets.
    * 
    * @return  new dimension object reduced by width and height deltas.
    */
   public Dimension reducedBy(Insets insets)
   {
      return new Dimension(width - insets.widthDelta(), 
                           height - insets.heightDelta());
   }
   
   /**
    * Get dimension height.
    * 
    * @return  height.
    */
   public double height()
   {
      return height;
   }
   
   /**
    * Get dimension width.
    * 
    * @return  width.
    */
   public double width() 
   {
      return width;
   }
   
   /**
    * Returns a string representation of the object.
    * 
    * @return  See above.
    */
   @Override
   public String toString()
   {
      return "Dimension[" + width + "," + height + "]";  
   }
}