/*
** Module   : ScrollBarController.java
** Abstract : Scroll bar controller.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SVL 20151101 Initial version.
** 002 VIG 20160325 Added scroll step type to positionUpdated().
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.client.ScrollBar.*;

/**
 * Scroll bar controller. Allows to fully control a scroll bar.
 */
public interface ScrollBarController
{
   /**
    * Notify controller that thumb drag has started.
    */
   public void startThumbDrag();

   /**
    * Notify controller that thumb drag has ended.
    */
   public void stopThumbDrag();

   /**
    * Get maximum value for the scroll bar.
    *
    * @return maximum value for the scroll bar.
    */
   public int getMax();

   /**
    * Get step for the scroll bar.
    *
    * @return step for the scroll bar.
    */
   public int getStep();

   /**
    * Get enable state for the scroll bar.
    *
    * @return <code>true</code> if scroll bar is enabled.
    */
   public boolean isEnabled();

   /**
    * Get visibility state for the scroll bar.
    *
    * @return <code>true</code> if scroll bar is visible.
    */
   public boolean isVisible();

   /**
    * Get position for the scroll bar.
    *
    * @return position for the scroll bar.
    */
   public int getPosition();

   /**
    * Get thumb size for the scroll bar.
    *
    * @return thumb size for the scroll bar (in pixels).
    */
   public int getThumbSize();

   /**
    * Get thumb position for the scroll bar.
    *
    * @return thumb position for the scroll bar (distance between left/top button and thumb's
    *         left/top edge).
    */
   public int getThumbPosition();

   /**
    * Notify controller that position has been updated.
    *
    * @param   position
    *          New position of the scroll bar.
    * @param   scrollStep 
    *          Scroll step type.
    * @param   drag
    *          <code>true</code> if position was changed because of thumb dragging.
    *          <code>false</code> if position was changed because a scroll button was pressed.
    */
   public void positionUpdated(int position, ScrollStep scrollStep, boolean drag);

   /**
    * Notify controller that thumb has been dragged.
    *
    * @param thumbPos
    *        New thumb position for the scroll bar (distance between left/top button and thumb's
    *        left/top edge).
    */
   public void thumbMoved(int thumbPos);
}