/*
** Module   : SpareIteration.java
** Abstract : 
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SIY 20100915 Created initial version (extracted from nested class).
** 002 SIY 20111101 Moved from CHUI package. Minor cleanups/refactorings in preparation to GUI 
**                  client.
** 003 CA  20140805 Isolate ThinClient usage from other ChUI classes.  Added generics.
** 004 EVL 20160726 Changing usage screen buffer array in ThinClient view() method.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * Convenient container for 'spare' iteration data.
 * <p>
 * Since VIEW may display only part of the iteration data, in some cases it is
 * necessary to save iteration data in temporary container before processing
 * conditional UP/DOWN and then display them at correct position. This class
 * is used as a helper container for iteration data between VIEW and 
 * conditional UP/DOWN processing.  
 */
public class SpareIteration
{
   /** The frame or widget. */
   private final Widget<?> widget;
   
   /** The current values of widgets. */
   private final ScreenBuffer frameBuf;
   
   /** Array of field level widget IDs. */
   private final int[] widgetIds;
   
   /** DISPLAY/VIEW flag. */
   private final boolean isView;

   /**
    * Constructor.
    *
    * @param   widget
    *          The frame or widget.
    * @param   frameBuf
    *          The current values of widgets.
    * @param   widgetIds
    *          Array of field level widget IDs.
    * @param   isView
    *          <code>true</code> if parameters are for VIEW,
    *          <code>false</code> for DISPLAY.
    */
   public SpareIteration(Widget<?> widget, ScreenBuffer frameBuf, int[] widgetIds, boolean isView)
   {
      this.widget = widget;
      this.frameBuf = frameBuf;
      this.widgetIds = widgetIds;
      this.isView = isView;
   }

   /**
    * Display saved data.
    */
   public void view()
   {
      ThinClient.view(widget, new ScreenBuffer[] {frameBuf}, widgetIds, isView);
   }
}