/*
** Module   : ScrollBarController.java
** Abstract : Abstract scroll bar controller.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SVL 20151101 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.ui.client.gui;

import com.goldencode.p2j.ui.client.*;

/**
 * Scroll bar controller with basic functionality for all GUI scroll bar controllers.
 */
public abstract class ScrollBarControllerGuiImpl
implements ScrollBarController
{
   /** Scroll bar controlled by this controller. */
   protected ScrollBarGuiImpl scrollBar = null;

   /** If scroll bar thumb is being dragged. */
   protected boolean drag = false;

   /**
    * Current position of the dragged thumb (distance between left/top button and thumb's
    * left/top edge).
    */
   protected int dragPos = 0;

   /**
    * Constructor.
    */
   public ScrollBarControllerGuiImpl()
   {
   }

   /**
    * Set scroll bar controlled by this controller..
    *
    * @param scrollBar
    *        Scroll bar controlled by this controller.
    */
   public void setScrollBar(ScrollBarGuiImpl scrollBar)
   {
      this.scrollBar = scrollBar;
      scrollBar.setController(this);
   }

   /**
    * Notify controller that thumb drag has started.
    */
   @Override
   public void startThumbDrag()
   {
      drag = true;
   }

   /**
    * Notify controller that thumb drag has ended.
    */
   @Override
   public void stopThumbDrag()
   {
      drag = false;
   }

   /**
    * Notify controller that thumb has been dragged.
    *
    * @param thumbPos
    *        New thumb position for the scroll bar (distance between left/top button and thumb's
    *        left/top edge).
    */
   @Override
   public void thumbMoved(int thumbPos)
   {
      if (scrollBar == null)
      {
         return;
      }

      dragPos = thumbPos;
      dragPos = Math.max(0, dragPos);
      dragPos = Math.min(dragPos, scrollBar.getTravelSize() - getThumbSize());
   }

   /**
    * Get enable state for the scroll bar.
    *
    * @return <code>true</code> if scroll bar is enabled.
    */
   @Override
   public boolean isEnabled()
   {
      return getMax() > 0;
   }
}