/*
** Module   : ThreeDBorderGuiImpl.java
** Abstract : 3D border GUI implementation.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 HC  20151112 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.border.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * 3D border GUI implementation.
 */
public class ThreeDBorderGuiImpl
implements Border<GuiOutputManager> 
{
   /** Insets. **/
   private Insets insets = null;
   
   /** Native insets. */
   private NativeInsets ninsets = null;
   
   /** Cached driver. */
   private GuiDriver<?, ?> gd;
   
   /** Coordinate conversion reference. */
   private CoordinatesConversion c;
   
   /**
    * Default constructor.
    */
   public ThreeDBorderGuiImpl()
   {
      GuiOutputManager o = (GuiOutputManager) OutputManager.instance();
      gd = (GuiDriver<?, ?>) o.getInstanceDriver();
      c = o.coordinates();
      
      Point p = c.baseUnits();
      insets = new Insets(p.y * 2, p.x * 2, p.y * 2, p.x * 2);
      
      this.insets = new Insets(insets.top, insets.left, insets.bottom, insets.right);
      this.ninsets = new NativeInsets(c.rowToNative(insets.top), 
                                      c.columnToNative(insets.left), 
                                      c.rowToNative(insets.bottom), 
                                      c.columnToNative(insets.right));
   }
   
   /**
    * Get insets introduced by the border.
    * 
    * @return border insets.
    */
   @Override
   public Insets insets()
   {
      return new Insets(insets.top, insets.left, insets.bottom, insets.right);
   }

   /**
    * Draw border.
    * 
    * @param   widget
    *          Base widget. 
    * @param   size
    *          Border dimensions.
    */
   @Override
   public void drawBorder(Widget<GuiOutputManager> widget, Dimension size)
   {
      NativeDimension nsize = c.dimensionToNative(size);
      
      gd.setColor(gd.getSysColor(SysColor.COLOR_3DSHADOW));
      gd.drawLine(0, 0, 0, nsize.height - 2);
      gd.drawLine(0, 0, nsize.width - 2, 0);
      
      gd.setColor(gd.getSysColor(SysColor.COLOR_3DDKSHADOW));
      gd.drawLine(1, 1, 1, nsize.height - 3);
      gd.drawLine(1, 1, nsize.width - 3, 1);
      
      gd.setColor(gd.getSysColor(SysColor.COLOR_3DLIGHT));
      gd.drawLine(1, nsize.height - 2, nsize.width - 2, nsize.height - 2);
      gd.drawLine(nsize.width - 2, nsize.height - 2, nsize.width - 2, 1);

      gd.setColor(gd.getSysColor(SysColor.COLOR_WINDOW));
      gd.drawLine(0, nsize.height - 1, nsize.width - 1, nsize.height - 1);
      gd.drawLine(nsize.width - 1, nsize.height - 1, nsize.width - 1, 0);
      
      // be paranoid, restore the line stroke
      gd.setLineStroke(LineStroke.DEFAULT);
   }
}