/*
** Module   : USAKeyboardLayout.java
** Abstract : USA Keyboard layout class for Windows.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------Description----------------------------
** 001 VIG 20150814 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui;

import java.util.*;

/**
 * USA Keyboard layout class for Windows. Keyboard layout is the current layout set in Windows.
 */
public class USAKeyboardLayout
extends KeyboardLayout
{
   /** key names from layout by Progress key codes */
   private Map<Integer, String> keyNames = new HashMap<>();
   
   /**
    * Performs initialization.  
    */
   @Override
   public void init()
   {
      keyNames.put(8, "BkSp");
      keyNames.put(9, "Tab");
      keyNames.put(13, "Enter");
      keyNames.put(27, "Esc");
      
      keyNames.put(127, "Del");
      
      keyNames.put(501, "Up");
      keyNames.put(502, "Down");
      keyNames.put(503, "Right");
      keyNames.put(504, "Left");
      keyNames.put(505, "Home");
      keyNames.put(506, "End");
      keyNames.put(507, "PgUp");
      keyNames.put(508, "PgDn");
      
      keyNames.put(510, "Ins");

      keyNames.put(512, "Shift");
      keyNames.put(1024, "Alt");
      keyNames.put(2048, "Ctrl");
      
      // Remaining key names are not different than Progress key labels:
      // , - . / 0-9 = A-Z [ \ ] ` F1-F12
   }
   
   /**
    * Gets keyboard key name according to this layout
    * 
    * @param   keyCode
    *          Progress key code.
    *          
    * @return  Name of a key button or null if the key name is the same as Progress key label or
    *          specified code is incorrect (not a single key code or invalid value).
    */
   @Override
   public String getKeyName(int keyCode)
   {
      if (keyNames.containsKey(keyCode))
      {
         return keyNames.get(keyCode);
      }
      
      return null;
   }
}