/*
** Module   : SystemAction.java
** Abstract : Implementation of a system action.
**
** Copyright (c) 2016-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SBI 20160523 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui.driver;

// this usage of AWT events is intentional, they are being used separately from the AWT/Swing
// event model; no other AWT/Swing usage is occurring in this class
import java.awt.event.*;



/**
 * Listener for mouse system actions: copy, cut, paste, minimize. The action must be registered
 * MouseHandler.registerWidgetAction(menuId, new SystemAction(menuId, MouseEvt.PASTE, null));
 * to aware the web client about the widget that triggers this action if a mouse click occurs.
 * It is used to intercept the required copy, cut, paste and minimize actions to be handled by
 * the web client first. Also it can be used to split logic processing required for the Swing
 * client and the Web client. For an example, it can be required to process an action on the java
 * side for the Swing client only and to process an action on the Web client only.
 */
public class SystemAction
extends MouseWidgetAction
{
   /** All mouse events are forwarded to this mouse adapter */
   private final MouseAdapter delegate;
   
   /**
    * Create a system action for the specified widget.
    * 
    * @param    wid
    *           The widget id.
    * @param    eventId
    *           The event ID.
    * @param    delegate
    *           The mouse events handler.
    */
   public SystemAction(int wid, int eventId, MouseAdapter delegate)
   {
      super(wid, eventId);
      
      this.delegate = delegate;
   }

   /**
    * Notification of a mouse clicked event for this {@link MouseWidgetAction#widget}.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mouseClicked(MouseEvent e)
   {
      if (delegate != null)
      {
         delegate.mouseClicked(e);
      }
   }
}