/*
** Module   : CommonFile.java
** Abstract : Defines common APIs used by the FILE-INFO and procedure handles.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.

**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 CA  20130112 Created initial version.
** 002 EVL 20130911 Adding legacy attributes and methods annotation.
** 003 GES 20141124 Javadoc fix.
** 004 EVL 20160224 Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
** 005 SVL 20170126 Added setters for FILE-INFO.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Exposes the following methods or attributes, common for the FILE-INFO, RCODE-INFO and
 * PROCEDURE handles:
 * <table summary="">
 * <tr>
 *    <th>attribute or method</th>
 *    <th>API getter</th>
 *    <th>API setter</th>
 * </tr>
 * <tr>
 *    <td>FILE-NAME</td>
 *    <td>{@link #getFileName()}</td>
 *    <td>{@link #initFileInfo(character)}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapFile} API, which, depending on the 
 * handle's state and type, it either returns an 
 * {@link FileSystemOps#asHandle()} instance (for FILE-INFO handles),
 * {@link RcodeInfoOps#asHandle()} instance (for RCODE-INFO handles) or an
 * {@link ExternalProgramWrapper} instance (for PROCEDURE handles), or a 
 * dynamic proxy using {@link handle#invalidAttrAccessProxy}. 
 */
public interface CommonFile
{
   /**
    * Get the <code>file-name</code> attribute of this handle.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "FILE-NAME")
   public character getFileName();

   /**
    * Search for the specified file and initialize the related data
    * structures if found.  Relative filenames are searched for via the
    * PROPATH.  If <code>name</code> is <code>unknown</code> or not found,
    * the current data structures are cleared.
    *
    * @param    name
    *           Relative or absolute filename to obtain information on.
    */
   @LegacyAttribute(name = "FILE-NAME", setter = true)
   public void initFileInfo(character name);

   /**
    * Search for the specified file and initialize the related data
    * structures if found.  Relative filenames are searched for via the
    * {@link EnvironmentOps#getSearchPath}.  If <code>name</code> is
    * <code>unknown</code> or not found, the current data structures are
    * cleared.
    *
    * @param    name
    *           Relative or absolute filename to obtain information on.
    */
   @LegacyAttribute(name = "FILE-NAME", setter = true)
   public void initFileInfo(String name);
}