/*
** Module   : FileInfo.java
** Abstract : a container to store/transfer file attributes
**
** Copyright (c) 2006-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 GES 20060211 NEW   @24537 Created initial version, supporting the
**                               file name and size only.
** 002 GES 20080123 CHG   @36901 Must implement serializable to be sent over
**                               the network.
** 003 GES 20080321 CHG   @37629 Added all other missing data.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

import java.io.*;

/**
 * Provides a container to store/transfer file attributes as defined by the
 * Progress FILE-INFO system handle.
 * <p>
 * This class makes it easy to obtain information about a file on one system
 * and then transfer that information to another system.
 */
public class FileInfo
implements Serializable
{
   /** File name as specified on input. */
   public String name = null;
   
   /**
    * Absolute path if the input file name was specified in absolute terms or
    * the relative path name if the file was found via the PROPATH.
    */
   public String pathname = null;
   
   /** Canonicalized absolute path name. */
   public String fullname = null;
   
   /** File size in bytes. */
   public long size = -1;
   
   /** File type codes. */
   public String type = null;
   
   /** Last modification date and time. */
   public long modification = -1;
   
   /** Creation date and time. */
   public long creation = -1;
}