/*
** Module   : LineContent.java
** Abstract : Progress 4GL INI file content simple line to ignore class
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.

**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130701 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

import java.io.*;

/**
 * A class implementing the INI text file comment line or empty line. In both cases this is the
 * single line to ignore. The only we need to know is where it is located inside INI file. Can be
 * the part of the section or be the header of the file not belonging to any section.
 *
 * @author    EVL
 */
public class LineContent
extends Content
{
   /** The text associated with the line content. */
   private String text = null;
   
   /**
    * Constructs an instance of the ignored line in INI file. Just calls case constructor.
    *
    * @param text 
    *        The text corresponding to the whole line of the comment or empty space line.
    */
   public LineContent(String text)
   {
      super();
      this.text = text;
   }
   
   /**
    * Writes the content to the file via Writer object provided.
    * 
    * @param pw 
    *        The writer will be used to save content to the file.
    */
   public void write(PrintWriter pw)
   {
      pw.println(text);
   }
   
   /**
    * Checks if the content is empty or not.
    * 
    * @return <code>true</code> in case of not emptry, <code>false</code> otherwise.
    */
   public boolean notEmpty()
   {
      return text != null;
   }
}