/*
** Module   : RepeatedOutput.java
** Abstract : provides a callback mechanism to output repeatedly
**
** Copyright (c) 2006-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 GES 20060225 NEW   @24742 Provides a callback mechanism to generate
**                               common output repeatedly (like a header or
**                               footer for a report).
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Provides an output callback mechanism which allows arbitrary user defined
 * stream output to be generated at the top or bottom of a page in a paged
 * output stream.  This enables page headers and footers.
 * <p>
 * Please note that the only requirement for this output is that the number
 * of lines which it consumes MUST be constant across all invocations of the
 * {@link #render} method.  This is necessary to properly determine when a
 * header/footer needs to be emitted.  The output within the given number of
 * lines may vary (e.g. page numbers may increment).
 */
public interface RepeatedOutput
{
   /**
    * Returns the height of the output as the number of lines that will be
    * generated when {@link #render} is called. 
    *
    * @return   The height of the generated output in lines.
    */
   public int height();
   
   /**
    * Generate output and write it to the given stream.
    *
    * @param    out
    *           A stream that is opened for output.
    */
   public void render(Stream out);
}