/*
** Module   : comhandle.java
** Abstract : Progress 4GL compatible COM handle object
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- -------------------Description--------------------
** 001 GES 20101005          Placeholder version with some stubs for
**                           attributes and methods.
** 002 SIY 20101014          Changed signatures.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

import java.io.*;

/**
 * A class that represents a Progress 4GL compatible COM handle data
 * type. It provides a mechanism to group the functionality that is
 * specific to a COM handle in Progress.
 */
public class comhandle
extends handle
{
   /**
    * Default constructor, creates an instance that represents the unknown
    * value.
    */
   public comhandle()
   {
      super();
   }
   
   /**
    * Constructs an instance that has the contained object reference and
    * <code>unknown value</code> that exactly matches that of the passed-in 
    * instance.
    *
    * @param    value
    *           The instance to pattern from.
    */
   public comhandle(comhandle value)
   {
      super(value);
   }
   
   /**
    * Constructs an instance that wraps the given object reference as a
    * {@link WrappedResource}.
    *
    * @param    value
    *           The instance to contain.
    */
   public comhandle(Object value)
   {
      super();
      assign(new TransparentWrapper(value), false);
   }
   
   /**
    * Creates a new instance of the same type that represents the 
    * <code>unknown value</code>.
    *
    * @return   An instance that represents the <code>unknown value</code>.
    */
   public BaseDataType instantiateUnknown()   
   {
      return new comhandle();
   }
   
   /**
    * Does the same as standard <code>clone()</code> method but returns an
    * instance of <code>BaseDataType</code> and doesn't throw the
    * <code>CloneNotSupportedException</code>.
    *
    * @return   A clone of this instance.  
    */
   public BaseDataType duplicate()
   {
      return new comhandle(this);
   }
   
   /**
    * Loads the named OCX into the associated control frame.
    * <p>
    * This is a non-functional stub at this time.
    *
    * @param    ocx
    *           Name of the OCX to load.
    * @param    ctrl
    *           Control frame name.
    *            
    * @return   <code>true</code> if operation was successful.
    */
   public boolean loadControls(character ocx, character ctrl)
   {
      return false;
   }
   
   /**
    * Loads the named OCX into the associated control frame.
    * <p>
    * This is a non-functional stub at this time.
    *
    * @param    ocx
    *           Name of the OCX to load.
    * @param    ctrl
    *           Control frame name.
    *            
    * @return   <code>true</code> if operation was successful.
    */
   public boolean loadControls(String ocx, character ctrl)
   {
      return false;
   }
   
   /**
    * Loads the named OCX into the associated control frame.
    * <p>
    * This is a non-functional stub at this time.
    *
    * @param    ocx
    *           Name of the OCX to load.
    * @param    ctrl
    *           Control frame name. 
    *            
    * @return   <code>true</code> if operation was successful.
    */
   public boolean loadControls(character ocx, String ctrl)
   {
      return false;
   }
   
   /**
    * Loads the named OCX into the associated control frame.
    * <p>
    * This is a non-functional stub at this time.
    *
    * @param    ocx
    *           Name of the OCX to load.
    * @param    ctrl
    *           Control frame name. 
    *            
    * @return   <code>true</code> if operation was successful.
    */
   public boolean loadControls(String ocx, String ctrl)
   {
      return false;
   }
   
   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      throw new NotSerializableException("comhandle");
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      throw new NotSerializableException("comhandle");
   }
}