/*
** Module   : AbstractHandlerCommons.java
** Abstract : an abstract class used to load modules
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.

**
** -#- -I- --Date-- -------------------------------Description-----------------------------------
** 001 MAG 20140708 First version. Code moved from DojoToolkitHandler.
** 002 GES 20150308 Moved to a more generic package location.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.web;

import java.io.*;
import java.net.*;

import javax.servlet.http.*;

import org.eclipse.jetty.server.handler.*;

/**
 * This abstract class contains the common part of handlers which load javascript modules 
 * or other classpath resources like Dojo toolkit.
 */
public abstract class AbstractHandlerCommons
extends AbstractHandler
{
   /** Buffer size. */
   private static final int BUFFER_SIZE = 4096;
   
   /** Data buffer. */
   private byte[] buffer = new byte[BUFFER_SIZE];

   /**
    * Build HTTP response.
    * 
    * @param   response 
    *          The object or a wrapper of the response. 
    * @param   conn
    *          Module resource.
    * @param   module
    *          Module name.
    *          
    * @throws  IOException
    *          On I/O errors.
    */
   protected void sendResponse(HttpServletResponse response, URLConnection conn, String module) 
   throws IOException
   {
      // set response headers and status
      response.setStatus(HttpServletResponse.SC_OK);
      response.setContentType(detectContentType(module));
      response.setContentLength(conn.getContentLength());
      
      // build response body
      InputStream is = conn.getInputStream();
      OutputStream os = response.getOutputStream();
      
      int bytesRead = is.read(buffer);            
      while (bytesRead > 0)
      {
         os.write(buffer, 0, bytesRead);
         bytesRead = is.read(buffer);            
      }

      // cleanup
      os.flush();
      is.close();               
   }

   /**
    * Detect content type based on module file extension.
    * 
    * @param   module
    *          Module name.
    *          
    * @return  Content type (MIME).
    */
   protected String detectContentType(String module)
   {
      String contentType = "application/octet-stream";

      if (module.endsWith(".js"))
      {
         contentType = "application/javascript";
      }
      else if (module.endsWith(".css"))
      {
         contentType = "text/css";
      }
      else if (module.endsWith(".gif"))
      {
         contentType = "image/gif";
      }
      else if (module.endsWith(".png"))
      {
         contentType = "image/png";
      }
      else if (module.endsWith(".jpg") || module.endsWith(".jpeg"))
      {
         contentType = "image/jpeg";
      }
      else if (module.endsWith(".html") || module.endsWith(".htm"))
      {
         contentType = "txt/html";         
      }      
      
      return contentType;
   }
}