/*
** Module   : SaxAttributes
** Abstract : SAX attributes specific interface.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130208 Created initial version.
** 002 CA  20130221 Added ADM-DATA and UNIQUE-ID support. 
** 003 CA  20130927 Added LegacyResource annotation, to determine resource type.
** 004 EVL 20131008 Adding legacy attributes and methods annotation.
** 005 EVK 20131130 Renamed parameter in method copySaxAttributes, removed inheritance to the
**                  UniqueID and ADMData interfaces.
** 006 GES 20141202 Moved getNumItems() to the NumItems super-interface.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Interface describing SAX-Attributes specific features of the SAX objects.
 */
@LegacyResource(resource = LegacyResource.SAX_ATTRIBUTES)
public interface SaxAttributes
extends SaxEntity,
        NumItems
{
   /**
    * Gets the value of the atribute with given index.
    * 
    * @param    index
    *           The 1-based index of the attribute to get.
    * 
    * @return   The character value of the attribute with given number.
    */
   @LegacyMethod(name = "GET-VALUE-BY-INDEX")
   public character getValueByIndex(double index);

   /**
    * Gets the value of the atribute with given index.
    * 
    * @param    index
    *           The 1-based index of the attribute to get.
    * 
    * @return   The character value of the attribute with given number.
    */
   @LegacyMethod(name = "GET-VALUE-BY-INDEX")
   public character getValueByIndex(NumberType index);

   /**
    * Copies SAX attributes form source provided to the current SAX-Attributes object.
    * 
    * @param    attrHandle
    *           The handle of the SAX-Attribute object to be used as source.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "COPY-SAX-ATTRIBUTES")
   public logical copySaxAttributes(handle attrHandle);
   
   /**
    * Gets the 1-based index of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The integer value of the given attribute index.
    */
   @LegacyMethod(name = "GET-INDEX-BY-NAMESPACE-NAME")
   public integer getIndexByNamespaceName(Text uri, Text localname);
   
   /**
    * Gets the 1-based index of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The integer value of the given attribute index.
    */
   @LegacyMethod(name = "GET-INDEX-BY-NAMESPACE-NAME")
   public integer getIndexByNamespaceName(String uri, Text localname);
   
   /**
    * Gets the 1-based index of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The integer value of the given attribute index.
    */
   @LegacyMethod(name = "GET-INDEX-BY-NAMESPACE-NAME")
   public integer getIndexByNamespaceName(Text uri, String localname);
   
   /**
    * Gets the 1-based index of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The integer value of the given attribute index.
    */
   @LegacyMethod(name = "GET-INDEX-BY-NAMESPACE-NAME")
   public integer getIndexByNamespaceName(String uri, String localname);

   /**
    * Gets the 1-based index of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the index.
    * 
    * @return   The integer value of the given attribute index.
    */
   @LegacyMethod(name = "GET-INDEX-BY-QNAME")
   public integer getIndexByQname(Text qname);
   
   /**
    * Gets the 1-based index of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the index.
    * 
    * @return   The integer value of the given attribute index.
    */
   @LegacyMethod(name = "GET-INDEX-BY-QNAME")
   public integer getIndexByQname(String qname);

   /**
    * Gets the unqualified local name of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute name to get.
    * 
    * @return   The character value of the attribute local name with given number.
    */
   @LegacyMethod(name = "GET-LOCALNAME-BY-INDEX")
   public character getLocalnameByIndex(double index);

   /**
    * Gets the unqualified local name of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute name to get.
    * 
    * @return   The character value of the attribute local name with given number.
    */
   @LegacyMethod(name = "GET-LOCALNAME-BY-INDEX")
   public character getLocalnameByIndex(NumberType index);

   /**
    * Gets the qualified name of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute name to get.
    * 
    * @return   The character value of the attribute qualified name with given number.
    */
   @LegacyMethod(name = "GET-QNAME-BY-INDEX")
   public character getQnameByIndex(double index);

   /**
    * Gets the qualified local name of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute name to get.
    * 
    * @return   The character value of the attribute qualified name with given number.
    */
   @LegacyMethod(name = "GET-QNAME-BY-INDEX")
   public character getQnameByIndex(NumberType index);

   /**
    * Gets the type of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute type to get.
    * 
    * @return   The character value of the attribute type with given number.
    */
   @LegacyMethod(name = "GET-TYPE-BY-INDEX")
   public character getTypeByIndex(double index);

   /**
    * Gets the type of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the attribute type to get.
    * 
    * @return   The character value of the attribute type with given number.
    */
   @LegacyMethod(name = "GET-TYPE-BY-INDEX")
   public character getTypeByIndex(NumberType index);

   /**
    * Gets the type of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute type.
    */
   @LegacyMethod(name = "GET-TYPE-BY-NAMESPACE-NAME")
   public character getTypeByNamespaceName(Text uri, Text localname);
   
   /**
    * Gets the type of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute type.
    */
   @LegacyMethod(name = "GET-TYPE-BY-NAMESPACE-NAME")
   public character getTypeByNamespaceName(String uri, Text localname);
   
   /**
    * Gets the type of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute type.
    */
   @LegacyMethod(name = "GET-TYPE-BY-NAMESPACE-NAME")
   public character getTypeByNamespaceName(Text uri, String localname);
   
   /**
    * Gets the type of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute type.
    */
   @LegacyMethod(name = "GET-TYPE-BY-NAMESPACE-NAME")
   public character getTypeByNamespaceName(String uri, String localname);

   /**
    * Gets the type of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the type.
    * 
    * @return   The character value of the given attribute type.
    */
   @LegacyMethod(name = "GET-TYPE-BY-QNAME")
   public character getTypeByQname(character qname);
   
   /**
    * Gets the type of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the type.
    * 
    * @return   The character value of the given attribute type.
    */
   @LegacyMethod(name = "GET-TYPE-BY-QNAME")
   public character getTypeByQname(String qname);

   /**
    * Gets the namespace URI of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the namespace URI to get.
    * 
    * @return   The character value of the namespace URI with given number.
    */
   @LegacyMethod(name = "GET-URI-BY-INDEX")
   public character getUriByIndex(double index);

   /**
    * Gets the namespace URI of the current attribute object by the given 1-based index.
    * 
    * @param    index
    *           The 1-based index of the namespace URI to get.
    * 
    * @return   The character value of the namespace URI with given number.
    */
   @LegacyMethod(name = "GET-URI-BY-INDEX")
   public character getUriByIndex(NumberType index);

   /**
    * Gets the value of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute.
    */
   @LegacyMethod(name = "GET-VALUE-BY-NAMESPACE-NAME")
   public character getValueByNamespaceName(Text uri, Text localname);
   
   /**
    * Gets the value of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute.
    */
   @LegacyMethod(name = "GET-VALUE-BY-NAMESPACE-NAME")
   public character getValueByNamespaceName(String uri, Text localname);
   
   /**
    * Gets the value of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute.
    */
   @LegacyMethod(name = "GET-VALUE-BY-NAMESPACE-NAME")
   public character getValueByNamespaceName(Text uri, String localname);
   
   /**
    * Gets the value of the attribute with given namespace name.
    * 
    * @param    uri
    *           The namespace URI or empty string if none.
    * @param    localname
    *           The local(unqualified) name of the attribute.
    * 
    * @return   The character value of the given attribute.
    */
   @LegacyMethod(name = "GET-VALUE-BY-NAMESPACE-NAME")
   public character getValueByNamespaceName(String uri, String localname);

   /**
    * Gets the value of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the value.
    * 
    * @return   The character value of the given attribute.
    */
   @LegacyMethod(name = "GET-VALUE-BY-QNAME")
   public character getValueByQname(character qname);
   
   /**
    * Gets the value of the attribute with given XML qualified name.
    * 
    * @param    qname
    *           The XML qualified name of the attribute to get the value.
    * 
    * @return   The character value of the given attribute.
    */
   @LegacyMethod(name = "GET-VALUE-BY-QNAME")
   public character getValueByQname(String qname);
   
   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(character name, character value);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(character name, String value);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(character name, character value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(character name, String value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(character name, character value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(character name, String value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(String name, character value);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(String name, String value);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(String name, character value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(String name, String value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(String name, character value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    name
    *           The name of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(String name, String value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(NumberType index, character value);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(NumberType index, String value);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(NumberType index, character value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(NumberType index, String value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(NumberType index, character value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(NumberType index, String value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(double index, character value);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(double index, String value);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(double index, character value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(double index, String value, character uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(double index, character value, String uri);

   /**
    * Updates the specified attribute.
    * 
    * @param    index
    *           The 1-based position index of the attribute to update.
    * @param    value
    *           The new value of the updated attribute.
    * @param    uri
    *           The URI of the attribute to update.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @LegacyMethod(name = "UPDATE-ATTRIBUTE")
   public logical updateAttribute(double index, String value, String uri);
}