/*
** Module   : XmlSchema.java
** Abstract : Define getter and setter for SCHEMA-LOCATION attribute
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVK 20130508 Created initial version.
** 002 EVL 20130930 Adding legacy attributes and methods annotation.
** 003 EVL 20131009 Moved annotation for SCHEMA-LOCATION and SCHEMA-PATH here instead of upper
**                  interface
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Defines the SCHEMA-LOCATION and SCHEMA-PATH attribute.
 */
public interface XmlSchema
{
   /**
    * Determines the list of namespace/location pairs of an XML Schema file to validate
    * against.
    *
    * @param    schemaLocation
    *           Determines the list of namespace/location pairs of an XML Schema file to
    *           validate against.
    */
   @LegacyAttribute(name = "SCHEMA-LOCATION", setter = true)
   public void setSchemaLocation(character schemaLocation);

   /**
    * Determines the list of namespace/location pairs of an XML Schema file to validate
    * against.
    *
    * @return   schema location, default value is "".
    */
   @LegacyAttribute(name = "SCHEMA-LOCATION")
   public character getSchemaLocation();

   /**
    * A delimiter-separated list of directory paths used to locate the XML Document Type
    * Definition (DTD) associated with a particular XML document. It is searched if the XML
    * document contains a relative path to the DTD.
    *
    * @param    schemaPath
    *           To separate directory paths, use a comma, or a delimiter character that your
    *           operating system recognizes.
    */
   @LegacyAttribute(name = "SCHEMA-PATH", setter = true)
   public void setSchemaPath(character schemaPath);

   /**
    * A delimiter-separated list of directory paths used to locate the XML Document Type
    * Definition (DTD) associated with a particular XML document. It is searched if the XML
    * document contains a relative path to the DTD.
    *
    * @return   schema path, default value is unknown(?).
    */
   @LegacyAttribute(name = "SCHEMA-PATH")
   public character getSchemaPath();

   /**
    * Indicates if the namespace processing is suppressed or not.
    *
    * @return   The logical value representing the current suppress-namespace-processing
    *           attribute.
    */
   @LegacyAttribute(name = "SUPPRESS-NAMESPACE-PROCESSING")
   public logical getSuppressNamespaceProcessing();

   /**
    * Redefines whether the namespace processing is suppressed or not.
    *
    * @param    suppress
    *           The new value the current suppress-namespace-processing attribute.
    */
   @LegacyAttribute(name = "SUPPRESS-NAMESPACE-PROCESSING", setter = true)
   public void setSuppressNamespaceProcessing(logical suppress);

   /**
    * Redefines whether the namespace processing is suppressed or not.
    *
    * @param    suppress
    *           The new value the current suppress-namespace-processing attribute.
    */
   @LegacyAttribute(name = "SUPPRESS-NAMESPACE-PROCESSING", setter = true)
   public void setSuppressNamespaceProcessing(boolean suppress);
}