/*
** Module   : init.c
** Abstract : Initialize calls to prepare native layer to work properly.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20140117 Created initial version to implement init steps.
** 002 GES 20150108 Added functions for operating system pid, ppid and user name querying.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

#include "init.h"

#include <stdlib.h>

#include "com_goldencode_p2j_main_ClientCore.h"

/**
 * Java_com_goldencode_p2j_main_ClientCore_processInit - 
 *                         set up signal handling to ignore job control signals and redefine
 *                         CTRL-C handlers(reflecteds in the child process as well)
 *
 * @param   env   pointer to an object providing access to JNI environment
 *
 * @param   cls   reference to the Java class who called this function
 *                (from static context)
 *
 */ 
JNIEXPORT void JNICALL Java_com_goldencode_p2j_main_ClientCore_processInit(JNIEnv* env,
                                                                           jclass class)
{
   initSignalsProcessing(env);
}

/**
 * Read the operating system pid for the current JVM process.
 *
 * @param   env   pointer to an object providing access to JNI environment
 *
 * @param   cls   reference to the Java class who called this function
 *                (from static context)
 *
 */ 
JNIEXPORT jlong JNICALL Java_com_goldencode_p2j_main_ClientCore_getPid(JNIEnv* env, jclass class)
{
   return (jlong) os_pid();
}

/**
 * Read the operating system user name for the current JVM process.
 *
 * @param   env   pointer to an object providing access to JNI environment
 *
 * @param   cls   reference to the Java class who called this function
 *                (from static context)
 *
 */ 
JNIEXPORT jstring JNICALL Java_com_goldencode_p2j_main_ClientCore_getUserName(JNIEnv* env,
                                                                              jclass  class)
{
   int   len    = 256;
   char* buffer = malloc(len * sizeof(char));
   
   jstring name = NULL;
   
   if (buffer != NULL)
   {
      if (os_username(buffer, len))
      {
         name = (*env)->NewStringUTF(env, buffer);
      }
      
      free(buffer);
   }
   
   return name;
}
